@@ -2,7 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* global content */
+/* global content, APP_SHUTDOWN */
 /* exported startup, shutdown, install, uninstall */
 
 "use strict";
@@ -10,6 +10,13 @@
 const Cu = Components.utils;
 const Ci = Components.interfaces;
 const {Services} = Cu.import("resource://gre/modules/Services.jsm", {});
+const {NetUtil} = Cu.import("resource://gre/modules/NetUtil.jsm", {});
+const {AppConstants} = Cu.import("resource://gre/modules/AppConstants.jsm", {});
+
+// MultiWindowKeyListener instance for Ctrl+Alt+R key
+let listener;
+// nsIURI to the addon root folder
+let resourceURI;
 
 function actionOccurred(id) {
   let {require} = Cu.import("resource://devtools/shared/Loader.jsm", {});
@@ -18,6 +25,105 @@ function actionOccurred(id) {
   telemetry.actionOccurred(id);
 }
 
+// Synchronously fetch the content of a given URL
+function readURI(uri) {
+  let stream = NetUtil.newChannel({
+    uri: NetUtil.newURI(uri, "UTF-8"),
+    loadUsingSystemPrincipal: true}
+  ).open2();
+  let count = stream.available();
+  let data = NetUtil.readInputStreamToString(stream, count, {
+    charset: "UTF-8"
+  });
+
+  stream.close();
+
+  return data;
+}
+
+/**
+ * Interpret the processing instructions contained in a preferences file, based on a
+ * limited set of supported #if statements. After we ship as an addon, we don't want to
+ * introduce anymore processing instructions, so all unrecognized preprocessing
+ * instructions will be treated as an error.
+ *
+ * This function is mostly copied from devtools/client/inspector/webpack/prefs-loader.js
+ *
+ * @param  {String} content
+ *         The string content of a preferences file.
+ * @return {String} the content stripped of preprocessing instructions.
+ */
+function interpretPreprocessingInstructions(content) {
+  const ifMap = {
+    "#if MOZ_UPDATE_CHANNEL == beta": AppConstants.MOZ_UPDATE_CHANNEL === "beta",
+    "#if defined(NIGHTLY_BUILD)": AppConstants.NIGHTLY_BUILD,
+    "#ifdef MOZ_DEV_EDITION": AppConstants.MOZ_DEV_EDITION,
+    "#ifdef RELEASE_OR_BETA": AppConstants.RELEASE_OR_BETA,
+  };
+
+  let lines = content.split("\n");
+  let ignoring = false;
+  let newLines = [];
+  let continuation = false;
+  for (let line of lines) {
+    if (line.startsWith("#if")) {
+      if (!(line in ifMap)) {
+        throw new Error("missing line in ifMap: " + line);
+      }
+      ignoring = !ifMap[line];
+    } else if (line.startsWith("#else")) {
+      ignoring = !ignoring;
+    } else if (line.startsWith("#endif")) {
+      ignoring = false;
+    }
+
+    let isPrefLine = /^ *(sticky_)?pref\("([^"]+)"/.test(line);
+    if (continuation || (!ignoring && isPrefLine)) {
+      newLines.push(line);
+
+      // The call to pref(...); might span more than one line.
+      continuation = !/\);/.test(line);
+    }
+  }
+  return newLines.join("\n");
+}
+
+// Read a preference file and set all of its defined pref as default values
+// (This replicates the behavior of preferences files from mozilla-central)
+function processPrefFile(url) {
+  let content = readURI(url);
+  content = interpretPreprocessingInstructions(content);
+  content.match(/pref\("[^"]+",\s*.+\s*\)/g).forEach(item => {
+    let m = item.match(/pref\("([^"]+)",\s*(.+)\s*\)/);
+    let name = m[1];
+    let val = m[2].trim();
+
+    // Prevent overriding prefs that have been changed by the user
+    if (Services.prefs.prefHasUserValue(name)) {
+      return;
+    }
+    let defaultBranch = Services.prefs.getDefaultBranch("");
+    if ((val.startsWith("\"") && val.endsWith("\"")) ||
+        (val.startsWith("'") && val.endsWith("'"))) {
+      val = val.substr(1, val.length - 2);
+      val = val.replace(/\\"/g, '"');
+      defaultBranch.setCharPref(name, val);
+    } else if (val.match(/[0-9]+/)) {
+      defaultBranch.setIntPref(name, parseInt(val, 10));
+    } else if (val == "true" || val == "false") {
+      defaultBranch.setBoolPref(name, val == "true");
+    } else {
+      console.log("Unable to match preference type for value:", val);
+    }
+  });
+}
+
+function setPrefs() {
+  processPrefFile(resourceURI.spec + "./client/preferences/devtools.js");
+  processPrefFile(resourceURI.spec + "./client/preferences/debugger.js");
+  processPrefFile(resourceURI.spec + "./client/webide/webide-prefs.js");
+}
+
 // Helper to listen to a key on all windows
 function MultiWindowKeyListener({ keyCode, ctrlKey, altKey, callback }) {
   let keyListener = function (event) {
@@ -73,23 +179,60 @@ let getTopLevelWindow = function (window) {
                .getInterface(Ci.nsIDOMWindow);
 };
 
+function unload(reason) {
+  // This frame script is going to be executed in all processes:
+  // parent and child
+  Services.ppmm.loadProcessScript("data:,(" + function (scriptReason) {
+    /* Flush message manager cached frame scripts as well as chrome locales */
+    let obs = Components.classes["@mozilla.org/observer-service;1"]
+                        .getService(Components.interfaces.nsIObserverService);
+    obs.notifyObservers(null, "message-manager-flush-caches");
+
+    /* Also purge cached modules in child processes, we do it a few lines after
+       in the parent process */
+    if (Services.appinfo.processType == Services.appinfo.PROCESS_TYPE_CONTENT) {
+      Services.obs.notifyObservers(null, "devtools-unload", scriptReason);
+    }
+  } + ")(\"" + reason.replace(/"/g, '\\"') + "\")", false);
+
+  // As we can't get a reference to existing Loader.jsm instances, we send them
+  // an observer service notification to unload them.
+  Services.obs.notifyObservers(null, "devtools-unload", reason);
+
+  // Then spawn a brand new Loader.jsm instance and start the main module
+  Cu.unload("resource://devtools/shared/Loader.jsm");
+  // Also unload all resources loaded as jsm, hopefully all of them are going
+  // to be converted into regular modules
+  Cu.unload("resource://devtools/client/shared/browser-loader.js");
+  Cu.unload("resource://devtools/client/framework/ToolboxProcess.jsm");
+  Cu.unload("resource://devtools/shared/apps/Devices.jsm");
+  Cu.unload("resource://devtools/client/scratchpad/scratchpad-manager.jsm");
+  Cu.unload("resource://devtools/shared/Parser.jsm");
+  Cu.unload("resource://devtools/client/shared/DOMHelpers.jsm");
+  Cu.unload("resource://devtools/client/shared/widgets/VariablesView.jsm");
+  Cu.unload("resource://devtools/client/shared/widgets/AbstractTreeItem.jsm");
+  Cu.unload("resource://devtools/shared/deprecated-sync-thenables.js");
+}
+
 function reload(event) {
   // We automatically reload the toolbox if we are on a browser tab
   // with a toolbox already opened
-  let top = getTopLevelWindow(event.view);
-  let isBrowser = top.location.href.includes("/browser.xul");
   let reloadToolbox = false;
-  if (isBrowser && top.gBrowser) {
-    // We do not use any devtools code before the call to Loader.jsm reload as
-    // any attempt to use Loader.jsm to load a module will instanciate a new
-    // Loader.
-    let nbox = top.gBrowser.getNotificationBox();
-    reloadToolbox =
-      top.document.getAnonymousElementByAttribute(nbox, "class",
-        "devtools-toolbox-bottom-iframe") ||
-      top.document.getAnonymousElementByAttribute(nbox, "class",
-        "devtools-toolbox-side-iframe") ||
-      Services.wm.getMostRecentWindow("devtools:toolbox");
+  if (event) {
+    let top = getTopLevelWindow(event.view);
+    let isBrowser = top.location.href.includes("/browser.xul");
+    if (isBrowser && top.gBrowser) {
+      // We do not use any devtools code before the call to Loader.jsm reload as
+      // any attempt to use Loader.jsm to load a module will instanciate a new
+      // Loader.
+      let nbox = top.gBrowser.getNotificationBox();
+      reloadToolbox =
+        top.document.getAnonymousElementByAttribute(nbox, "class",
+          "devtools-toolbox-bottom-iframe") ||
+        top.document.getAnonymousElementByAttribute(nbox, "class",
+          "devtools-toolbox-side-iframe") ||
+        Services.wm.getMostRecentWindow("devtools:toolbox");
+    }
   }
   let browserConsole = Services.wm.getMostRecentWindow("devtools:webconsole");
   let reopenBrowserConsole = false;
@@ -101,29 +244,13 @@ function reload(event) {
   dump("Reload DevTools.  (reload-toolbox:" + reloadToolbox + ")\n");
 
   // Invalidate xul cache in order to see changes made to chrome:// files
-  Services.obs.notifyObservers(null, "startupcache-invalidate", null);
+  Services.obs.notifyObservers(null, "startupcache-invalidate");
 
-  // This frame script is going to be executed in all processes:
-  // parent and child
-  Services.ppmm.loadProcessScript("data:,new " + function () {
-    /* Flush message manager cached frame scripts as well as chrome locales */
-    let obs = Components.classes["@mozilla.org/observer-service;1"]
-                        .getService(Components.interfaces.nsIObserverService);
-    obs.notifyObservers(null, "message-manager-flush-caches", null);
+  unload("reload");
 
-    /* Also purge cached modules in child processes, we do it a few lines after
-       in the parent process */
-    if (Services.appinfo.processType == Services.appinfo.PROCESS_TYPE_CONTENT) {
-      Services.obs.notifyObservers(null, "devtools-unload", "reload");
-    }
-  }, false);
+  // Update the preferences before starting new code
+  setPrefs();
 
-  // As we can't get a reference to existing Loader.jsm instances, we send them
-  // an observer service notification to unload them.
-  Services.obs.notifyObservers(null, "devtools-unload", "reload");
-
-  // Then spawn a brand new Loader.jsm instance and start the main module
-  Cu.unload("resource://devtools/shared/Loader.jsm");
   const {devtools} = Cu.import("resource://devtools/shared/Loader.jsm", {});
   devtools.require("devtools/client/framework/devtools-browser");
 
@@ -166,6 +293,7 @@ function reload(event) {
     setTimeout(() => {
       let { TargetFactory } = devtools.require("devtools/client/framework/target");
       let { gDevTools } = devtools.require("devtools/client/framework/devtools");
+      let top = getTopLevelWindow(event.view);
       let target = TargetFactory.forTab(top.gBrowser.selectedTab);
       gDevTools.showToolbox(target);
     }, 1000);
@@ -175,59 +303,43 @@ function reload(event) {
   // HUDService is going to close it on unload.
   // Instead we have to manually toggle it.
   if (reopenBrowserConsole) {
-    let HUDService = devtools.require("devtools/client/webconsole/hudservice");
+    let {HUDService} = devtools.require("devtools/client/webconsole/hudservice");
     HUDService.toggleBrowserConsole();
   }
 
   actionOccurred("reloadAddonReload");
 }
 
-let prefs = {
-  // Enable dump as some errors are only printed on the stdout
-  "browser.dom.window.dump.enabled": true,
-  // Enable the browser toolbox and various chrome-only features
-  "devtools.chrome.enabled": true,
-  "devtools.debugger.remote-enabled": true,
-  // Disable the prompt to ease usage of the browser toolbox
-  "devtools.debugger.prompt-connection": false,
-};
-let originalPrefValues = {};
-
-let listener;
-function startup() {
+function startup(data) {
   dump("DevTools addon started.\n");
+
+  resourceURI = data.resourceURI;
+
   listener = new MultiWindowKeyListener({
     keyCode: Ci.nsIDOMKeyEvent.DOM_VK_R, ctrlKey: true, altKey: true,
     callback: reload
   });
   listener.start();
 
-  // Toggle development prefs and save original values
-  originalPrefValues = {};
-  for (let name in prefs) {
-    let value = prefs[name];
-    let userValue = Services.prefs.getBoolPref(name);
-    // Only toggle if the pref isn't already set to the right value
-    if (userValue != value) {
-      Services.prefs.setBoolPref(name, value);
-      originalPrefValues[name] = userValue;
-    }
-  }
+  reload();
 }
-function shutdown() {
+function shutdown(data, reason) {
+  // On browser shutdown, do not try to cleanup anything
+  if (reason == APP_SHUTDOWN) {
+    return;
+  }
+
   listener.stop();
   listener = null;
 
-  // Restore preferences that used to be before the addon was installed
-  for (let name in originalPrefValues) {
-    let userValue = Services.prefs.getBoolPref(name);
-    // Only reset the pref if it hasn't changed
-    if (userValue == prefs[name]) {
-      Services.prefs.setBoolPref(name, originalPrefValues[name]);
-    }
-  }
+  unload("disable");
 }
 function install() {
-  actionOccurred("reloadAddonInstalled");
+  try {
+    actionOccurred("reloadAddonInstalled");
+  } catch (e) {
+    // When installing on Firefox builds without devtools, telemetry doesn't
+    // work yet and throws.
+  }
 }
 function uninstall() {}
