@@ -33,19 +33,37 @@
  */
 
 // shared-head.js handles imports, constants, and utility functions
-Services.scriptloader.loadSubScript("chrome://mochitests/content/browser/devtools/client/framework/test/shared-head.js", this);
+Services.scriptloader.loadSubScript(
+  "chrome://mochitests/content/browser/devtools/client/framework/test/shared-head.js",
+  this
+);
 var { Toolbox } = require("devtools/client/framework/toolbox");
-const EXAMPLE_URL = "http://example.com/browser/devtools/client/debugger/new/test/mochitest/examples/";
+const sourceUtils = {
+  isLoaded: source => source.get("loadedState") === "loaded"
+};
+
+const EXAMPLE_URL =
+  "http://example.com/browser/devtools/client/debugger/new/test/mochitest/examples/";
 
 Services.prefs.setBoolPref("devtools.debugger.new-debugger-frontend", true);
-Services.prefs.clearUserPref("devtools.debugger.tabs")
-Services.prefs.clearUserPref("devtools.debugger.pending-selected-location")
 
 registerCleanupFunction(() => {
   Services.prefs.clearUserPref("devtools.debugger.new-debugger-frontend");
   delete window.resumeTest;
 });
 
+function log(msg, data) {
+  info(`${msg} ${!data ? "" : JSON.stringify(data)}`);
+}
+
+function logThreadEvents(dbg, event) {
+  const thread = dbg.toolbox.threadClient;
+
+  thread.addListener(event, function onEvent(eventName, ...args) {
+    info(`Thread event '${eventName}' fired.`);
+  });
+}
+
 // Wait until an action of `type` is dispatched. This is different
 // then `_afterDispatchDone` because it doesn't wait for async actions
 // to be done/errored. Use this if you want to listen for the "start"
@@ -76,9 +94,9 @@ function _afterDispatchDone(store, type) {
       type: "@@service/waitUntil",
       predicate: action => {
         if (action.type === type) {
-          return action.status ?
-            (action.status === "done" || action.status === "error") :
-            true;
+          return action.status
+            ? action.status === "done" || action.status === "error"
+            : true;
         }
       },
       run: (dispatch, getState, action) => {
@@ -102,12 +120,12 @@ function _afterDispatchDone(store, type) {
 function waitForDispatch(dbg, type, eventRepeat = 1) {
   let count = 0;
 
-  return Task.spawn(function* () {
-    info("Waiting for " + type + " to dispatch " + eventRepeat + " time(s)");
+  return Task.spawn(function*() {
+    info(`Waiting for ${type} to dispatch ${eventRepeat} time(s)`);
     while (count < eventRepeat) {
       yield _afterDispatchDone(dbg.store, type);
       count++;
-      info(type + " dispatched " + count + " time(s)");
+      info(`${type} dispatched ${count} time(s)`);
     }
   });
 }
@@ -122,12 +140,12 @@ function waitForDispatch(dbg, type, eventRepeat = 1) {
  * @static
  */
 function waitForThreadEvents(dbg, eventName) {
-  info("Waiting for thread event '" + eventName + "' to fire.");
+  info(`Waiting for thread event '${eventName}' to fire.`);
   const thread = dbg.toolbox.threadClient;
 
   return new Promise(function(resolve, reject) {
     thread.addListener(eventName, function onEvent(eventName, ...args) {
-      info("Thread event '" + eventName + "' fired.");
+      info(`Thread event '${eventName}' fired.`);
       thread.removeListener(eventName, onEvent);
       resolve.apply(resolve, args);
     });
@@ -143,10 +161,16 @@ function waitForThreadEvents(dbg, eventName) {
  * @return {Promise}
  * @static
  */
-function waitForState(dbg, predicate) {
+function waitForState(dbg, predicate, msg) {
   return new Promise(resolve => {
+    info(`Waiting for state change: ${msg || ""}`);
+    if (predicate(dbg.store.getState())) {
+      return resolve();
+    }
+
     const unsubscribe = dbg.store.subscribe(() => {
       if (predicate(dbg.store.getState())) {
+        info(`Finished waiting for state change: ${msg || ""}`);
         unsubscribe();
         resolve();
       }
@@ -168,23 +192,77 @@ function waitForSources(dbg, ...sources) {
     return Promise.resolve();
   }
 
-  info("Waiting on sources: " + sources.join(", "));
+  info(`Waiting on sources: ${sources.join(", ")}`);
   const { selectors: { getSources }, store } = dbg;
-  return Promise.all(sources.map(url => {
-    function sourceExists(state) {
-      return getSources(state).some(s => {
-        return s.get("url").includes(url);
-      });
-    }
+  return Promise.all(
+    sources.map(url => {
+      function sourceExists(state) {
+        return getSources(state).some(s => {
+          return (s.get("url") || "").includes(url);
+        });
+      }
 
-    if (!sourceExists(store.getState())) {
-      return waitForState(dbg, sourceExists);
-    }
-  }));
+      if (!sourceExists(store.getState())) {
+        return waitForState(dbg, sourceExists);
+      }
+    })
+  );
 }
 
-function waitForElement(dbg, selector) {
-  return waitUntil(() => findElementWithSelector(dbg, selector))
+/**
+ * Waits for a source to be loaded.
+ *
+ * @memberof mochitest/waits
+ * @param {Object} dbg
+ * @param {String} source
+ * @return {Promise}
+ * @static
+ */
+function waitForSource(dbg, url) {
+  return waitForState(dbg, state => {
+    const sources = dbg.selectors.getSources(state);
+    return sources.find(s => (s.get("url") || "").includes(url));
+  });
+}
+
+async function waitForElement(dbg, selector) {
+  await waitUntil(() => findElementWithSelector(dbg, selector));
+  return findElementWithSelector(dbg, selector);
+}
+
+function waitForSelectedSource(dbg, url) {
+  return waitForState(
+    dbg,
+    state => {
+      const source = dbg.selectors.getSelectedSource(state);
+      const isLoaded = source && sourceUtils.isLoaded(source);
+      if (!isLoaded) {
+        return false;
+      }
+
+      if (!url) {
+        return true;
+      }
+
+      const newSource = findSource(dbg, url, { silent: true });
+      if (newSource.id != source.get("id")) {
+        return false;
+      }
+
+      // wait for async work to be done
+      return dbg.selectors.hasSymbols(state, source.toJS());
+    },
+    "selected source"
+  );
+}
+
+/**
+ * Assert that the debugger is not currently paused.
+ * @memberof mochitest/asserts
+ * @static
+ */
+function assertNotPaused(dbg) {
+  ok(!isPaused(dbg), "client is not paused");
 }
 
 /**
@@ -196,21 +274,50 @@ function waitForElement(dbg, selector) {
  * @param {Number} line
  * @static
  */
-function assertPausedLocation(dbg, source, line) {
+function assertPausedLocation(dbg) {
   const { selectors: { getSelectedSource, getPause }, getState } = dbg;
-  source = findSource(dbg, source);
 
-  // Check the selected source
-  is(getSelectedSource(getState()).get("id"), source.id);
+  ok(isTopFrameSelected(dbg, getState()), "top frame's source is selected");
 
   // Check the pause location
-  const location = getPause(getState()).getIn(["frame", "location"]);
-  is(location.get("sourceId"), source.id);
-  is(location.get("line"), line);
+  const pause = getPause(getState());
+  const pauseLine = pause && pause.frame && pause.frame.location.line;
+  assertDebugLine(dbg, pauseLine);
 
+  ok(isVisibleInEditor(dbg, getCM(dbg).display.gutters), "gutter is visible");
+}
+
+function assertDebugLine(dbg, line) {
   // Check the debug line
-  ok(dbg.win.cm.lineInfo(line - 1).wrapClass.includes("debug-line"),
-     "Line is highlighted as paused");
+  const lineInfo = getCM(dbg).lineInfo(line - 1);
+  const source = dbg.selectors.getSelectedSource(dbg.getState());
+  if (source && source.get("loadedState") == "loading") {
+    const url = source.get("url");
+    ok(
+      false,
+      `Looks like the source ${url} is still loading. Try adding waitForLoadedSource in the test.`
+    );
+    return;
+  }
+
+  ok(
+    lineInfo.wrapClass.includes("new-debug-line"),
+    "Line is highlighted as paused"
+  );
+
+  const debugLine = findElementWithSelector(dbg, ".new-debug-line")
+                    || findElementWithSelector(dbg, ".new-debug-line-error");
+
+  ok(isVisibleInEditor(dbg, debugLine), "debug line is visible");
+
+  const markedSpans = lineInfo.handle.markedSpans;
+  if (markedSpans && markedSpans.length > 0) {
+    const marker = markedSpans[0].marker;
+    ok(
+      marker.className.includes("debug-expression"),
+      "expression is highlighted as paused"
+    );
+  }
 }
 
 /**
@@ -232,10 +339,13 @@ function assertHighlightLocation(dbg, source, line) {
   // Check the highlight line
   const lineEl = findElement(dbg, "highlightLine");
   ok(lineEl, "Line is highlighted");
-  // ok(isVisibleWithin(findElement(dbg, "codeMirror"), lineEl),
-  //    "Highlighted line is visible");
-  ok(dbg.win.cm.lineInfo(line - 1).wrapClass.includes("highlight-line"),
-     "Line is highlighted");
+  ok(isVisibleInEditor(dbg, lineEl), "Highlighted line is visible");
+  ok(
+    getCM(dbg)
+      .lineInfo(line - 1)
+      .wrapClass.includes("highlight-line"),
+    "Line is highlighted"
+  );
 }
 
 /**
@@ -250,6 +360,14 @@ function isPaused(dbg) {
   return !!getPause(getState());
 }
 
+async function waitForLoadedObjects(dbg) {
+  const { hasLoadingObjects } = dbg.selectors;
+  return waitForState(
+    dbg,
+    state => !hasLoadingObjects(state),
+    "loaded objects"
+  );
+}
 /**
  * Waits for the debugger to be fully paused.
  *
@@ -257,55 +375,122 @@ function isPaused(dbg) {
  * @param {Object} dbg
  * @static
  */
-function waitForPaused(dbg) {
-  return Task.spawn(function* () {
-    // We want to make sure that we get both a real paused event and
-    // that the state is fully populated. The client may do some more
-    // work (call other client methods) before populating the state.
-    yield waitForThreadEvents(dbg, "paused"),
-    yield waitForState(dbg, state => {
-      const pause = dbg.selectors.getPause(state);
-      // Make sure we have the paused state.
-      if (!pause) {
-        return false;
-      }
-      // Make sure the source text is completely loaded for the
-      // source we are paused in.
-      const sourceId = pause.getIn(["frame", "location", "sourceId"]);
-      const sourceText = dbg.selectors.getSourceText(dbg.getState(), sourceId);
-      return sourceText && !sourceText.get("loading");
-    });
-  });
+async function waitForPaused(dbg) {
+  const { getSelectedScope, hasLoadingObjects } = dbg.selectors;
+
+  return waitForState(
+    dbg,
+    state => {
+      const paused = isPaused(dbg);
+      const scope = !!getSelectedScope(state);
+      const loaded = !hasLoadingObjects(state);
+      return paused && scope && loaded;
+    },
+    "paused"
+  );
+}
+
+/*
+ * useful for when you want to see what is happening
+ * e.g await waitForever()
+ */
+function waitForever() {
+  return new Promise(r => {});
+}
+
+/*
+ * useful for waiting for a short amount of time as
+ * a placeholder for a better waitForX handler.
+ *
+ * e.g await waitForTime(500)
+ */
+function waitForTime(ms) {
+  return new Promise(r => setTimeout(r, ms));
+}
+
+/**
+ * Waits for the debugger to be fully paused.
+ *
+ * @memberof mochitest/waits
+ * @param {Object} dbg
+ * @static
+ */
+async function waitForMappedScopes(dbg) {
+  await waitForState(
+    dbg,
+    state => {
+      const scopes = dbg.selectors.getScopes(state);
+      return scopes && scopes.sourceBindings;
+    },
+    "mapped scopes"
+  );
+}
+
+function isTopFrameSelected(dbg, state) {
+  const pause = dbg.selectors.getPause(state);
+
+  // Make sure we have the paused state.
+  if (!pause) {
+    return false;
+  }
+
+  // Make sure the source text is completely loaded for the
+  // source we are paused in.
+  const sourceId = pause.frame && pause.frame.location.sourceId;
+  const source = dbg.selectors.getSelectedSource(state);
+
+  if (!source) {
+    return false;
+  }
+
+  const isLoaded = source.has("loadedState") && sourceUtils.isLoaded(source);
+  if (!isLoaded) {
+    return false;
+  }
+
+  return source.get("id") == sourceId;
 }
 
 function createDebuggerContext(toolbox) {
-  const win = toolbox.getPanel("jsdebugger").panelWin;
-  const store = win.Debugger.store;
+  const panel = toolbox.getPanel("jsdebugger");
+  const win = panel.panelWin;
+  const { store, client, selectors, actions } = panel.getVarsForTests();
 
   return {
-    actions: win.Debugger.actions,
-    selectors: win.Debugger.selectors,
+    actions: actions,
+    selectors: selectors,
     getState: store.getState,
     store: store,
-    client: win.Debugger.client,
+    client: client,
     toolbox: toolbox,
-    win: win
+    win: win,
+    panel: panel
   };
 }
 
+/**
+ * Clear all the debugger related preferences.
+ */
+function clearDebuggerPreferences() {
+  Services.prefs.clearUserPref("devtools.debugger.pause-on-exceptions");
+  Services.prefs.clearUserPref("devtools.debugger.ignore-caught-exceptions");
+  Services.prefs.clearUserPref("devtools.debugger.tabs");
+  Services.prefs.clearUserPref("devtools.debugger.pending-selected-location");
+  Services.prefs.clearUserPref("devtools.debugger.pending-breakpoints");
+  Services.prefs.clearUserPref("devtools.debugger.expressions");
+}
+
 /**
  * Intilializes the debugger.
  *
  * @memberof mochitest
  * @param {String} url
- * @param {Array} sources
  * @return {Promise} dbg
  * @static
  */
-function initDebugger(url, ...sources) {
-  return Task.spawn(function* () {
-    Services.prefs.clearUserPref("devtools.debugger.tabs")
-    Services.prefs.clearUserPref("devtools.debugger.pending-selected-location")
+function initDebugger(url) {
+  return Task.spawn(function*() {
+    clearDebuggerPreferences();
     const toolbox = yield openNewTabAndToolbox(EXAMPLE_URL + url, "jsdebugger");
     return createDebuggerContext(toolbox);
   });
@@ -321,7 +506,7 @@ window.resumeTest = undefined;
  */
 function pauseTest() {
   info("Test paused. Invoke resumeTest to continue.");
-  return new Promise(resolve => resumeTest = resolve);
+  return new Promise(resolve => (resumeTest = resolve));
 }
 
 // Actions
@@ -334,7 +519,7 @@ function pauseTest() {
  * @return {Object} source
  * @static
  */
-function findSource(dbg, url) {
+function findSource(dbg, url, { silent } = { silent: false }) {
   if (typeof url !== "string") {
     // Support passing in a source object itelf all APIs that use this
     // function support both styles
@@ -343,15 +528,37 @@ function findSource(dbg, url) {
   }
 
   const sources = dbg.selectors.getSources(dbg.getState());
-  const source = sources.find(s => s.get("url").includes(url));
+  const source = sources.find(s => (s.get("url") || "").includes(url));
 
   if (!source) {
-    throw new Error("Unable to find source: " + url);
+    if (silent) {
+      return false;
+    }
+
+    throw new Error(`Unable to find source: ${url}`);
   }
 
   return source.toJS();
 }
 
+function waitForLoadedSource(dbg, url) {
+  return waitForState(
+    dbg,
+    state => findSource(dbg, url, { silent: true }).loadedState == "loaded",
+    "loaded source"
+  );
+}
+
+function waitForLoadedSources(dbg) {
+  return waitForState(
+    dbg,
+    state => {
+      const sources = dbg.selectors.getSources(state).valueSeq().toJS()
+      return !sources.some(source => source.loadedState == "loading")
+    },
+    "loaded source"
+  );
+}
 /**
  * Selects the source.
  *
@@ -363,14 +570,15 @@ function findSource(dbg, url) {
  * @static
  */
 function selectSource(dbg, url, line) {
-  info("Selecting source: " + url);
+  info(`Selecting source: ${url}`);
   const source = findSource(dbg, url);
-  const hasText = !!dbg.selectors.getSourceText(dbg.getState(), source.id);
-  dbg.actions.selectSource(source.id, { line });
+  return dbg.actions.selectSource(source.id, { location: { line } });
+}
 
-  if (!hasText) {
-    return waitForDispatch(dbg, "LOAD_SOURCE_TEXT");
-  }
+function closeTab(dbg, url) {
+  info(`Closing tab: ${url}`);
+  const source = findSource(dbg, url);
+  return dbg.actions.closeTab(source.url);
 }
 
 /**
@@ -381,9 +589,9 @@ function selectSource(dbg, url, line) {
  * @return {Promise}
  * @static
  */
-function stepOver(dbg) {
+async function stepOver(dbg) {
   info("Stepping over");
-  dbg.actions.stepOver();
+  await dbg.actions.stepOver();
   return waitForPaused(dbg);
 }
 
@@ -395,9 +603,9 @@ function stepOver(dbg) {
  * @return {Promise}
  * @static
  */
-function stepIn(dbg) {
+async function stepIn(dbg) {
   info("Stepping in");
-  dbg.actions.stepIn();
+  await dbg.actions.stepIn();
   return waitForPaused(dbg);
 }
 
@@ -409,9 +617,9 @@ function stepIn(dbg) {
  * @return {Promise}
  * @static
  */
-function stepOut(dbg) {
+async function stepOut(dbg) {
   info("Stepping out");
-  dbg.actions.stepOut();
+  await dbg.actions.stepOut();
   return waitForPaused(dbg);
 }
 
@@ -425,8 +633,12 @@ function stepOut(dbg) {
  */
 function resume(dbg) {
   info("Resuming");
-  dbg.actions.resume();
-  return waitForThreadEvents(dbg, "resumed");
+  return dbg.actions.resume();
+}
+
+function deleteExpression(dbg, input) {
+  info(`Delete expression "${input}"`);
+  return dbg.actions.deleteExpression({ input });
 }
 
 /**
@@ -439,7 +651,9 @@ function resume(dbg) {
  * @static
  */
 function reload(dbg, ...sources) {
-  return dbg.client.reload().then(() => waitForSources(...sources));
+  return dbg.client
+    .reload()
+    .then(() => waitForSources(dbg, ...sources), "reloaded");
 }
 
 /**
@@ -475,6 +689,13 @@ function addBreakpoint(dbg, source, line, col) {
   return waitForDispatch(dbg, "ADD_BREAKPOINT");
 }
 
+function disableBreakpoint(dbg, source, line, col) {
+  source = findSource(dbg, source);
+  const sourceId = source.id;
+  dbg.actions.disableBreakpoint({ sourceId, line, col });
+  return waitForDispatch(dbg, "DISABLE_BREAKPOINT");
+}
+
 /**
  * Removes a breakpoint from a source at line/col.
  *
@@ -500,52 +721,98 @@ function removeBreakpoint(dbg, sourceId, line, col) {
  * @return {Promise}
  * @static
  */
-function togglePauseOnExceptions(dbg,
-  pauseOnExceptions, ignoreCaughtExceptions) {
+async function togglePauseOnExceptions(
+  dbg,
+  pauseOnExceptions,
+  ignoreCaughtExceptions
+) {
   const command = dbg.actions.pauseOnExceptions(
     pauseOnExceptions,
     ignoreCaughtExceptions
   );
 
   if (!isPaused(dbg)) {
-    return waitForThreadEvents(dbg, "resumed");
+    await waitForThreadEvents(dbg, "resumed");
+    await waitForLoadedObjects(dbg);
   }
 
   return command;
 }
 
+function waitForActive(dbg) {
+  return waitForState(dbg, state => !dbg.selectors.isPaused(state), "active");
+}
+
 // Helpers
 
 /**
  * Invokes a global function in the debuggee tab.
  *
  * @memberof mochitest/helpers
- * @param {String} fnc
+ * @param {String} fnc The name of a global function on the content window to
+ *                     call. This is applied to structured clones of the
+ *                     remaining arguments to invokeInTab.
+ * @param {Any} ...args Remaining args to serialize and pass to fnc.
  * @return {Promise}
  * @static
  */
-function invokeInTab(fnc) {
-  info(`Invoking function ${fnc} in tab`);
-  return ContentTask.spawn(gBrowser.selectedBrowser, fnc, function* (fnc) {
-    content.wrappedJSObject[fnc](); // eslint-disable-line mozilla/no-cpows-in-tests, max-len
+function invokeInTab(fnc, ...args) {
+  info(`Invoking in tab: ${fnc}(${args.map(uneval).join(",")})`);
+  return ContentTask.spawn(gBrowser.selectedBrowser, { fnc, args }, function*({
+    fnc,
+    args
+  }) {
+    content.wrappedJSObject[fnc](...args); // eslint-disable-line mozilla/no-cpows-in-tests, max-len
   });
 }
 
 const isLinux = Services.appinfo.OS === "Linux";
+const isMac = Services.appinfo.OS === "Darwin";
 const cmdOrCtrl = isLinux ? { ctrlKey: true } : { metaKey: true };
+const shiftOrAlt = isMac
+  ? { accelKey: true, shiftKey: true }
+  : { accelKey: true, altKey: true };
+
+const cmdShift = isMac
+  ? { accelKey: true, shiftKey: true, metaKey: true }
+  : { accelKey: true, shiftKey: true, ctrlKey: true };
+
+// On Mac, going to beginning/end only works with meta+left/right.  On
+// Windows, it only works with home/end.  On Linux, apparently, either
+// ctrl+left/right or home/end work.
+const endKey = isMac
+  ? { code: "VK_RIGHT", modifiers: cmdOrCtrl }
+  : { code: "VK_END" };
+const startKey = isMac
+  ? { code: "VK_LEFT", modifiers: cmdOrCtrl }
+  : { code: "VK_HOME" };
+
 const keyMappings = {
-  sourceSearch: { code: "p", modifiers: cmdOrCtrl},
-  fileSearch: { code: "f", modifiers: cmdOrCtrl},
-  "Enter": { code: "VK_RETURN" },
-  "Up": { code: "VK_UP" },
-  "Down": { code: "VK_DOWN" },
-  "Tab": { code: "VK_TAB" },
-  "Escape": { code: "VK_ESCAPE" },
+  debugger: { code: "s", modifiers: shiftOrAlt },
+  inspector: { code: "c", modifiers: shiftOrAlt },
+  quickOpen: { code: "p", modifiers: cmdOrCtrl },
+  quickOpenFunc: { code: "o", modifiers: cmdShift },
+  quickOpenLine: { code: ":", modifiers: cmdOrCtrl },
+  fileSearch: { code: "f", modifiers: cmdOrCtrl },
+  Enter: { code: "VK_RETURN" },
+  ShiftEnter: { code: "VK_RETURN", modifiers: shiftOrAlt },
+  Up: { code: "VK_UP" },
+  Down: { code: "VK_DOWN" },
+  Right: { code: "VK_RIGHT" },
+  Left: { code: "VK_LEFT" },
+  End: endKey,
+  Start: startKey,
+  Tab: { code: "VK_TAB" },
+  Escape: { code: "VK_ESCAPE" },
+  Delete: { code: "VK_DELETE" },
   pauseKey: { code: "VK_F8" },
   resumeKey: { code: "VK_F8" },
   stepOverKey: { code: "VK_F10" },
-  stepInKey: { code: "VK_F11", modifiers: { ctrlKey: isLinux }},
-  stepOutKey: { code: "VK_F11", modifiers: { ctrlKey: isLinux, shiftKey: true }}
+  stepInKey: { code: "VK_F11", modifiers: { ctrlKey: isLinux } },
+  stepOutKey: {
+    code: "VK_F11",
+    modifiers: { ctrlKey: isLinux, shiftKey: true }
+  }
 };
 
 /**
@@ -558,38 +825,81 @@ const keyMappings = {
  * @static
  */
 function pressKey(dbg, keyName) {
-  let keyEvent = keyMappings[keyName];
+  const keyEvent = keyMappings[keyName];
 
   const { code, modifiers } = keyEvent;
-  return EventUtils.synthesizeKey(
-    code,
-    modifiers || {},
-    dbg.win
-  );
+  return EventUtils.synthesizeKey(code, modifiers || {}, dbg.win);
 }
 
 function type(dbg, string) {
-  string.split("").forEach(char => {
-    EventUtils.synthesizeKey(char, {}, dbg.win);
-  });
+  string.split("").forEach(char => EventUtils.synthesizeKey(char, {}, dbg.win));
 }
 
-function isVisibleWithin(outerEl, innerEl) {
-  info(`isVisibleWithin`);
+
+/*
+ * Checks to see if the inner element is visible inside the editor.
+ *
+ * @memberof mochitest/helpers
+ * @param {Object} dbg
+ * @param {HTMLElement} inner element
+ * @return {boolean}
+ * @static
+ */
+
+function isVisibleInEditor(dbg, element) {
+  return isVisible(findElement(dbg, "codeMirror"), element);
+}
+
+/*
+ * Checks to see if the inner element is visible inside the
+ * outer element.
+ *
+ * Note, the inner element does not need to be entirely visible,
+ * it is possible for it to be somewhat clipped by the outer element's
+ * bounding element or for it to span the entire length, starting before the
+ * outer element and ending after.
+ *
+ * @memberof mochitest/helpers
+ * @param {HTMLElement} outer element
+ * @param {HTMLElement} inner element
+ * @return {boolean}
+ * @static
+ */
+function isVisible(outerEl, innerEl) {
+  if (!innerEl || !outerEl) {
+    return false;
+  }
+
   const innerRect = innerEl.getBoundingClientRect();
   const outerRect = outerEl.getBoundingClientRect();
-  info(`isVisibleWithin innerRect.top ${innerRect.top} outerRect.top ${outerRect.top} innerRect.bottom ${innerRect.bottom} outerRect.bottom ${outerRect.bottom}`);
 
-  return innerRect.top > outerRect.top &&
-    innerRect.bottom < outerRect.bottom;
+  const verticallyVisible =
+    (innerRect.top >= outerRect.top || innerRect.bottom <= outerRect.bottom)
+    || (innerRect.top < outerRect.top && innerRect.bottom > outerRect.bottom);
+
+  const horizontallyVisible =
+    (innerRect.left >= outerRect.left || innerRect.right <= outerRect.right)
+    || (innerRect.left < outerRect.left && innerRect.right > outerRect.right);
+
+  const visible = verticallyVisible && horizontallyVisible;
+  return visible;
 }
 
 const selectors = {
   callStackHeader: ".call-stack-pane ._header",
   callStackBody: ".call-stack-pane .pane",
+  expressionNode: i =>
+    `.expressions-list .expression-container:nth-child(${i}) .object-label`,
+  expressionValue: i =>
+    `.expressions-list .expression-container:nth-child(${i}) .object-delimiter + *`,
+  expressionClose: i =>
+    `.expressions-list .expression-container:nth-child(${i}) .close`,
+  expressionNodes: ".expressions-list .tree-node",
   scopesHeader: ".scopes-pane ._header",
   breakpointItem: i => `.breakpoints-list .breakpoint:nth-child(${i})`,
   scopeNode: i => `.scopes-list .tree-node:nth-child(${i}) .object-label`,
+  scopeValue: i =>
+    `.scopes-list .tree-node:nth-child(${i}) .object-delimiter + *`,
   frame: i => `.frames ul li:nth-child(${i})`,
   frames: ".frames ul li",
   gutter: i => `.CodeMirror-code *:nth-child(${i}) .CodeMirror-linenumber`,
@@ -599,15 +909,21 @@ const selectors = {
   highlightLine: ".CodeMirror-code > .highlight-line",
   codeMirror: ".CodeMirror",
   resume: ".resume.active",
+  sourceTabs: ".source-tabs",
   stepOver: ".stepOver.active",
   stepOut: ".stepOut.active",
   stepIn: ".stepIn.active",
   toggleBreakpoints: ".breakpoints-toggle",
   prettyPrintButton: ".prettyPrint",
-  sourceFooter: ".source-footer",
+  sourcesFooter: ".sources-panel .source-footer",
+  editorFooter: ".editor-pane .source-footer",
   sourceNode: i => `.sources-list .tree-node:nth-child(${i})`,
   sourceNodes: ".sources-list .tree-node",
   sourceArrow: i => `.sources-list .tree-node:nth-child(${i}) .arrow`,
+  resultItems: ".result-list .result-item",
+  fileMatch: ".managed-tree .result",
+  popup: ".popover",
+  tooltip: ".tooltip"
 };
 
 function getSelector(elementName, ...args) {
@@ -647,21 +963,39 @@ function findAllElements(dbg, elementName, ...args) {
  * @return {Promise}
  * @static
  */
-function clickElement(dbg, elementName, ...args) {
+async function clickElement(dbg, elementName, ...args) {
   const selector = getSelector(elementName, ...args);
-  return EventUtils.synthesizeMouseAtCenter(
+  const el = await waitForElement(dbg, selector);
+
+  el.scrollIntoView();
+
+  return clickElementWithSelector(dbg, selector);
+}
+
+function clickElementWithSelector(dbg, selector) {
+  EventUtils.synthesizeMouseAtCenter(
     findElementWithSelector(dbg, selector),
     {},
     dbg.win
   );
 }
 
+function dblClickElement(dbg, elementName, ...args) {
+  const selector = getSelector(elementName, ...args);
+
+  return EventUtils.synthesizeMouseAtCenter(
+    findElementWithSelector(dbg, selector),
+    { clickCount: 2 },
+    dbg.win
+  );
+}
+
 function rightClickElement(dbg, elementName, ...args) {
   const selector = getSelector(elementName, ...args);
   const doc = dbg.win.document;
   return EventUtils.synthesizeMouseAtCenter(
     doc.querySelector(selector),
-    {type: "contextmenu"},
+    { type: "contextmenu" },
     dbg.win
   );
 }
@@ -671,10 +1005,10 @@ function selectMenuItem(dbg, index) {
   const doc = dbg.toolbox.win.document;
 
   // there are several context menus, we want the one with the menu-api
-  const popup = doc.querySelector("menupopup[menu-api=\"true\"]");
+  const popup = doc.querySelector('menupopup[menu-api="true"]');
 
   const item = popup.querySelector(`menuitem:nth-child(${index})`);
-  return EventUtils.synthesizeMouseAtCenter(item, {}, dbg.toolbox.win );
+  return EventUtils.synthesizeMouseAtCenter(item, {}, dbg.toolbox.win);
 }
 
 /**
@@ -692,3 +1026,8 @@ function toggleCallStack(dbg) {
 function toggleScopes(dbg) {
   return findElement(dbg, "scopesHeader").click();
 }
+
+function getCM(dbg) {
+  const el = dbg.win.document.querySelector(".CodeMirror");
+  return el.CodeMirror;
+}
