@@ -3,9 +3,11 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const {DOM, createClass, createFactory, PropTypes} = require("devtools/client/shared/vendor/react");
+const {DOM, Component, createFactory, PropTypes} = require("devtools/client/shared/vendor/react");
 const {div, button} = DOM;
+
 const ToolboxTab = createFactory(require("devtools/client/framework/components/toolbox-tab"));
+const ToolboxTabs = createFactory(require("devtools/client/framework/components/toolbox-tabs"));
 
 /**
  * This is the overall component for the toolbox toolbar. It is designed to not know how
@@ -13,33 +15,35 @@ const ToolboxTab = createFactory(require("devtools/client/framework/components/t
  * ToolboxController component controls the changing state, and passes in everything as
  * props.
  */
-module.exports = createClass({
-  displayName: "ToolboxToolbar",
-
-  propTypes: {
-    // The currently focused item (for arrow keyboard navigation)
-    // This ID determines the tabindex being 0 or -1.
-    focusedButton: PropTypes.string,
-    // List of command button definitions.
-    toolboxButtons: PropTypes.array,
-    // The id of the currently selected tool, e.g. "inspector"
-    currentToolId: PropTypes.string,
-    // An optionally highlighted tool, e.g. "inspector"
-    highlightedTool: PropTypes.string,
-    // List of tool panel definitions.
-    panelDefinitions: PropTypes.array,
-    // Function to select a tool based on its id.
-    selectTool: PropTypes.func,
-    // Keep a record of what button is focused.
-    focusButton: PropTypes.func,
-    // The options button definition.
-    optionsPanel: PropTypes.object,
-    // Hold off displaying the toolbar until enough information is ready for it to render
-    // nicely.
-    canRender: PropTypes.bool,
-    // Localization interface.
-    L10N: PropTypes.object,
-  },
+class ToolboxToolbar extends Component {
+  static get propTypes() {
+    return {
+      // The currently focused item (for arrow keyboard navigation)
+      // This ID determines the tabindex being 0 or -1.
+      focusedButton: PropTypes.string,
+      // List of command button definitions.
+      toolboxButtons: PropTypes.array,
+      // The id of the currently selected tool, e.g. "inspector"
+      currentToolId: PropTypes.string,
+      // An optionally highlighted tool, e.g. "inspector"
+      highlightedTool: PropTypes.string,
+      // List of tool panel definitions.
+      panelDefinitions: PropTypes.array,
+      // Function to select a tool based on its id.
+      selectTool: PropTypes.func,
+      // Keep a record of what button is focused.
+      focusButton: PropTypes.func,
+      // The options button definition.
+      optionsPanel: PropTypes.object,
+      // Hold off displaying the toolbar until enough information is ready for
+      // it to render nicely.
+      canRender: PropTypes.bool,
+      // Localization interface.
+      L10N: PropTypes.object,
+      // The devtools toolbox
+      toolbox: PropTypes.object,
+    };
+  }
 
   /**
    * The render function is kept fairly short for maintainability. See the individual
@@ -52,7 +56,7 @@ module.exports = createClass({
         div(
           containerProps,
           renderToolboxButtonsStart(this.props),
-          renderTabs(this.props),
+          ToolboxTabs(this.props),
           renderToolboxButtonsEnd(this.props),
           renderOptions(this.props),
           renderSeparator(),
@@ -61,36 +65,10 @@ module.exports = createClass({
       )
       : div(containerProps);
   }
-});
-
-/**
- * Render all of the tabs, this takes in the panel definitions and builds out
- * the buttons for each of them.
- *
- * @param {Array}    panelDefinitions - Array of objects that define panels.
- * @param {String}   currentToolId - The currently selected tool's id; e.g. "inspector".
- * @param {String}   highlightedTool - If a tool is highlighted, this is it's id.
- * @param {Function} selectTool - Function to select a tool in the toolbox.
- * @param {String}   focusedButton - The id of the focused button.
- * @param {Function} focusButton - Keep a record of the currently focused button.
- */
-function renderTabs({panelDefinitions, currentToolId, highlightedTool, selectTool,
-                     focusedButton, focusButton}) {
-  // A wrapper is needed to get flex sizing correct in XUL.
-  return div({className: "toolbox-tabs-wrapper"},
-    div({className: "toolbox-tabs"},
-      ...panelDefinitions.map(panelDefinition => ToolboxTab({
-        panelDefinition,
-        currentToolId,
-        highlightedTool,
-        selectTool,
-        focusedButton,
-        focusButton,
-      }))
-    )
-  );
 }
 
+module.exports = ToolboxToolbar;
+
 /**
  * A little helper function to call renderToolboxButtons for buttons at the start
  * of the toolbox.
@@ -130,12 +108,19 @@ function renderToolboxButtons({toolboxButtons, focusedButton, focusButton}, isSt
 
   return div({id: `toolbox-buttons-${isStart ? "start" : "end"}`},
     ...visibleButtons.map(command => {
-      const {id, description, onClick, isChecked, className: buttonClass} = command;
+      const {
+        id,
+        description,
+        onClick,
+        isChecked,
+        className: buttonClass,
+        onKeyDown
+      } = command;
       return button({
         id,
         title: description,
         className: (
-          "command-button command-button-invertable devtools-button "
+          "command-button devtools-button "
           + buttonClass + (isChecked ? " checked" : "")
         ),
         onClick: (event) => {
@@ -143,14 +128,18 @@ function renderToolboxButtons({toolboxButtons, focusedButton, focusButton}, isSt
           focusButton(id);
         },
         onFocus: () => focusButton(id),
-        tabIndex: id === focusedButton ? "0" : "-1"
+        tabIndex: id === focusedButton ? "0" : "-1",
+        onKeyDown: (event) => {
+          onKeyDown(event);
+        }
       });
-    })
+    }),
+    isStart ? div({className: "devtools-separator"}) : null
   );
 }
 
 /**
- * The options button is a ToolboxTab just like in the renderTabs() function. However
+ * The options button is a ToolboxTab just like in the ToolboxTabs component. However
  * it is separate from the normal tabs, so deal with it separately here.
  *
  * @param {Object}   optionsPanel - A single panel definition for the options panel.
@@ -174,10 +163,7 @@ function renderOptions({optionsPanel, currentToolId, selectTool, focusedButton,
  * Render a separator.
  */
 function renderSeparator() {
-  return div({
-    id: "toolbox-controls-separator",
-    className: "devtools-separator"
-  });
+  return div({className: "devtools-separator"});
 }
 
 /**
