@@ -5,6 +5,18 @@
 
 // Tests that the keybindings for opening and closing the inspector work as expected
 // Can probably make this a shared test that tests all of the tools global keybindings
+const TEST_URL = "data:text/html,<html><head><title>Test for the " +
+                 "highlighter keybindings</title></head><body>" +
+                 "<h1>Keybindings!</h1></body></html>"
+
+// Use the new debugger frontend because the old one swallows the netmonitor shortcut:
+// https://bugzilla.mozilla.org/show_bug.cgi?id=1370442#c7
+Services.prefs.setBoolPref("devtools.debugger.new-debugger-frontend", true);
+registerCleanupFunction(function* () {
+  Services.prefs.clearUserPref("devtools.debugger.new-debugger-frontend");
+});
+
+const {gDevToolsBrowser} = require("devtools/client/framework/devtools-browser");
 
 function test()
 {
@@ -15,20 +27,14 @@ function test()
   let inspector;
   let keysetMap = { };
 
-  gBrowser.selectedTab = gBrowser.addTab();
-  gBrowser.selectedBrowser.addEventListener("load", function onload() {
-    gBrowser.selectedBrowser.removeEventListener("load", onload, true);
+  addTab(TEST_URL).then(function () {
     doc = content.document;
     node = doc.querySelector("h1");
     waitForFocus(setupKeyBindingsTest);
-  }, true);
-
-  content.location = "data:text/html,<html><head><title>Test for the " +
-                     "highlighter keybindings</title></head><body>" +
-                     "<h1>Keybindings!</h1></body></html>";
+  });
 
   function buildDevtoolsKeysetMap(keyset) {
-    [].forEach.call(keyset.querySelectorAll("key"), function(key) {
+    [].forEach.call(keyset.querySelectorAll("key"), function (key) {
 
       if (!key.getAttribute("key")) {
         return;
@@ -46,10 +52,10 @@ function test()
           metaKey: modifiers.match("meta"),
           accelKey: modifiers.match("accel")
         },
-        synthesizeKey: function() {
+        synthesizeKey: function () {
           EventUtils.synthesizeKey(this.key, this.modifierOpt);
         }
-      }
+      };
     });
   }
 
@@ -68,7 +74,7 @@ function test()
 
   function inspectorShouldBeOpenAndHighlighting(aInspector, aToolbox)
   {
-    is (aToolbox.currentToolId, "inspector", "Correct tool has been loaded");
+    is(aToolbox.currentToolId, "inspector", "Correct tool has been loaded");
 
     aToolbox.once("picker-started", () => {
       ok(true, "picker-started event received, highlighter started");
@@ -86,29 +92,29 @@ function test()
 
   function webconsoleShouldBeSelected(aToolbox)
   {
-      is (aToolbox.currentToolId, "webconsole", "webconsole should be selected.");
+    is(aToolbox.currentToolId, "webconsole", "webconsole should be selected.");
 
-      gDevTools.once("select-tool-command", () => {
-        jsdebuggerShouldBeSelected(aToolbox);
-      });
-      keysetMap.jsdebugger.synthesizeKey();
+    gDevTools.once("select-tool-command", () => {
+      jsdebuggerShouldBeSelected(aToolbox);
+    });
+    keysetMap.jsdebugger.synthesizeKey();
   }
 
   function jsdebuggerShouldBeSelected(aToolbox)
   {
-      is (aToolbox.currentToolId, "jsdebugger", "jsdebugger should be selected.");
+    is(aToolbox.currentToolId, "jsdebugger", "jsdebugger should be selected.");
 
-      gDevTools.once("select-tool-command", () => {
-        netmonitorShouldBeSelected(aToolbox);
-      });
+    gDevTools.once("select-tool-command", () => {
+      netmonitorShouldBeSelected(aToolbox);
+    });
 
-      keysetMap.netmonitor.synthesizeKey();
+    keysetMap.netmonitor.synthesizeKey();
   }
 
   function netmonitorShouldBeSelected(aToolbox, panel)
   {
-      is (aToolbox.currentToolId, "netmonitor", "netmonitor should be selected.");
-      finishUp();
+    is(aToolbox.currentToolId, "netmonitor", "netmonitor should be selected.");
+    finishUp();
   }
 
   function finishUp() {
