@@ -4,35 +4,43 @@
 
 "use strict";
 
-const { addons, createClass, createFactory, DOM: dom, PropTypes } = require("devtools/client/shared/vendor/react");
+const {
+  DOM: dom,
+  PropTypes,
+  PureComponent,
+} = require("devtools/client/shared/vendor/react");
 const { findDOMNode } = require("devtools/client/shared/vendor/react-dom");
 
 // Reps
-const { REPS } = require("devtools/client/shared/components/reps/reps");
-const Rep = createFactory(REPS.Rep);
+const { REPS, MODE } = require("devtools/client/shared/components/reps/reps");
+const { Rep } = REPS;
 const ElementNode = REPS.ElementNode;
 
 const Types = require("../types");
 
-module.exports = createClass({
+class GridItem extends PureComponent {
+  static get propTypes() {
+    return {
+      getSwatchColorPickerTooltip: PropTypes.func.isRequired,
+      grid: PropTypes.shape(Types.grid).isRequired,
+      setSelectedNode: PropTypes.func.isRequired,
+      onHideBoxModelHighlighter: PropTypes.func.isRequired,
+      onSetGridOverlayColor: PropTypes.func.isRequired,
+      onShowBoxModelHighlighterForNode: PropTypes.func.isRequired,
+      onToggleGridHighlighter: PropTypes.func.isRequired,
+    };
+  }
 
-  displayName: "GridItem",
-
-  propTypes: {
-    getSwatchColorPickerTooltip: PropTypes.func.isRequired,
-    grid: PropTypes.shape(Types.grid).isRequired,
-    setSelectedNode: PropTypes.func.isRequired,
-    onHideBoxModelHighlighter: PropTypes.func.isRequired,
-    onSetGridOverlayColor: PropTypes.func.isRequired,
-    onShowBoxModelHighlighterForNode: PropTypes.func.isRequired,
-    onToggleGridHighlighter: PropTypes.func.isRequired,
-  },
-
-  mixins: [ addons.PureRenderMixin ],
+  constructor(props) {
+    super(props);
+    this.setGridColor = this.setGridColor.bind(this);
+    this.translateNodeFrontToGrip = this.translateNodeFrontToGrip.bind(this);
+    this.onGridCheckboxClick = this.onGridCheckboxClick.bind(this);
+  }
 
   componentDidMount() {
-    let tooltip = this.props.getSwatchColorPickerTooltip();
     let swatchEl = findDOMNode(this).querySelector(".grid-color-swatch");
+    let tooltip = this.props.getSwatchColorPickerTooltip();
 
     let previousColor;
     tooltip.addSwatch(swatchEl, {
@@ -45,18 +53,18 @@ module.exports = createClass({
         previousColor = this.props.grid.color;
       },
     });
-  },
+  }
 
   componentWillUnmount() {
-    let tooltip = this.props.getSwatchColorPickerTooltip();
     let swatchEl = findDOMNode(this).querySelector(".grid-color-swatch");
+    let tooltip = this.props.getSwatchColorPickerTooltip();
     tooltip.removeSwatch(swatchEl);
-  },
+  }
 
   setGridColor() {
     let color = findDOMNode(this).querySelector(".grid-color-value").textContent;
     this.props.onSetGridOverlayColor(this.props.grid.nodeFront, color);
-  },
+  }
 
   /**
    * While waiting for a reps fix in https://github.com/devtools-html/reps/issues/92,
@@ -81,12 +89,14 @@ module.exports = createClass({
       preview: {
         attributes: attributesMap,
         attributesLength: attributes.length,
+        // All the grid containers are assumed to be in the DOM tree.
+        isConnected: true,
         // nodeName is already lowerCased in Node grips
         nodeName: nodeFront.nodeName.toLowerCase(),
         nodeType: nodeFront.nodeType,
       }
     };
-  },
+  }
 
   onGridCheckboxClick(e) {
     // If the click was on the svg icon to select the node in the inspector, bail out.
@@ -104,39 +114,37 @@ module.exports = createClass({
     } = this.props;
 
     onToggleGridHighlighter(grid.nodeFront);
-  },
+  }
 
   render() {
     let {
       grid,
+      setSelectedNode,
       onHideBoxModelHighlighter,
       onShowBoxModelHighlighterForNode,
-      setSelectedNode,
     } = this.props;
     let { nodeFront } = grid;
 
     return dom.li(
-      {
-        key: grid.id,
-        className: "grid-item",
-      },
+      {},
       dom.label(
         {},
         dom.input(
           {
+            checked: grid.highlighted,
             type: "checkbox",
             value: grid.id,
-            checked: grid.highlighted,
             onChange: this.onGridCheckboxClick,
           }
         ),
         Rep(
           {
             defaultRep: ElementNode,
+            mode: MODE.TINY,
             object: this.translateNodeFrontToGrip(nodeFront),
             onDOMNodeMouseOut: () => onHideBoxModelHighlighter(),
             onDOMNodeMouseOver: () => onShowBoxModelHighlighterForNode(nodeFront),
-            onInspectIconClick: () => setSelectedNode(nodeFront),
+            onInspectIconClick: () => setSelectedNode(nodeFront, "layout-panel"),
           }
         )
       ),
@@ -160,6 +168,7 @@ module.exports = createClass({
         grid.color
       )
     );
-  },
+  }
+}
 
-});
+module.exports = GridItem;
