@@ -12,10 +12,10 @@ const {ELEMENT_STYLE} = require("devtools/shared/specs/styles");
 const {TextProperty} =
       require("devtools/client/inspector/rules/models/text-property");
 const {promiseWarn} = require("devtools/client/inspector/shared/utils");
-const {parseDeclarations} = require("devtools/shared/css/parsing-utils");
+const {parseNamedDeclarations} = require("devtools/shared/css/parsing-utils");
 const Services = require("Services");
 
-const STYLE_INSPECTOR_PROPERTIES = "devtools-shared/locale/styleinspector.properties";
+const STYLE_INSPECTOR_PROPERTIES = "devtools/shared/locales/styleinspector.properties";
 const {LocalizationHelper} = require("devtools/shared/l10n");
 const STYLE_INSPECTOR_L10N = new LocalizationHelper(STYLE_INSPECTOR_PROPERTIES);
 
@@ -117,7 +117,7 @@ Rule.prototype = {
    * The rule's line within a stylesheet
    */
   get ruleLine() {
-    return this.domRule ? this.domRule.line : "";
+    return this.domRule ? this.domRule.line : -1;
   },
 
   /**
@@ -127,30 +127,6 @@ Rule.prototype = {
     return this.domRule ? this.domRule.column : null;
   },
 
-  /**
-   * Get display name for this rule based on the original source
-   * for this rule's style sheet.
-   *
-   * @return {Promise}
-   *         Promise which resolves with location as an object containing
-   *         both the full and short version of the source string.
-   */
-  getOriginalSourceStrings: function () {
-    return this.domRule.getOriginalLocation().then(({href,
-                                                     line, mediaText}) => {
-      let mediaString = mediaText ? " @" + mediaText : "";
-      let linePart = line > 0 ? (":" + line) : "";
-
-      let sourceStrings = {
-        full: (href || CssLogic.l10n("rule.sourceInline")) + linePart +
-          mediaString,
-        short: CssLogic.shortSource({href: href}) + linePart + mediaString
-      };
-
-      return sourceStrings;
-    });
-  },
-
   /**
    * Returns true if the rule matches the creation options
    * specified.
@@ -242,8 +218,8 @@ Rule.prototype = {
       // Note that even though StyleRuleActors normally provide parsed
       // declarations already, _applyPropertiesNoAuthored is only used when
       // connected to older backend that do not provide them. So parse here.
-      for (let cssProp of parseDeclarations(this.cssProperties.isKnown,
-                                            this.style.authoredText)) {
+      for (let cssProp of parseNamedDeclarations(this.cssProperties.isKnown,
+                                                 this.style.authoredText)) {
         cssProps[cssProp.name] = cssProp;
       }
 
@@ -440,18 +416,15 @@ Rule.prototype = {
     // Starting with FF49, StyleRuleActors provide parsed declarations.
     let props = this.style.declarations;
     if (!props.length) {
-      props = parseDeclarations(this.cssProperties.isKnown,
-                                this.style.authoredText, true);
+      // If the authored text has an invalid property, it will show up
+      // as nameless.  Skip these as we don't currently have a good
+      // way to display them.
+      props = parseNamedDeclarations(this.cssProperties.isKnown,
+                                     this.style.authoredText, true);
     }
 
     for (let prop of props) {
       let name = prop.name;
-      // If the authored text has an invalid property, it will show up
-      // as nameless.  Skip these as we don't currently have a good
-      // way to display them.
-      if (!name) {
-        continue;
-      }
       // In an inherited rule, we only show inherited properties.
       // However, we must keep all properties in order for rule
       // rewriting to work properly.  So, compute the "invisible"
@@ -683,4 +656,4 @@ Rule.prototype = {
   }
 };
 
-exports.Rule = Rule;
+module.exports = Rule;
