@@ -6,18 +6,26 @@
 
 "use strict";
 
-const {Cc, Ci, Cu} = require("chrome");
+const {Cc, Ci, Cu, CC} = require("chrome");
 const { XPCOMUtils } = Cu.import("resource://gre/modules/XPCOMUtils.jsm", {});
 const Services = require("Services");
 
 loader.lazyRequireGetter(this, "NetworkHelper",
                                "devtools/shared/webconsole/network-helper");
-loader.lazyRequireGetter(this, "JsonViewUtils",
-                               "devtools/client/jsonview/utils");
+loader.lazyGetter(this, "debug", function () {
+  let {AppConstants} = require("resource://gre/modules/AppConstants.jsm");
+  return !!(AppConstants.DEBUG || AppConstants.DEBUG_JS_MODULES);
+});
 
 const childProcessMessageManager =
   Cc["@mozilla.org/childprocessmessagemanager;1"]
     .getService(Ci.nsISyncMessageSender);
+const BinaryInput = CC("@mozilla.org/binaryinputstream;1",
+                       "nsIBinaryInputStream", "setInputStream");
+const BufferStream = CC("@mozilla.org/io/arraybuffer-input-stream;1",
+                       "nsIArrayBufferInputStream", "setData");
+const encodingLength = 0;
+const encoder = new TextEncoder();
 
 // Localization
 loader.lazyGetter(this, "jsonViewStrings", () => {
@@ -50,8 +58,8 @@ Converter.prototype = {
    * 1. asyncConvertData captures the listener
    * 2. onStartRequest fires, initializes stuff, modifies the listener
    *    to match our output type
-   * 3. onDataAvailable spits it back to the listener
-   * 4. onStopRequest spits it back to the listener
+   * 3. onDataAvailable converts to UTF-8 and spits back to the listener
+   * 4. onStopRequest flushes data and spits back to the listener
    * 5. convert does nothing, it's just the synchronous version
    *    of asyncConvertData
    */
@@ -64,7 +72,29 @@ Converter.prototype = {
   },
 
   onDataAvailable: function (request, context, inputStream, offset, count) {
-    this.listener.onDataAvailable(...arguments);
+    // If the encoding is not known, store data in an array until we have enough bytes.
+    if (this.encodingArray) {
+      let desired = encodingLength - this.encodingArray.length;
+      let n = Math.min(desired, count);
+      let bytes = new BinaryInput(inputStream).readByteArray(n);
+      offset += n;
+      count -= n;
+      this.encodingArray.push(...bytes);
+      if (n < desired) {
+        // Wait until there is more data.
+        return;
+      }
+      this.determineEncoding(request, context);
+    }
+
+    // Spit back the data if the encoding is UTF-8, otherwise convert it first.
+    if (!this.decoder) {
+      this.listener.onDataAvailable(request, context, inputStream, offset, count);
+    } else {
+      let buffer = new ArrayBuffer(count);
+      new BinaryInput(inputStream).readArrayBuffer(count, buffer);
+      this.convertAndSendBuffer(request, context, buffer);
+    }
   },
 
   onStartRequest: function (request, context) {
@@ -74,7 +104,7 @@ Converter.prototype = {
     request.QueryInterface(Ci.nsIChannel);
     request.contentType = "text/html";
 
-    // JSON enforces UTF-8 charset (see bug 741776).
+    // Don't honor the charset parameter and use UTF-8 (see bug 741776).
     request.contentCharset = "UTF-8";
 
     // Changing the content type breaks saving functionality. Fix it.
@@ -90,22 +120,67 @@ Converter.prototype = {
 
     // Initialize stuff.
     let win = NetworkHelper.getWindowForRequest(request);
-    exportData(win, request);
+    this.data = exportData(win, request);
     win.addEventListener("DOMContentLoaded", event => {
       win.addEventListener("contentMessage", onContentMessage, false, true);
     }, {once: true});
+    keepThemeUpdated(win);
 
-    // Insert the initial HTML code.
-    let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
-                      .createInstance(Ci.nsIScriptableUnicodeConverter);
-    converter.charset = "UTF-8";
-    let stream = converter.convertToInputStream(initialHTML(win.document));
-    this.listener.onDataAvailable(request, context, stream, 0, stream.available());
+    // Send the initial HTML code.
+    let bytes = encoder.encode(initialHTML(win.document));
+    this.convertAndSendBuffer(request, context, bytes.buffer);
+
+    // Create an array to store data until the encoding is determined.
+    this.encodingArray = [];
   },
 
   onStopRequest: function (request, context, statusCode) {
+    // Flush data.
+    if (this.encodingArray) {
+      this.determineEncoding(request, context, true);
+    } else {
+      this.convertAndSendBuffer(request, context, new ArrayBuffer(0), true);
+    }
+
+    // Stop the request.
     this.listener.onStopRequest(request, context, statusCode);
     this.listener = null;
+    this.decoder = null;
+    this.data = null;
+  },
+
+  // Determines the encoding of the response.
+  determineEncoding: function (request, context, flush = false) {
+    // Always use UTF-8
+    let encoding = "UTF-8";
+    let bytes = this.encodingArray;
+
+    // Create a decoder unless the data is already in UTF-8.
+    if (encoding !== "UTF-8") {
+      this.decoder = new TextDecoder(encoding, {ignoreBOM: true});
+    }
+
+    this.data.encoding = encoding;
+
+    // Send the bytes in encodingArray, and remove it.
+    let buffer = new Uint8Array(bytes).buffer;
+    this.convertAndSendBuffer(request, context, buffer, flush);
+    this.encodingArray = null;
+  },
+
+  // Converts an ArrayBuffer to UTF-8 and sends it.
+  convertAndSendBuffer: function (request, context, buffer, flush = false) {
+    // If the encoding is not UTF-8, decode the buffer and encode into UTF-8.
+    if (this.decoder) {
+      let data = this.decoder.decode(buffer, {stream: !flush});
+      buffer = encoder.encode(data).buffer;
+    }
+
+    // Create an input stream that contains the bytes in the buffer.
+    let stream = new BufferStream(buffer, 0, buffer.byteLength);
+
+    // Send the input stream.
+    this.listener.onDataAvailable(request, context, stream, 0, stream.available());
   }
 };
 
@@ -113,24 +188,23 @@ Converter.prototype = {
 // To save with the proper extension we need the original content type,
 // which has been replaced by application/vnd.mozilla.json.view
 function fixSave(request) {
-  let originalType;
+  let match;
   if (request instanceof Ci.nsIHttpChannel) {
     try {
       let header = request.getResponseHeader("Content-Type");
-      originalType = header.split(";")[0];
+      match = header.match(/^(application\/(?:[^;]+\+)?json)(?:;|$)/);
     } catch (err) {
       // Handled below
     }
   } else {
     let uri = request.QueryInterface(Ci.nsIChannel).URI.spec;
-    let match = uri.match(/^data:(.*?)[,;]/);
-    if (match) {
-      originalType = match[1];
-    }
+    match = uri.match(/^data:(application\/(?:[^;,]+\+)?json)[;,]/);
   }
-  const JSON_TYPES = ["application/json", "application/manifest+json"];
-  if (!JSON_TYPES.includes(originalType)) {
-    originalType = JSON_TYPES[0];
+  let originalType;
+  if (match) {
+    originalType = match[1];
+  } else {
+    originalType = "application/json";
   }
   request.QueryInterface(Ci.nsIWritablePropertyBag);
   request.setProperty("contentType", originalType);
@@ -138,6 +212,12 @@ function fixSave(request) {
 
 // Exports variables that will be accessed by the non-privileged scripts.
 function exportData(win, request) {
+  let data = Cu.createObjectIn(win, {
+    defineAs: "JSONView"
+  });
+
+  data.debug = debug;
+
   let Locale = {
     $STR: key => {
       try {
@@ -148,7 +228,7 @@ function exportData(win, request) {
       }
     }
   };
-  JsonViewUtils.exportIntoContentScope(win, Locale, "Locale");
+  data.Locale = Cu.cloneInto(Locale, win, {cloneFunctions: true});
 
   let headers = {
     response: [],
@@ -168,7 +248,9 @@ function exportData(win, request) {
       }
     });
   }
-  JsonViewUtils.exportIntoContentScope(win, headers, "headers");
+  data.headers = Cu.cloneInto(headers, win);
+
+  return data;
 }
 
 // Serializes a qualifiedName and an optional set of attributes into an HTML
@@ -198,26 +280,27 @@ function initialHTML(doc) {
     os = "linux";
   }
 
-  let base = doc.createElement("base");
-  base.href = "resource://devtools/client/jsonview/";
+  // The base URI is prepended to all URIs instead of using a <base> element
+  // because the latter can be blocked by a CSP base-uri directive (bug 1316393)
+  let baseURI = "resource://devtools-client-jsonview/";
 
   let style = doc.createElement("link");
   style.rel = "stylesheet";
   style.type = "text/css";
-  style.href = "css/main.css";
+  style.href = baseURI + "css/main.css";
 
   let script = doc.createElement("script");
-  script.src = "lib/require.js";
-  script.dataset.main = "viewer-config";
+  script.src = baseURI + "lib/require.js";
+  script.dataset.main = baseURI + "viewer-config.js";
   script.defer = true;
 
   let head = doc.createElement("head");
-  head.append(base, style, script);
+  head.append(style, script);
 
   return "<!DOCTYPE html>\n" +
     startTag("html", {
       "platform": os,
-      "class": "theme-" + JsonViewUtils.getCurrentTheme(),
+      "class": "theme-" + Services.prefs.getCharPref("devtools.theme"),
       "dir": Services.locale.isAppLocaleRTL ? "rtl" : "ltr"
     }) +
     head.outerHTML +
@@ -226,6 +309,18 @@ function initialHTML(doc) {
     startTag("plaintext", {"id": "json"});
 }
 
+function keepThemeUpdated(win) {
+  let listener = function () {
+    let theme = Services.prefs.getCharPref("devtools.theme");
+    win.document.documentElement.className = "theme-" + theme;
+  };
+  Services.prefs.addObserver("devtools.theme", listener);
+  win.addEventListener("unload", function (event) {
+    Services.prefs.removeObserver("devtools.theme", listener);
+    win = null;
+  }, {once: true});
+}
+
 // Chrome <-> Content communication
 function onContentMessage(e) {
   // Do not handle events from different documents.
@@ -245,11 +340,8 @@ function onContentMessage(e) {
       break;
 
     case "save":
-      // The window ID is needed when the JSON Viewer is inside an iframe.
-      let windowID = win.QueryInterface(Ci.nsIInterfaceRequestor)
-        .getInterface(Ci.nsIDOMWindowUtils).outerWindowID;
       childProcessMessageManager.sendAsyncMessage(
-        "devtools:jsonview:save", {url: value, windowID: windowID});
+        "devtools:jsonview:save", value);
   }
 }
 
