@@ -8,32 +8,47 @@
 
 define(function (require, exports, module) {
   const { render } = require("devtools/client/shared/vendor/react-dom");
-  const { createFactories } = require("devtools/client/shared/components/reps/reps");
-  const { MainTabbedArea } = createFactories(require("./components/main-tabbed-area"));
+  const { createFactories } = require("devtools/client/shared/react-utils");
+  const { MainTabbedArea } = createFactories(require("./components/MainTabbedArea"));
+  const TreeViewClass = require("devtools/client/shared/components/tree/TreeView");
 
   const json = document.getElementById("json");
-  const headers = document.getElementById("headers");
+  const AUTO_EXPAND_MAX_SIZE = 100 * 1024;
+  const AUTO_EXPAND_MAX_LEVEL = 7;
 
-  let jsonData;
-
-  try {
-    jsonData = JSON.parse(json.textContent);
-  } catch (err) {
-    jsonData = err + "";
-  }
+  let prettyURL;
 
   // Application state object.
   let input = {
     jsonText: json.textContent,
     jsonPretty: null,
-    json: jsonData,
-    headers: JSON.parse(headers.textContent),
+    headers: JSONView.headers,
     tabActive: 0,
     prettified: false
   };
 
+  // Remove BOM.
+  if (input.jsonText.startsWith("\ufeff")) {
+    input.jsonText = input.jsonText.slice(1);
+  }
+
+  try {
+    input.json = JSON.parse(input.jsonText);
+  } catch (err) {
+    input.json = err;
+  }
+
+  // Expand the document by default if its size isn't bigger than 100KB.
+  if (!(input.json instanceof Error) && input.jsonText.length <= AUTO_EXPAND_MAX_SIZE) {
+    input.expandedNodes = TreeViewClass.getExpandedNodes(
+      input.json,
+      {maxLevel: AUTO_EXPAND_MAX_LEVEL}
+    );
+  } else {
+    input.expandedNodes = new Set();
+  }
+
   json.remove();
-  headers.remove();
 
   /**
    * Application actions/commands. This list implements all commands
@@ -45,7 +60,10 @@ define(function (require, exports, module) {
     },
 
     onSaveJson: function () {
-      dispatchEvent("save", input.prettified ? input.jsonPretty : input.jsonText);
+      if (input.prettified && !prettyURL) {
+        prettyURL = URL.createObjectURL(new window.Blob([input.jsonPretty]));
+      }
+      dispatchEvent("save", input.prettified ? prettyURL : null);
     },
 
     onCopyHeaders: function () {
@@ -57,11 +75,15 @@ define(function (require, exports, module) {
     },
 
     onPrettify: function (data) {
+      if (input.json instanceof Error) {
+        // Cannot prettify invalid JSON
+        return;
+      }
       if (input.prettified) {
         theApp.setState({jsonText: input.jsonText});
       } else {
         if (!input.jsonPretty) {
-          input.jsonPretty = JSON.stringify(jsonData, null, "  ");
+          input.jsonPretty = JSON.stringify(input.json, null, "  ");
         }
         theApp.setState({jsonText: input.jsonPretty});
       }
@@ -95,17 +117,9 @@ define(function (require, exports, module) {
   let content = document.getElementById("content");
   let theApp = render(MainTabbedArea(input), content);
 
-  let onResize = event => {
-    window.document.body.style.height = window.innerHeight + "px";
-    window.document.body.style.width = window.innerWidth + "px";
-  };
-
-  window.addEventListener("resize", onResize);
-  onResize();
-
   // Send notification event to the window. Can be useful for
   // tests as well as extensions.
   let event = new CustomEvent("JSONViewInitialized", {});
-  window.jsonViewInitialized = true;
+  JSONView.initialized = true;
   window.dispatchEvent(event);
 });
