@@ -3,31 +3,36 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+/* global requirejs */
+
+"use strict";
 
 /**
  * RequireJS configuration for JSON Viewer.
  *
- * ReactJS library is shared among DevTools. Both, the minified (production)
- * version and developer versions of the library are available.
+ * ReactJS library is shared among DevTools. The minified (production) version
+ * of the library is always available, and is used by default.
  *
  * In order to use the developer version you need to specify the following
  * in your .mozconfig (see also bug 1181646):
  * ac_add_options --enable-debug-js-modules
  *
- * The path mapping uses paths fallback (a feature supported by RequireJS)
- * See also: http://requirejs.org/docs/api.html#pathsfallbacks
- *
  * React module ID is using exactly the same (relative) path as the rest
  * of the code base, so it's consistent and modules can be easily reused.
  */
 require.config({
-  baseUrl: ".",
+  baseUrl: "resource://devtools-client-jsonview/",
   paths: {
-    "devtools/client/shared": "resource://devtools/client/shared",
-    "devtools/client/shared/vendor/react": [
-      "resource://devtools/client/shared/vendor/react-dev",
-      "resource://devtools/client/shared/vendor/react"
-    ],
+    "devtools/client/shared": "resource://devtools-client-shared",
+    "devtools/shared": "resource://devtools/shared",
+    "devtools/client/shared/vendor/react":
+      JSONView.debug
+      ? "resource://devtools-client-shared/vendor/react-dev"
+      : "resource://devtools-client-shared/vendor/react",
+    "devtools/client/shared/vendor/react-dom":
+      JSONView.debug
+      ? "resource://devtools-client-shared/vendor/react-dom-dev"
+      : "resource://devtools-client-shared/vendor/react-dom",
   }
 });
 
