@@ -2,9 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+"use strict";
+
 const { assert } = require("devtools/shared/DevToolsUtils");
 const { appinfo } = require("Services");
-const { DOM: dom, createClass, createFactory, PropTypes } = require("devtools/client/shared/vendor/react");
+const { DOM: dom, Component, createFactory, PropTypes } = require("devtools/client/shared/vendor/react");
 const { connect } = require("devtools/client/shared/vendor/react-redux");
 const { censusDisplays, labelDisplays, treeMapDisplays, diffingState, viewState } = require("./constants");
 const { toggleRecordingAllocationStacks } = require("./actions/allocations");
@@ -43,37 +45,36 @@ const {
 } = require("./actions/snapshot");
 const { changeViewAndRefresh, popViewAndRefresh } = require("./actions/view");
 const { resizeShortestPaths } = require("./actions/sizes");
-const Toolbar = createFactory(require("./components/toolbar"));
-const List = createFactory(require("./components/list"));
-const SnapshotListItem = createFactory(require("./components/snapshot-list-item"));
-const Heap = createFactory(require("./components/heap"));
+const Toolbar = createFactory(require("./components/Toolbar"));
+const List = createFactory(require("./components/List"));
+const SnapshotListItem = createFactory(require("./components/SnapshotListItem"));
+const Heap = createFactory(require("./components/Heap"));
 const { app: appModel } = require("./models");
 
-const MemoryApp = createClass({
-  displayName: "MemoryApp",
+class MemoryApp extends Component {
+  static get propTypes() {
+    return appModel;
+  }
 
-  propTypes: appModel,
+  static get childContextTypes() {
+    return {
+      front: PropTypes.any,
+      heapWorker: PropTypes.any,
+      toolbox: PropTypes.any,
+    };
+  }
 
-  getDefaultProps() {
+  static get defaultProps() {
     return {};
-  },
+  }
 
-  componentDidMount() {
-    // Attach the keydown listener directly to the window. When an element that
-    // has the focus (such as a tree node) is removed from the DOM, the focus
-    // falls back to the body.
-    window.addEventListener("keydown", this.onKeyDown);
-  },
-
-  componentWillUnmount() {
-    window.removeEventListener("keydown", this.onKeyDown);
-  },
-
-  childContextTypes: {
-    front: PropTypes.any,
-    heapWorker: PropTypes.any,
-    toolbox: PropTypes.any,
-  },
+  constructor(props) {
+    super(props);
+    this.onKeyDown = this.onKeyDown.bind(this);
+    this._getCensusDisplays = this._getCensusDisplays.bind(this);
+    this._getLabelDisplays = this._getLabelDisplays.bind(this);
+    this._getTreeMapDisplays = this._getTreeMapDisplays.bind(this);
+  }
 
   getChildContext() {
     return {
@@ -81,7 +82,18 @@ const MemoryApp = createClass({
       heapWorker: this.props.heapWorker,
       toolbox: this.props.toolbox,
     };
-  },
+  }
+
+  componentDidMount() {
+    // Attach the keydown listener directly to the window. When an element that
+    // has the focus (such as a tree node) is removed from the DOM, the focus
+    // falls back to the body.
+    window.addEventListener("keydown", this.onKeyDown);
+  }
+
+  componentWillUnmount() {
+    window.removeEventListener("keydown", this.onKeyDown);
+  }
 
   onKeyDown(e) {
     let { snapshots, dispatch, heapWorker } = this.props;
@@ -104,7 +116,7 @@ const MemoryApp = createClass({
       let nextSnapshotId = snapshots[nextIndex].id;
       dispatch(selectSnapshotAndRefresh(heapWorker, nextSnapshotId));
     }
-  },
+  }
 
   _getCensusDisplays() {
     const customDisplays = getCustomCensusDisplays();
@@ -118,7 +130,7 @@ const MemoryApp = createClass({
       censusDisplays.allocationStack,
       censusDisplays.invertedAllocationStack,
     ].concat(custom);
-  },
+  }
 
   _getLabelDisplays() {
     const customDisplays = getCustomLabelDisplays();
@@ -131,7 +143,7 @@ const MemoryApp = createClass({
       labelDisplays.coarseType,
       labelDisplays.allocationStack,
     ].concat(custom);
-  },
+  }
 
   _getTreeMapDisplays() {
     const customDisplays = getCustomTreeMapDisplays();
@@ -143,7 +155,7 @@ const MemoryApp = createClass({
     return [
       treeMapDisplays.coarseType
     ].concat(custom);
-  },
+  }
 
   render() {
     let {
@@ -219,7 +231,8 @@ const MemoryApp = createClass({
           Heap({
             snapshot: selectedSnapshot,
             diffing,
-            onViewSourceInDebugger: frame => toolbox.viewSourceInDebugger(frame.source, frame.line),
+            onViewSourceInDebugger: frame =>
+              toolbox.viewSourceInDebugger(frame.source, frame.line),
             onSnapshotClick: () =>
               dispatch(takeSnapshotAndCensus(front, heapWorker)),
             onLoadMoreSiblings: lazyChildren =>
@@ -249,7 +262,8 @@ const MemoryApp = createClass({
                 dispatch(expandDiffingCensusNode(node));
               } else {
                 assert(selectedSnapshot && selectedSnapshot.census === census,
-                       "If not diffing, should be expanding on selected snapshot's census");
+                       "If not diffing, " +
+                       "should be expanding on selected snapshot's census");
                 dispatch(expandCensusNode(selectedSnapshot.id, node));
               }
             },
@@ -260,7 +274,8 @@ const MemoryApp = createClass({
                 dispatch(collapseDiffingCensusNode(node));
               } else {
                 assert(selectedSnapshot && selectedSnapshot.census === census,
-                       "If not diffing, should be collapsing on selected snapshot's census");
+                       "If not diffing, " +
+                       "should be collapsing on selected snapshot's census");
                 dispatch(collapseCensusNode(selectedSnapshot.id, node));
               }
             },
@@ -271,13 +286,15 @@ const MemoryApp = createClass({
                 dispatch(focusDiffingCensusNode(node));
               } else {
                 assert(selectedSnapshot && selectedSnapshot.census === census,
-                       "If not diffing, should be focusing on nodes in selected snapshot's census");
+                       "If not diffing, " +
+                       "should be focusing on nodes in selected snapshot's census");
                 dispatch(focusCensusNode(selectedSnapshot.id, node));
               }
             },
             onDominatorTreeExpand: node => {
               assert(view.state === viewState.DOMINATOR_TREE,
-                     "If expanding dominator tree nodes, should be in dominator tree view");
+                     "If expanding dominator tree nodes, " +
+                     "should be in dominator tree view");
               assert(selectedSnapshot, "...and we should have a selected snapshot");
               assert(selectedSnapshot.dominatorTree,
                      "...and that snapshot should have a dominator tree");
@@ -285,7 +302,8 @@ const MemoryApp = createClass({
             },
             onDominatorTreeCollapse: node => {
               assert(view.state === viewState.DOMINATOR_TREE,
-                     "If collapsing dominator tree nodes, should be in dominator tree view");
+                     "If collapsing dominator tree nodes, " +
+                     "should be in dominator tree view");
               assert(selectedSnapshot, "...and we should have a selected snapshot");
               assert(selectedSnapshot.dominatorTree,
                      "...and that snapshot should have a dominator tree");
@@ -293,7 +311,8 @@ const MemoryApp = createClass({
             },
             onDominatorTreeFocus: node => {
               assert(view.state === viewState.DOMINATOR_TREE,
-                     "If focusing dominator tree nodes, should be in dominator tree view");
+                     "If focusing dominator tree nodes, " +
+                     "should be in dominator tree view");
               assert(selectedSnapshot, "...and we should have a selected snapshot");
               assert(selectedSnapshot.dominatorTree,
                      "...and that snapshot should have a dominator tree");
@@ -308,14 +327,14 @@ const MemoryApp = createClass({
         )
       )
     );
-  },
-});
+  }
+}
 
 /**
  * Passed into react-redux's `connect` method that is called on store change
  * and passed to components.
  */
-function mapStateToProps (state) {
+function mapStateToProps(state) {
   return state;
 }
 
