@@ -1,5 +1,6 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
+
 "use strict";
 
 var { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;
@@ -7,7 +8,7 @@ var { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;
 var { BrowserLoader } = Cu.import("resource://devtools/client/shared/browser-loader.js", {});
 var { require } = BrowserLoader({
   baseURI: "resource://devtools/client/memory/",
-  window: this
+  window
 });
 var { Assert } = require("resource://testing-common/Assert.jsm");
 var Services = require("Services");
@@ -17,8 +18,9 @@ var EXPECTED_DTU_ASSERT_FAILURE_COUNT = 0;
 
 SimpleTest.registerCleanupFunction(function () {
   if (DevToolsUtils.assertionFailureCount !== EXPECTED_DTU_ASSERT_FAILURE_COUNT) {
-    ok(false, "Should have had the expected number of DevToolsUtils.assert() failures. Expected " +
-       EXPECTED_DTU_ASSERT_FAILURE_COUNT + ", got " + DevToolsUtils.assertionFailureCount);
+    ok(false, "Should have had the expected number of DevToolsUtils.assert() failures." +
+      "Expected " + EXPECTED_DTU_ASSERT_FAILURE_COUNT +
+      ", got " + DevToolsUtils.assertionFailureCount);
   }
 });
 
@@ -47,15 +49,15 @@ var models = require("devtools/client/memory/models");
 var Immutable = require("devtools/client/shared/vendor/immutable");
 var React = require("devtools/client/shared/vendor/react");
 var ReactDOM = require("devtools/client/shared/vendor/react-dom");
-var Heap = React.createFactory(require("devtools/client/memory/components/heap"));
-var CensusTreeItem = React.createFactory(require("devtools/client/memory/components/census-tree-item"));
-var DominatorTreeComponent = React.createFactory(require("devtools/client/memory/components/dominator-tree"));
-var DominatorTreeItem = React.createFactory(require("devtools/client/memory/components/dominator-tree-item"));
-var ShortestPaths = React.createFactory(require("devtools/client/memory/components/shortest-paths"));
-var TreeMap = React.createFactory(require("devtools/client/memory/components/tree-map"));
-var SnapshotListItem = React.createFactory(require("devtools/client/memory/components/snapshot-list-item"));
-var List = React.createFactory(require("devtools/client/memory/components/list"));
-var Toolbar = React.createFactory(require("devtools/client/memory/components/toolbar"));
+var Heap = React.createFactory(require("devtools/client/memory/components/Heap"));
+var CensusTreeItem = React.createFactory(require("devtools/client/memory/components/CensusTreeItem"));
+var DominatorTreeComponent = React.createFactory(require("devtools/client/memory/components/DominatorTree"));
+var DominatorTreeItem = React.createFactory(require("devtools/client/memory/components/DominatorTreeItem"));
+var ShortestPaths = React.createFactory(require("devtools/client/memory/components/ShortestPaths"));
+var TreeMap = React.createFactory(require("devtools/client/memory/components/TreeMap"));
+var SnapshotListItem = React.createFactory(require("devtools/client/memory/components/SnapshotListItem"));
+var List = React.createFactory(require("devtools/client/memory/components/List"));
+var Toolbar = React.createFactory(require("devtools/client/memory/components/Toolbar"));
 
 // All tests are asynchronous.
 SimpleTest.waitForExplicitFinish();
@@ -115,7 +117,9 @@ function makeTestDominatorTreeNode(opts, children) {
   }, opts);
 
   if (children && children.length) {
-    children.map(c => c.parentId = node.nodeId);
+    children.map(c => {
+      c.parentId = node.nodeId;
+    });
   }
 
   return node;
@@ -318,18 +322,6 @@ function renderComponent(element, container) {
   });
 }
 
-function setState(component, newState) {
-  return new Promise(resolve => {
-    component.setState(newState, onNextAnimationFrame(resolve));
-  });
-}
-
-function setProps(component, newProps) {
-  return new Promise(resolve => {
-    component.setProps(newProps, onNextAnimationFrame(resolve));
-  });
-}
-
 function dumpn(msg) {
   dump(`MEMORY-TEST: ${msg}\n`);
 }
