@@ -8,15 +8,18 @@
  */
 
 add_task(function* () {
-  let { L10N } = require("devtools/client/netmonitor/l10n");
-
   let { tab, monitor } = yield initNetMonitor(CONTENT_TYPE_WITHOUT_CACHE_URL);
   info("Starting test... ");
 
-  let { document, NetMonitorView } = monitor.panelWin;
-  let { RequestsMenu } = NetMonitorView;
+  let { document, store, windowRequire } = monitor.panelWin;
+  let Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
+  let { L10N } = windowRequire("devtools/client/netmonitor/src/utils/l10n");
+  let {
+    getDisplayedRequests,
+    getSortedRequests,
+  } = windowRequire("devtools/client/netmonitor/src/selectors/index");
 
-  RequestsMenu.lazyUpdate = false;
+  store.dispatch(Actions.batchEnable(false));
 
   let wait = waitForNetworkEvents(monitor, CONTENT_TYPE_WITHOUT_CACHE_REQUESTS);
   yield ContentTask.spawn(tab.linkedBrowser, {}, function* () {
@@ -24,53 +27,95 @@ add_task(function* () {
   });
   yield wait;
 
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(0),
-    "GET", CONTENT_TYPE_SJS + "?fmt=xml", {
+  for (let requestItem of document.querySelectorAll(".request-list-item")) {
+    let requestsListStatus = requestItem.querySelector(".requests-list-status");
+    requestItem.scrollIntoView();
+    EventUtils.sendMouseEvent({ type: "mouseover" }, requestsListStatus);
+    yield waitUntil(() => requestsListStatus.title);
+  }
+
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(0),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=xml",
+    {
       status: 200,
       statusText: "OK",
       type: "xml",
       fullMimeType: "text/xml; charset=utf-8",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 42),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(1),
-    "GET", CONTENT_TYPE_SJS + "?fmt=css", {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(1),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=css",
+    {
       status: 200,
       statusText: "OK",
       type: "css",
       fullMimeType: "text/css; charset=utf-8",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 34),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(2),
-    "GET", CONTENT_TYPE_SJS + "?fmt=js", {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(2),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=js",
+    {
       status: 200,
       statusText: "OK",
       type: "js",
       fullMimeType: "application/javascript; charset=utf-8",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 34),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(3),
-    "GET", CONTENT_TYPE_SJS + "?fmt=json", {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(3),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=json",
+    {
       status: 200,
       statusText: "OK",
       type: "json",
       fullMimeType: "application/json; charset=utf-8",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 29),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(4),
-    "GET", CONTENT_TYPE_SJS + "?fmt=bogus", {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(4),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=bogus",
+    {
       status: 404,
       statusText: "Not Found",
       type: "html",
       fullMimeType: "text/html; charset=utf-8",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 24),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(5),
-    "GET", TEST_IMAGE, {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(5),
+    "GET",
+    TEST_IMAGE,
+    {
       fuzzyUrl: true,
       status: 200,
       statusText: "OK",
@@ -78,52 +123,50 @@ add_task(function* () {
       fullMimeType: "image/png",
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 580),
       time: true
-    });
-  verifyRequestItemTarget(RequestsMenu, RequestsMenu.getItemAtIndex(6),
-    "GET", CONTENT_TYPE_SJS + "?fmt=gzip", {
+    }
+  );
+  verifyRequestItemTarget(
+    document,
+    getDisplayedRequests(store.getState()),
+    getSortedRequests(store.getState()).get(6),
+    "GET",
+    CONTENT_TYPE_SJS + "?fmt=gzip",
+    {
       status: 200,
       statusText: "OK",
       type: "plain",
       fullMimeType: "text/plain",
-      transferred: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 73),
+      transferred: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 324),
       size: L10N.getFormatStrWithNumbers("networkMenu.sizeKB", 10.73),
       time: true
-    });
+    }
+  );
 
-  wait = waitForDOM(document, "#response-tabpanel");
-  EventUtils.sendMouseEvent({ type: "mousedown" },
-    document.getElementById("details-pane-toggle"));
-  EventUtils.sendMouseEvent({ type: "mousedown" },
-    document.querySelectorAll("#details-pane tab")[3]);
-  yield wait;
-
-  RequestsMenu.selectedIndex = -1;
-
-  yield selectIndexAndWaitForEditor(0);
+  yield selectIndexAndWaitForSourceEditor(0);
   yield testResponseTab("xml");
 
-  yield selectIndexAndWaitForEditor(1);
+  yield selectIndexAndWaitForSourceEditor(1);
   yield testResponseTab("css");
 
-  yield selectIndexAndWaitForEditor(2);
+  yield selectIndexAndWaitForSourceEditor(2);
   yield testResponseTab("js");
 
   yield selectIndexAndWaitForJSONView(3);
   yield testResponseTab("json");
 
-  yield selectIndexAndWaitForEditor(4);
+  yield selectIndexAndWaitForSourceEditor(4);
   yield testResponseTab("html");
 
   yield selectIndexAndWaitForImageView(5);
   yield testResponseTab("png");
 
-  yield selectIndexAndWaitForEditor(6);
+  yield selectIndexAndWaitForSourceEditor(6);
   yield testResponseTab("gzip");
 
   yield teardown(monitor);
 
   function* testResponseTab(type) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[3];
+    let tabpanel = document.querySelector("#response-panel");
 
     function checkVisibility(box) {
       is(tabpanel.querySelector(".response-error-header") === null,
@@ -133,8 +176,8 @@ add_task(function* () {
       is(jsonView.textContent !== L10N.getStr("jsonScopeName"),
         box != "json",
         "The response json view doesn't display");
-      is(tabpanel.querySelector(".editor-mount") === null,
-        box != "textarea",
+      is(tabpanel.querySelector(".CodeMirror-code") === null,
+        (box !== "textarea" && box !== "json"),
         "The response editor doesn't display");
       is(tabpanel.querySelector(".response-image-box") === null,
         box != "image",
@@ -145,8 +188,7 @@ add_task(function* () {
       case "xml": {
         checkVisibility("textarea");
 
-        let editor = tabpanel.querySelector(".editor-mount iframe");
-        let text = editor.contentDocument .querySelector(".CodeMirror-line").textContent;
+        let text = document.querySelector(".CodeMirror-line").textContent;
 
         is(text, "<label value='greeting'>Hello XML!</label>",
           "The text shown in the source editor is incorrect for the xml request.");
@@ -155,8 +197,7 @@ add_task(function* () {
       case "css": {
         checkVisibility("textarea");
 
-        let editor = tabpanel.querySelector(".editor-mount iframe");
-        let text = editor.contentDocument.querySelector(".CodeMirror-line").textContent;
+        let text = document.querySelector(".CodeMirror-line").textContent;
 
         is(text, "body:pre { content: 'Hello CSS!' }",
           "The text shown in the source editor is incorrect for the css request.");
@@ -165,8 +206,7 @@ add_task(function* () {
       case "js": {
         checkVisibility("textarea");
 
-        let editor = tabpanel.querySelector(".editor-mount iframe");
-        let text = editor.contentDocument.querySelector(".CodeMirror-line").textContent;
+        let text = document.querySelector(".CodeMirror-line").textContent;
 
         is(text, "function() { return 'Hello JS!'; }",
           "The text shown in the source editor is incorrect for the js request.");
@@ -175,8 +215,8 @@ add_task(function* () {
       case "json": {
         checkVisibility("json");
 
-        is(tabpanel.querySelectorAll(".tree-section").length, 1,
-          "There should be 1 tree sections displayed in this tabpanel.");
+        is(tabpanel.querySelectorAll(".tree-section").length, 2,
+          "There should be 2 tree sections displayed in this tabpanel.");
         is(tabpanel.querySelectorAll(".empty-notice").length, 0,
           "The empty notice should not be displayed in this tabpanel.");
 
@@ -192,14 +232,13 @@ add_task(function* () {
         is(labels[0].textContent, "greeting",
           "The first json property name was incorrect.");
         is(values[0].textContent,
-          "\"Hello JSON!\"", "The first json property value was incorrect.");
+          "Hello JSON!", "The first json property value was incorrect.");
         break;
       }
       case "html": {
         checkVisibility("textarea");
 
-        let editor = document.querySelector(".editor-mount iframe");
-        let text = editor.contentDocument.querySelector(".CodeMirror-line").textContent;
+        let text = document.querySelector(".CodeMirror-line").textContent;
 
         is(text, "<blink>Not Found</blink>",
           "The text shown in the source editor is incorrect for the html request.");
@@ -222,8 +261,7 @@ add_task(function* () {
       case "gzip": {
         checkVisibility("textarea");
 
-        let editor = tabpanel.querySelector(".editor-mount iframe");
-        let text = editor.contentDocument.querySelector(".CodeMirror-line").textContent;
+        let text = document.querySelector(".CodeMirror-line").textContent;
 
         is(text, new Array(1000).join("Hello gzip!"),
           "The text shown in the source editor is incorrect for the gzip request.");
@@ -232,32 +270,31 @@ add_task(function* () {
     }
   }
 
-  function* selectIndexAndWaitForEditor(index) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[3];
-    let editor = tabpanel.querySelector(".editor-mount iframe");
+  function* selectIndexAndWaitForSourceEditor(index) {
+    let editor = document.querySelector("#response-panel .CodeMirror-code");
     if (!editor) {
-      let waitDOM = waitForDOM(tabpanel, ".editor-mount iframe");
-      RequestsMenu.selectedIndex = index;
-      [editor] = yield waitDOM;
-      yield once(editor, "DOMContentLoaded");
+      let waitDOM = waitForDOM(document, "#response-panel .CodeMirror-code");
+      EventUtils.sendMouseEvent({ type: "mousedown" },
+        document.querySelectorAll(".request-list-item")[index]);
+      document.querySelector("#response-tab").click();
+      yield waitDOM;
     } else {
-      RequestsMenu.selectedIndex = index;
+      EventUtils.sendMouseEvent({ type: "mousedown" },
+        document.querySelectorAll(".request-list-item")[index]);
     }
-
-    yield waitForDOM(editor.contentDocument, ".CodeMirror-code");
   }
 
   function* selectIndexAndWaitForJSONView(index) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[3];
+    let tabpanel = document.querySelector("#response-panel");
     let waitDOM = waitForDOM(tabpanel, ".treeTable");
-    RequestsMenu.selectedIndex = index;
+    store.dispatch(Actions.selectRequestByIndex(index));
     yield waitDOM;
   }
 
   function* selectIndexAndWaitForImageView(index) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[3];
+    let tabpanel = document.querySelector("#response-panel");
     let waitDOM = waitForDOM(tabpanel, ".response-image");
-    RequestsMenu.selectedIndex = index;
+    store.dispatch(Actions.selectRequestByIndex(index));
     let [imageNode] = yield waitDOM;
     yield once(imageNode, "load");
   }
