@@ -9,15 +9,15 @@
  */
 
 add_task(function* () {
-  let { L10N } = require("devtools/client/netmonitor/l10n");
+  let { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
 
   let { tab, monitor } = yield initNetMonitor(POST_RAW_WITH_HEADERS_URL);
   info("Starting test... ");
 
-  let { document, NetMonitorView } = monitor.panelWin;
-  let { RequestsMenu } = NetMonitorView;
+  let { document, store, windowRequire } = monitor.panelWin;
+  let Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
 
-  RequestsMenu.lazyUpdate = false;
+  store.dispatch(Actions.batchEnable(false));
 
   let wait = waitForNetworkEvents(monitor, 0, 1);
   yield ContentTask.spawn(tab.linkedBrowser, {}, function* () {
@@ -26,14 +26,14 @@ add_task(function* () {
   yield wait;
 
   // Wait for all tree view updated by react
-  wait = waitForDOM(document, ".properties-view .treeTable");
-  EventUtils.sendMouseEvent({ type: "mousedown" },
-    document.getElementById("details-pane-toggle"));
-  EventUtils.sendMouseEvent({ type: "mousedown" },
-    document.querySelectorAll("#details-pane tab")[0]);
+  wait = waitForDOM(document, "#headers-panel");
+  EventUtils.sendMouseEvent({ type: "click" },
+    document.querySelector(".network-details-panel-toggle"));
+  EventUtils.sendMouseEvent({ type: "click" },
+    document.querySelector("#headers-tab"));
   yield wait;
 
-  let tabpanel = document.querySelectorAll("#details-pane tabpanel")[0];
+  let tabpanel = document.querySelector("#headers-panel");
 
   is(tabpanel.querySelectorAll(".tree-section .treeLabel").length, 3,
     "There should be 3 header sections displayed in this tabpanel.");
@@ -50,20 +50,20 @@ add_task(function* () {
 
   is(labels[labels.length - 2].textContent, "content-type",
     "The first request header name was incorrect.");
-  is(values[values.length - 2].textContent, "\"application/x-www-form-urlencoded\"",
+  is(values[values.length - 2].textContent, "application/x-www-form-urlencoded",
     "The first request header value was incorrect.");
   is(labels[labels.length - 1].textContent, "custom-header",
     "The second request header name was incorrect.");
-  is(values[values.length - 1].textContent, "\"hello world!\"",
+  is(values[values.length - 1].textContent, "hello world!",
     "The second request header value was incorrect.");
 
   // Wait for all tree sections updated by react
-  wait = waitForDOM(document, "#params-tabpanel .tree-section");
-  EventUtils.sendMouseEvent({ type: "mousedown" },
-    document.querySelectorAll("#details-pane tab")[2]);
+  wait = waitForDOM(document, "#params-panel .tree-section");
+  EventUtils.sendMouseEvent({ type: "click" },
+    document.querySelector("#params-tab"));
   yield wait;
 
-  tabpanel = document.querySelectorAll("#details-pane tabpanel")[2];
+  tabpanel = document.querySelector("#params-panel");
 
   ok(tabpanel.querySelector(".treeTable"),
     "The params tree view should be displayed.");
@@ -79,10 +79,10 @@ add_task(function* () {
   values = tabpanel
     .querySelectorAll("tr:not(.tree-section) .treeValueCell .objectBox");
 
-  is(labels[0].textContent, "foo", "The first payload param name was incorrect.");
-  is(values[0].textContent, "\"bar\"", "The first payload param value was incorrect.");
-  is(labels[1].textContent, "baz", "The second payload param name was incorrect.");
-  is(values[1].textContent, "\"123\"", "The second payload param value was incorrect.");
+  is(labels[0].textContent, "baz", "The first payload param name was incorrect.");
+  is(values[0].textContent, "123", "The first payload param value was incorrect.");
+  is(labels[1].textContent, "foo", "The second payload param name was incorrect.");
+  is(values[1].textContent, "bar", "The second payload param value was incorrect.");
 
   return teardown(monitor);
 });
