@@ -8,23 +8,29 @@
  */
 
 add_task(function* () {
-  let { L10N } = require("devtools/client/netmonitor/l10n");
+  let { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
 
   let { tab, monitor } = yield initNetMonitor(STATUS_CODES_URL);
 
   info("Starting test... ");
 
-  let { document, EVENTS, NetMonitorView } = monitor.panelWin;
-  let { RequestsMenu, NetworkDetails } = NetMonitorView;
-  let requestItems = [];
+  let { document, store, windowRequire } = monitor.panelWin;
+  let Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
+  let {
+    getDisplayedRequests,
+    getSortedRequests,
+  } = windowRequire("devtools/client/netmonitor/src/selectors/index");
 
-  RequestsMenu.lazyUpdate = false;
+  store.dispatch(Actions.batchEnable(false));
+
+  let requestItems = [];
 
   const REQUEST_DATA = [
     {
       // request #0
       method: "GET",
       uri: STATUS_CODES_SJS + "?sts=100",
+      correctUri: STATUS_CODES_SJS + "?sts=100",
       details: {
         status: 101,
         statusText: "Switching Protocols",
@@ -37,7 +43,8 @@ add_task(function* () {
     {
       // request #1
       method: "GET",
-      uri: STATUS_CODES_SJS + "?sts=200",
+      uri: STATUS_CODES_SJS + "?sts=200#doh",
+      correctUri: STATUS_CODES_SJS + "?sts=200",
       details: {
         status: 202,
         statusText: "Created",
@@ -51,6 +58,7 @@ add_task(function* () {
       // request #2
       method: "GET",
       uri: STATUS_CODES_SJS + "?sts=300",
+      correctUri: STATUS_CODES_SJS + "?sts=300",
       details: {
         status: 303,
         statusText: "See Other",
@@ -64,6 +72,7 @@ add_task(function* () {
       // request #3
       method: "GET",
       uri: STATUS_CODES_SJS + "?sts=400",
+      correctUri: STATUS_CODES_SJS + "?sts=400",
       details: {
         status: 404,
         statusText: "Not Found",
@@ -77,6 +86,7 @@ add_task(function* () {
       // request #4
       method: "GET",
       uri: STATUS_CODES_SJS + "?sts=500",
+      correctUri: STATUS_CODES_SJS + "?sts=500",
       details: {
         status: 501,
         statusText: "Not Implemented",
@@ -96,25 +106,39 @@ add_task(function* () {
 
   info("Performing tests");
   yield verifyRequests();
-  yield testTab(0, testSummary);
+  yield testTab(0, testHeaders);
   yield testTab(2, testParams);
 
   return teardown(monitor);
 
   /**
    * A helper that verifies all requests show the correct information and caches
-   * RequestsMenu items to requestItems array.
+   * request list items to requestItems array.
    */
   function* verifyRequests() {
+    let requestListItems = document.querySelectorAll(".request-list-item");
+    for (let requestItem of requestListItems) {
+      requestItem.scrollIntoView();
+      let requestsListStatus = requestItem.querySelector(".requests-list-status");
+      EventUtils.sendMouseEvent({ type: "mouseover" }, requestsListStatus);
+      yield waitUntil(() => requestsListStatus.title);
+    }
+
     info("Verifying requests contain correct information.");
     let index = 0;
     for (let request of REQUEST_DATA) {
-      let item = RequestsMenu.getItemAtIndex(index);
+      let item = getSortedRequests(store.getState()).get(index);
       requestItems[index] = item;
 
       info("Verifying request #" + index);
-      yield verifyRequestItemTarget(RequestsMenu, item,
-        request.method, request.uri, request.details);
+      yield verifyRequestItemTarget(
+        document,
+        getDisplayedRequests(store.getState()),
+        item,
+        request.method,
+        request.uri,
+        request.details
+      );
 
       index++;
     }
@@ -125,64 +149,66 @@ add_task(function* () {
    * all requests to the given test function.
    *
    * @param Number tabIdx
-   *               The index of NetworkDetails tab to activate.
+   *               The index of tab to activate.
    * @param Function testFn(requestItem)
    *        A function that should perform all necessary tests. It's called once
    *        for every item of REQUEST_DATA with that item being selected in the
    *        NetworkMonitor.
    */
   function* testTab(tabIdx, testFn) {
-    info("Testing tab #" + tabIdx);
-    EventUtils.sendMouseEvent({ type: "mousedown" },
-          document.querySelectorAll("#details-pane tab")[tabIdx]);
-
     let counter = 0;
     for (let item of REQUEST_DATA) {
-      info("Waiting tab #" + tabIdx + " to update with request #" + counter);
-      yield chooseRequest(counter);
-
-      info("Tab updated. Performing checks");
-      yield testFn(item);
+      info("Testing tab #" + tabIdx + " to update with request #" + counter);
+      yield testFn(item, counter);
 
       counter++;
     }
   }
 
   /**
-   * A function that tests "Summary" contains correct information.
+   * A function that tests "Headers" panel contains correct information.
    */
-  function* testSummary(data) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[0];
+  function* testHeaders(data, index) {
+    EventUtils.sendMouseEvent({ type: "mousedown" },
+      document.querySelectorAll(".request-list-item")[index]);
 
-    let { method, uri, details: { status, statusText } } = data;
-    let summaryValues = tabpanel.querySelectorAll(".tabpanel-summary-value.textbox-input");
-    is(summaryValues[0].value, uri, "The url summary value is incorrect.");
+    let panel = document.querySelector("#headers-panel");
+    let summaryValues = panel.querySelectorAll(".tabpanel-summary-value.textbox-input");
+    let { method, correctUri, details: { status, statusText } } = data;
+
+    is(summaryValues[0].value, correctUri,
+      "The url summary value is incorrect.");
     is(summaryValues[1].value, method, "The method summary value is incorrect.");
-    is(tabpanel.querySelector(".requests-menu-status-icon").dataset.code, status,
+    is(panel.querySelector(".requests-list-status-icon").dataset.code, status,
       "The status summary code is incorrect.");
     is(summaryValues[3].value, status + " " + statusText,
       "The status summary value is incorrect.");
   }
 
   /**
-   * A function that tests "Params" tab contains correct information.
+   * A function that tests "Params" panel contains correct information.
    */
-  function* testParams(data) {
-    let tabpanel = document.querySelectorAll("#details-pane tabpanel")[2];
-    let statusParamValue = data.uri.split("=").pop();
-    let statusParamShownValue = "\"" + statusParamValue + "\"";
-    let treeSections = tabpanel.querySelectorAll(".tree-section");
+  function* testParams(data, index) {
+    EventUtils.sendMouseEvent({ type: "mousedown" },
+      document.querySelectorAll(".request-list-item")[index]);
+    EventUtils.sendMouseEvent({ type: "click" },
+      document.querySelector("#params-tab"));
+
+    let panel = document.querySelector("#params-panel");
+    // Bug 1414981 - Request URL should not show #hash
+    let statusParamValue = data.uri.split("=").pop().split("#")[0];
+    let treeSections = panel.querySelectorAll(".tree-section");
 
     is(treeSections.length, 1,
-      "There should be 1 param section displayed in this tabpanel.");
-    is(tabpanel.querySelectorAll("tr:not(.tree-section).treeRow").length, 1,
-      "There should be 1 param row displayed in this tabpanel.");
-    is(tabpanel.querySelectorAll(".empty-notice").length, 0,
-      "The empty notice should not be displayed in this tabpanel.");
+      "There should be 1 param section displayed in this panel.");
+    is(panel.querySelectorAll("tr:not(.tree-section).treeRow").length, 1,
+      "There should be 1 param row displayed in this panel.");
+    is(panel.querySelectorAll(".empty-notice").length, 0,
+      "The empty notice should not be displayed in this panel.");
 
-    let labels = tabpanel
+    let labels = panel
       .querySelectorAll("tr:not(.tree-section) .treeLabelCell .treeLabel");
-    let values = tabpanel
+    let values = panel
       .querySelectorAll("tr:not(.tree-section) .treeValueCell .objectBox");
 
     is(treeSections[0].querySelector(".treeLabel").textContent,
@@ -190,23 +216,12 @@ add_task(function* () {
       "The params scope doesn't have the correct title.");
 
     is(labels[0].textContent, "sts", "The param name was incorrect.");
-    is(values[0].textContent, statusParamShownValue, "The param value was incorrect.");
+    is(values[0].textContent, statusParamValue, "The param value was incorrect.");
 
-    ok(tabpanel.querySelector(".treeTable"),
+    ok(panel.querySelector(".treeTable"),
       "The request params tree view should be displayed.");
-    is(tabpanel.querySelector(".editor-mount") === null,
+    is(panel.querySelector(".editor-mount") === null,
       true,
       "The request post data editor should be hidden.");
   }
-
-  /**
-   * A helper that clicks on a specified request and returns a promise resolved
-   * when NetworkDetails has been populated with the data of the given request.
-   */
-  function chooseRequest(index) {
-    let onTabUpdated = monitor.panelWin.once(EVENTS.TAB_UPDATED);
-    let target = getItemTarget(RequestsMenu, requestItems[index]);
-    EventUtils.sendMouseEvent({ type: "mousedown" }, target);
-    return onTabUpdated;
-  }
 });
