@@ -5,7 +5,7 @@
 
 /* globals window, document, PerformanceView, ToolbarView, RecordingsView, DetailsView */
 
-/* exported Cc, Ci, Cu, Cr, loader */
+/* exported Cc, Ci, Cu, Cr, loader, Promise */
 var { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;
 var BrowserLoaderModule = {};
 Cu.import("resource://devtools/client/shared/browser-loader.js", BrowserLoaderModule);
@@ -18,6 +18,10 @@ var { Task } = require("devtools/shared/task");
 var { Heritage, ViewHelpers, WidgetMethods, setNamedTimeout, clearNamedTimeout } = require("devtools/client/shared/widgets/view-helpers");
 var { PrefObserver } = require("devtools/client/shared/prefs");
 
+// Use privileged promise in panel documents to prevent having them to freeze
+// during toolbox destruction. See bug 1402779.
+var Promise = require("Promise");
+
 // Events emitted by various objects in the panel.
 var EVENTS = require("devtools/client/performance/events");
 Object.defineProperty(this, "EVENTS", {
@@ -40,7 +44,8 @@ var RecordingListItem = React.createFactory(require("devtools/client/performance
 
 var Services = require("Services");
 var promise = require("promise");
-var EventEmitter = require("devtools/shared/event-emitter");
+const defer = require("devtools/shared/defer");
+var EventEmitter = require("devtools/shared/old-event-emitter");
 var DevToolsUtils = require("devtools/shared/DevToolsUtils");
 var flags = require("devtools/shared/flags");
 var system = require("devtools/shared/system");
@@ -235,11 +240,6 @@ var PerformanceController = {
    * @return Promise:boolean
    */
   canCurrentlyRecord: Task.async(function* () {
-    // If we're testing the legacy front, the performance actor will exist,
-    // with `canCurrentlyRecord` method; this ensures we test the legacy path.
-    if (gFront.LEGACY_FRONT) {
-      return true;
-    }
     let hasActor = yield gTarget.hasActor("performance");
     if (!hasActor) {
       return true;
@@ -296,7 +296,7 @@ var PerformanceController = {
    *
    * @param PerformanceRecording recording
    *        The model that holds the recording data.
-   * @param nsILocalFile file
+   * @param nsIFile file
    *        The file to stream the data into.
    */
   exportRecording: Task.async(function* (_, recording, file) {
@@ -340,7 +340,7 @@ var PerformanceController = {
    * Loads a recording from a file, adding it to the recordings list. Emits
    * `EVENTS.RECORDING_IMPORTED` when the file was loaded.
    *
-   * @param nsILocalFile file
+   * @param nsIFile file
    *        The file to import the data from.
    */
   importRecording: Task.async(function* (_, file) {
@@ -517,18 +517,17 @@ var PerformanceController = {
    * @return {object}
    */
   getMultiprocessStatus: function () {
-    // If testing, set both supported and enabled to true so we
-    // have realtime rendering tests in non-e10s. This function is
-    // overridden wholesale in tests when we want to test multiprocess support
+    // If testing, set enabled to true so we have realtime rendering tests
+    // in non-e10s. This function is overridden wholesale in tests
+    // when we want to test multiprocess support
     // specifically.
     if (flags.testing) {
-      return { supported: true, enabled: true };
+      return { enabled: true };
     }
-    let supported = system.constants.E10S_TESTING_ONLY;
     // This is only checked on tool startup -- requires a restart if
     // e10s subsequently enabled.
     let enabled = this._e10s;
-    return { supported, enabled };
+    return { enabled };
   },
 
   /**
@@ -540,7 +539,7 @@ var PerformanceController = {
    * @return {Promise}
    */
   waitForStateChangeOnRecording: Task.async(function* (recording, expectedState) {
-    let deferred = promise.defer();
+    let deferred = defer();
     this.on(EVENTS.RECORDING_STATE_CHANGE, function handler(state, model) {
       if (state === expectedState && model === recording) {
         this.off(EVENTS.RECORDING_STATE_CHANGE, handler);
@@ -556,12 +555,9 @@ var PerformanceController = {
    * if e10s is not possible on the platform. If e10s is on, no attribute is set.
    */
   _setMultiprocessAttributes: function () {
-    let { enabled, supported } = this.getMultiprocessStatus();
-    if (!enabled && supported) {
+    let { enabled } = this.getMultiprocessStatus();
+    if (!enabled) {
       $("#performance-view").setAttribute("e10s", "disabled");
-    } else if (!enabled && !supported) {
-      // Could be a chance where the directive goes away yet e10s is still on
-      $("#performance-view").setAttribute("e10s", "unsupported");
     }
   },
 
