@@ -4,21 +4,25 @@
 "use strict";
 
 var Cu = Components.utils;
-const loaders = Cu.import("resource://gre/modules/commonjs/toolkit/loader.js", {});
+const loaders = Cu.import("resource://devtools/shared/base-loader.js", {});
 const { devtools } = Cu.import("resource://devtools/shared/Loader.jsm", {});
 const { joinURI } = devtools.require("devtools/shared/path");
 const { assert } = devtools.require("devtools/shared/DevToolsUtils");
-const Services = devtools.require("Services");
 const { AppConstants } = devtools.require("resource://gre/modules/AppConstants.jsm");
 
 const BROWSER_BASED_DIRS = [
   "resource://devtools/client/inspector/boxmodel",
+  "resource://devtools/client/inspector/changes",
   "resource://devtools/client/inspector/computed",
+  "resource://devtools/client/inspector/events",
+  "resource://devtools/client/inspector/flexbox",
+  "resource://devtools/client/inspector/fonts",
   "resource://devtools/client/inspector/grids",
   "resource://devtools/client/inspector/layout",
   "resource://devtools/client/jsonview",
-  "resource://devtools/client/shared/vendor",
+  "resource://devtools/client/shared/source-map",
   "resource://devtools/client/shared/redux",
+  "resource://devtools/client/shared/vendor",
 ];
 
 const COMMON_LIBRARY_DIRS = [
@@ -35,10 +39,6 @@ const COMMON_LIBRARY_DIRS = [
 const browserBasedDirsRegExp =
   /^resource\:\/\/devtools\/client\/\S*\/components\//;
 
-function clearCache() {
-  Services.obs.notifyObservers(null, "startupcache-invalidate", null);
-}
-
 /*
  * Create a loader to be used in a browser environment. This evaluates
  * modules in their own environment, but sets window (the normal
@@ -99,17 +99,20 @@ function BrowserLoaderBuilder({ baseURI, window, useOnlyShared, commonLibRequire
 
   const loaderOptions = devtools.require("@loader/options");
   const dynamicPaths = {};
-  const componentProxies = new Map();
 
   if (AppConstants.DEBUG || AppConstants.DEBUG_JS_MODULES) {
     dynamicPaths["devtools/client/shared/vendor/react"] =
       "resource://devtools/client/shared/vendor/react-dev";
+    dynamicPaths["devtools/client/shared/vendor/react-dom"] =
+      "resource://devtools/client/shared/vendor/react-dom-dev";
+    dynamicPaths["devtools/client/shared/vendor/react-dom-server"] =
+      "resource://devtools/client/shared/vendor/react-dom-server-dev";
   }
 
   const opts = {
-    id: "browser-loader",
     sharedGlobal: true,
     sandboxPrototype: window,
+    sandboxName: "DevTools (UI loader)",
     paths: Object.assign({}, dynamicPaths, loaderOptions.paths),
     invisibleToDebugger: loaderOptions.invisibleToDebugger,
     requireHook: (id, require) => {
@@ -165,39 +168,6 @@ function BrowserLoaderBuilder({ baseURI, window, useOnlyShared, commonLibRequire
     }
   };
 
-  if (Services.prefs.getBoolPref("devtools.loader.hotreload")) {
-    opts.loadModuleHook = (module, require) => {
-      const { uri, exports } = module;
-
-      if (exports.prototype &&
-          exports.prototype.isReactComponent) {
-        const { createProxy, getForceUpdate } =
-              require("devtools/client/shared/vendor/react-proxy");
-        const React = require("devtools/client/shared/vendor/react");
-
-        if (!componentProxies.get(uri)) {
-          const proxy = createProxy(exports);
-          componentProxies.set(uri, proxy);
-          module.exports = proxy.get();
-        } else {
-          const proxy = componentProxies.get(uri);
-          const instances = proxy.update(exports);
-          instances.forEach(getForceUpdate(React));
-          module.exports = proxy.get();
-        }
-      }
-      return exports;
-    };
-    const watcher = devtools.require("devtools/client/shared/devtools-file-watcher");
-    let onFileChanged = (_, relativePath, path) => {
-      this.hotReloadFile(componentProxies, "resource://devtools/" + relativePath);
-    };
-    watcher.on("file-changed", onFileChanged);
-    window.addEventListener("unload", () => {
-      watcher.off("file-changed", onFileChanged);
-    });
-  }
-
   const mainModule = loaders.Module(baseURI, joinURI(baseURI, "main.js"));
   this.loader = loaders.Loader(opts);
   this.require = loaders.Require(this.loader, mainModule);
@@ -224,20 +194,6 @@ BrowserLoaderBuilder.prototype = {
           ? this.require(module)[property]
           : this.require(module || property);
     });
-  },
-
-  hotReloadFile: function (componentProxies, fileURI) {
-    if (fileURI.match(/\.js$/)) {
-      // Test for React proxy components
-      const proxy = componentProxies.get(fileURI);
-      if (proxy) {
-        // Remove the old module and re-require the new one; the require
-        // hook in the loader will take care of the rest
-        delete this.loader.modules[fileURI];
-        clearCache();
-        this.require(fileURI);
-      }
-    }
   }
 };
 
