@@ -21,21 +21,20 @@ const TEST_HOST = "mochi.test:8888";
  * @param {Window} win The window to add the tab to (default: current window).
  * @return a promise that resolves to the tab object when the url is loaded
  */
-var addTab = function(url, win) {
+var addTab = function (url, win) {
   info("Adding a new tab with URL: '" + url + "'");
-  let def = promise.defer();
 
-  let targetWindow = win || window;
-  let targetBrowser = targetWindow.gBrowser;
+  return new Promise(resolve => {
+    let targetWindow = win || window;
+    let targetBrowser = targetWindow.gBrowser;
 
-  let tab = targetBrowser.selectedTab = targetBrowser.addTab(url);
-  targetBrowser.selectedBrowser.addEventListener("load", function onload() {
-    targetBrowser.selectedBrowser.removeEventListener("load", onload, true);
-    info("URL '" + url + "' loading complete");
-    def.resolve(tab);
-  }, true);
-
-  return def.promise;
+    let tab = targetBrowser.selectedTab = targetBrowser.addTab(url);
+    BrowserTestUtils.browserLoaded(targetBrowser.selectedBrowser)
+      .then(function () {
+        info("URL '" + url + "' loading complete");
+        resolve(tab);
+      });
+  });
 };
 
 /**
@@ -43,20 +42,18 @@ var addTab = function(url, win) {
  * @param {String} url The url to be loaded in the current tab.
  * @return a promise that resolves when the page has fully loaded.
  */
-var navigateTo = Task.async(function* (url) {
+var navigateTo = function (url) {
   info(`Navigating to ${url}`);
   let browser = gBrowser.selectedBrowser;
 
-  let navigating = promise.defer();
-  browser.addEventListener("load", function onload() {
-    browser.removeEventListener("load", onload, true);
-    navigating.resolve();
-  }, true);
+  return new Promise(resolve => {
+    browser.addEventListener("load", function () {
+      resolve();
+    }, {capture: true, once: true});
 
-  browser.loadURI(url);
-
-  yield navigating.promise;
-});
+    browser.loadURI(url);
+  });
+};
 
 var navigateToAndWaitForStyleSheets = Task.async(function* (url, ui) {
   let onReset = ui.once("stylesheets-reset");
@@ -85,6 +82,10 @@ var openStyleEditor = Task.async(function* (tab) {
   let panel = toolbox.getPanel("styleeditor");
   let ui = panel.UI;
 
+  // The stylesheet list appears with an animation. Let this animation finish.
+  let animations = ui._root.getAnimations({subtree: true});
+  yield Promise.all(animations.map(a => a.finished));
+
   return { toolbox, panel, ui };
 });
 
@@ -112,7 +113,7 @@ var openStyleEditorForURL = Task.async(function* (url, win) {
  */
 var getComputedStyleProperty = function* (args) {
   return yield ContentTask.spawn(gBrowser.selectedBrowser, args,
-    function({selector, pseudo, name}) {
+    function ({selector, pseudo, name}) {
       let element = content.document.querySelector(selector);
       let style = content.getComputedStyle(element, pseudo);
       return style.getPropertyValue(name);
