@@ -3,37 +3,34 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const React = require("devtools/client/shared/vendor/react");
-
-// Shortcuts
-const DOM = React.DOM;
-const PropTypes = React.PropTypes;
+const { Component, DOM, PropTypes } =
+  require("devtools/client/shared/vendor/react");
 
 /**
  * This template renders list of parameters within a group.
  * It's essentially a list of name + value pairs.
  */
-var NetInfoParams = React.createClass({
-  propTypes: {
-    params: PropTypes.arrayOf(PropTypes.shape({
-      name: PropTypes.string.isRequired,
-      value: PropTypes.string.isRequired
-    })).isRequired,
-  },
-
-  displayName: "NetInfoParams",
+class NetInfoParams extends Component {
+  static get propTypes() {
+    return {
+      params: PropTypes.arrayOf(PropTypes.shape({
+        name: PropTypes.string.isRequired,
+        value: PropTypes.string.isRequired
+      })).isRequired,
+    };
+  }
 
   render() {
     let params = this.props.params || [];
 
-    params.sort(function(a, b) {
+    params.sort(function (a, b) {
       return a.name > b.name ? 1 : -1;
     });
 
     let rows = [];
-    params.forEach(param => {
+    params.forEach((param, index) => {
       rows.push(
-        DOM.tr({key: param.name},
+        DOM.tr({key: index},
           DOM.td({className: "netInfoParamName"},
             DOM.span({title: param.name}, param.name)
           ),
@@ -52,7 +49,7 @@ var NetInfoParams = React.createClass({
       )
     );
   }
-});
+}
 
 // Exports from this module
 module.exports = NetInfoParams;
