@@ -6,14 +6,13 @@
 const expect = require("expect");
 const {
   renderComponent,
-  shallowRenderComponent
 } = require("devtools/client/webconsole/new-console-output/test/helpers");
 
 // Components under test.
-const { MessageContainer } = require("devtools/client/webconsole/new-console-output/components/message-container");
-const ConsoleApiCall = require("devtools/client/webconsole/new-console-output/components/message-types/console-api-call");
-const EvaluationResult = require("devtools/client/webconsole/new-console-output/components/message-types/evaluation-result");
-const PageError = require("devtools/client/webconsole/new-console-output/components/message-types/page-error");
+const { MessageContainer, getMessageComponent } = require("devtools/client/webconsole/new-console-output/components/MessageContainer");
+const ConsoleApiCall = require("devtools/client/webconsole/new-console-output/components/message-types/ConsoleApiCall");
+const EvaluationResult = require("devtools/client/webconsole/new-console-output/components/message-types/EvaluationResult");
+const PageError = require("devtools/client/webconsole/new-console-output/components/message-types/PageError");
 
 // Test fakes.
 const { stubPreparedMessages } = require("devtools/client/webconsole/new-console-output/test/fixtures/stubs/index");
@@ -22,7 +21,10 @@ const serviceContainer = require("devtools/client/webconsole/new-console-output/
 describe("MessageContainer component:", () => {
   it("pipes data to children as expected", () => {
     const message = stubPreparedMessages.get("console.log('foobar', 'test')");
-    const rendered = renderComponent(MessageContainer, {message, serviceContainer});
+    const rendered = renderComponent(MessageContainer, {
+      getMessage: () => message,
+      serviceContainer
+    });
 
     expect(rendered.textContent.includes("foobar")).toBe(true);
   });
@@ -50,11 +52,7 @@ describe("MessageContainer component:", () => {
 
     messageTypes.forEach(info => {
       const { component, message } = info;
-      const rendered = shallowRenderComponent(MessageContainer, {
-        message,
-        serviceContainer,
-      });
-      expect(rendered.type).toBe(component);
+      expect(getMessageComponent(message)).toBe(component);
     });
   });
 });
