@@ -11,13 +11,19 @@
 const TEST_URI = "http://example.com/browser/devtools/client/webconsole/" +
                  "test/test-autocomplete-in-stackframe.html";
 
+// Force the old debugger UI since it's directly used (see Bug 1301705)
+Services.prefs.setBoolPref("devtools.debugger.new-debugger-frontend", false);
+registerCleanupFunction(function* () {
+  Services.prefs.clearUserPref("devtools.debugger.new-debugger-frontend");
+});
+
 var gStackframes;
-registerCleanupFunction(function() {
+registerCleanupFunction(function () {
   gStackframes = null;
 });
 
 requestLongerTimeout(2);
-add_task(function*() {
+add_task(function* () {
   yield loadTab(TEST_URI);
   let hud = yield openConsole();
   yield testCompletion(hud);
@@ -37,15 +43,15 @@ function* testCompletion(hud) {
 
   let newItems = popup.getItems();
   ok(newItems.length > 0, "'document.title.' gave a list of suggestions");
-  ok(newItems.some(function(item) {
-       return item.label == "substr";
-     }), "autocomplete results do contain substr");
-  ok(newItems.some(function(item) {
-       return item.label == "toLowerCase";
-     }), "autocomplete results do contain toLowerCase");
-  ok(newItems.some(function(item) {
-       return item.label == "strike";
-     }), "autocomplete results do contain strike");
+  ok(newItems.some(function (item) {
+    return item.label == "substr";
+  }), "autocomplete results do contain substr");
+  ok(newItems.some(function (item) {
+    return item.label == "toLowerCase";
+  }), "autocomplete results do contain toLowerCase");
+  ok(newItems.some(function (item) {
+    return item.label == "strike";
+  }), "autocomplete results do contain strike");
 
   // Test if 'f' gives 'foo1' but not 'foo2' or 'foo3'
   input.value = "f";
@@ -56,24 +62,24 @@ function* testCompletion(hud) {
 
   newItems = popup.getItems();
   ok(newItems.length > 0, "'f' gave a list of suggestions");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1";
-     }), "autocomplete results do contain foo1");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1Obj";
-     }), "autocomplete results do contain foo1Obj");
-  ok(newItems.every(function(item) {
-       return item.label != "foo2";
-     }), "autocomplete results do not contain foo2");
-  ok(newItems.every(function(item) {
-       return item.label != "foo2Obj";
-     }), "autocomplete results do not contain foo2Obj");
-  ok(newItems.every(function(item) {
-       return item.label != "foo3";
-     }), "autocomplete results do not contain foo3");
-  ok(newItems.every(function(item) {
-       return item.label != "foo3Obj";
-     }), "autocomplete results do not contain foo3Obj");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1";
+  }), "autocomplete results do contain foo1");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1Obj";
+  }), "autocomplete results do contain foo1Obj");
+  ok(newItems.every(function (item) {
+    return item.label != "foo2";
+  }), "autocomplete results do not contain foo2");
+  ok(newItems.every(function (item) {
+    return item.label != "foo2Obj";
+  }), "autocomplete results do not contain foo2Obj");
+  ok(newItems.every(function (item) {
+    return item.label != "foo3";
+  }), "autocomplete results do not contain foo3");
+  ok(newItems.every(function (item) {
+    return item.label != "foo3Obj";
+  }), "autocomplete results do not contain foo3Obj");
 
   // Test if 'foo1Obj.' gives 'prop1' and 'prop2'
   input.value = "foo1Obj.";
@@ -83,12 +89,12 @@ function* testCompletion(hud) {
   });
 
   newItems = popup.getItems();
-  ok(!newItems.every(function(item) {
-       return item.label != "prop1";
-     }), "autocomplete results do contain prop1");
-  ok(!newItems.every(function(item) {
-       return item.label != "prop2";
-     }), "autocomplete results do contain prop2");
+  ok(!newItems.every(function (item) {
+    return item.label != "prop1";
+  }), "autocomplete results do contain prop1");
+  ok(!newItems.every(function (item) {
+    return item.label != "prop2";
+  }), "autocomplete results do contain prop2");
 
   // Test if 'foo1Obj.prop2.' gives 'prop21'
   input.value = "foo1Obj.prop2.";
@@ -98,9 +104,9 @@ function* testCompletion(hud) {
   });
 
   newItems = popup.getItems();
-  ok(!newItems.every(function(item) {
-       return item.label != "prop21";
-     }), "autocomplete results do contain prop21");
+  ok(!newItems.every(function (item) {
+    return item.label != "prop21";
+  }), "autocomplete results do contain prop21");
 
   info("Opening Debugger");
   let dbg = yield openDebugger();
@@ -121,24 +127,24 @@ function* testCompletion(hud) {
 
   newItems = popup.getItems();
   ok(newItems.length > 0, "'f' gave a list of suggestions");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo3";
-     }), "autocomplete results do contain foo3");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo3Obj";
-     }), "autocomplete results do contain foo3Obj");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1";
-     }), "autocomplete results do contain foo1");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1Obj";
-     }), "autocomplete results do contain foo1Obj");
-  ok(newItems.every(function(item) {
-       return item.label != "foo2";
-     }), "autocomplete results do not contain foo2");
-  ok(newItems.every(function(item) {
-       return item.label != "foo2Obj";
-     }), "autocomplete results do not contain foo2Obj");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo3";
+  }), "autocomplete results do contain foo3");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo3Obj";
+  }), "autocomplete results do contain foo3Obj");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1";
+  }), "autocomplete results do contain foo1");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1Obj";
+  }), "autocomplete results do contain foo1Obj");
+  ok(newItems.every(function (item) {
+    return item.label != "foo2";
+  }), "autocomplete results do not contain foo2");
+  ok(newItems.every(function (item) {
+    return item.label != "foo2Obj";
+  }), "autocomplete results do not contain foo2Obj");
 
   yield openDebugger();
 
@@ -156,24 +162,24 @@ function* testCompletion(hud) {
 
   newItems = popup.getItems();
   ok(newItems.length > 0, "'f' gave a list of suggestions");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo2";
-     }), "autocomplete results do contain foo2");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo2Obj";
-     }), "autocomplete results do contain foo2Obj");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1";
-     }), "autocomplete results do contain foo1");
-  ok(!newItems.every(function(item) {
-       return item.label != "foo1Obj";
-     }), "autocomplete results do contain foo1Obj");
-  ok(newItems.every(function(item) {
-       return item.label != "foo3";
-     }), "autocomplete results do not contain foo3");
-  ok(newItems.every(function(item) {
-       return item.label != "foo3Obj";
-     }), "autocomplete results do not contain foo3Obj");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo2";
+  }), "autocomplete results do contain foo2");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo2Obj";
+  }), "autocomplete results do contain foo2Obj");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1";
+  }), "autocomplete results do contain foo1");
+  ok(!newItems.every(function (item) {
+    return item.label != "foo1Obj";
+  }), "autocomplete results do contain foo1Obj");
+  ok(newItems.every(function (item) {
+    return item.label != "foo3";
+  }), "autocomplete results do not contain foo3");
+  ok(newItems.every(function (item) {
+    return item.label != "foo3Obj";
+  }), "autocomplete results do not contain foo3Obj");
 
   // Test if 'foo2Obj.' gives 'prop1'
   input.value = "foo2Obj.";
@@ -183,9 +189,9 @@ function* testCompletion(hud) {
   });
 
   newItems = popup.getItems();
-  ok(!newItems.every(function(item) {
-       return item.label != "prop1";
-     }), "autocomplete results do contain prop1");
+  ok(!newItems.every(function (item) {
+    return item.label != "prop1";
+  }), "autocomplete results do contain prop1");
 
   // Test if 'foo2Obj.prop1.' gives 'prop11'
   input.value = "foo2Obj.prop1.";
@@ -195,9 +201,9 @@ function* testCompletion(hud) {
   });
 
   newItems = popup.getItems();
-  ok(!newItems.every(function(item) {
-       return item.label != "prop11";
-     }), "autocomplete results do contain prop11");
+  ok(!newItems.every(function (item) {
+    return item.label != "prop11";
+  }), "autocomplete results do contain prop11");
 
   // Test if 'foo2Obj.prop1.prop11.' gives suggestions for a string
   // i.e. 'length'
@@ -208,9 +214,9 @@ function* testCompletion(hud) {
   });
 
   newItems = popup.getItems();
-  ok(!newItems.every(function(item) {
-       return item.label != "length";
-     }), "autocomplete results do contain length");
+  ok(!newItems.every(function (item) {
+    return item.label != "length";
+  }), "autocomplete results do contain length");
 
   // Test if 'foo1Obj[0].' throws no errors.
   input.value = "foo2Obj[0].";
@@ -232,7 +238,7 @@ function pauseDebugger(aResult) {
     thread.addOneTimeListener("framesadded", resolve);
 
     info("firstCall()");
-    ContentTask.spawn(gBrowser.selectedBrowser, {}, function*() {
+    ContentTask.spawn(gBrowser.selectedBrowser, {}, function* () {
       content.wrappedJSObject.firstCall();
     });
   });
