@@ -18,8 +18,8 @@
 
 const TEST_URI = "https://example.com/browser/devtools/client/webconsole/" +
                  "test/test-mixedcontent-securityerrors.html";
-const LEARN_MORE_URI = "https://developer.mozilla.org/docs/Security/" +
-                       "MixedContent";
+const LEARN_MORE_URI = "https://developer.mozilla.org/docs/Web/Security/" +
+                       "Mixed_content" + DOCS_GA_PARAMS;
 
 add_task(function* () {
   yield pushPrefEnv();
@@ -33,14 +33,14 @@ add_task(function* () {
     messages: [
       {
         name: "Logged blocking mixed active content",
-        text: "Blocked loading mixed active content \"http://example.com/\"",
+        text: "Blocked loading mixed active content \u201chttp://example.com/\u201d",
         category: CATEGORY_SECURITY,
         severity: SEVERITY_ERROR,
         objects: true,
       },
       {
         name: "Logged blocking mixed passive content - image",
-        text: "Blocked loading mixed active content \"http://example.com/\"",
+        text: "Blocked loading mixed active content \u201chttp://example.com/\u201d",
         category: CATEGORY_SECURITY,
         severity: SEVERITY_ERROR,
         objects: true,
@@ -56,11 +56,13 @@ add_task(function* () {
 });
 
 function pushPrefEnv() {
-  let deferred = promise.defer();
+  let deferred = defer();
   let options = {
     "set": [
       ["security.mixed_content.block_active_content", true],
-      ["security.mixed_content.block_display_content", true]
+      ["security.mixed_content.block_display_content", true],
+      ["security.mixed_content.use_hsts", false],
+      ["security.mixed_content.send_hsts_priming", false],
     ]
   };
   SpecialPowers.pushPrefEnv(options, deferred.resolve);
@@ -68,7 +70,7 @@ function pushPrefEnv() {
 }
 
 function mixedContentOverrideTest2(hud, browser) {
-  let deferred = promise.defer();
+  let deferred = defer();
   let {gIdentityHandler} = browser.ownerGlobal;
   ok(gIdentityHandler._identityBox.classList.contains("mixedActiveBlocked"),
     "Mixed Active Content state appeared on identity box");
@@ -80,7 +82,7 @@ function mixedContentOverrideTest2(hud, browser) {
       {
         name: "Logged blocking mixed active content",
         text: "Loading mixed (insecure) active content " +
-              "\"http://example.com/\" on a secure page",
+              "\u201chttp://example.com/\u201d on a secure page",
         category: CATEGORY_SECURITY,
         severity: SEVERITY_WARNING,
         objects: true,
@@ -88,14 +90,14 @@ function mixedContentOverrideTest2(hud, browser) {
       {
         name: "Logged blocking mixed passive content - image",
         text: "Loading mixed (insecure) display content" +
-          " \"http://example.com/tests/image/test/mochitest/blue.png\"" +
+          " \u201chttp://example.com/tests/image/test/mochitest/blue.png\u201d" +
           " on a secure page",
         category: CATEGORY_SECURITY,
         severity: SEVERITY_WARNING,
         objects: true,
       },
     ],
-  }).then(msgs => deferred.resolve(msgs), Cu.reportError);
+  }).then(msgs => deferred.resolve(msgs), console.error);
 
   return deferred.promise;
 }
