@@ -4,23 +4,30 @@
 
 "use strict";
 
-const {Cc, Ci} = require("chrome");
+const {Ci} = require("chrome");
 const promise = require("promise");
 const protocol = require("devtools/shared/protocol");
 const {LongStringActor} = require("devtools/server/actors/string");
-const {getDefinedGeometryProperties} = require("devtools/server/actors/highlighters/geometry-editor");
-const {parseNamedDeclarations} = require("devtools/shared/css/parsing-utils");
-const {isCssPropertyKnown} = require("devtools/server/actors/css-properties");
 const {Task} = require("devtools/shared/task");
-const events = require("sdk/event/core");
 
 // This will also add the "stylesheet" actor type for protocol.js to recognize
-const {UPDATE_PRESERVING_RULES, UPDATE_GENERAL} = require("devtools/server/actors/stylesheets");
+
 const {pageStyleSpec, styleRuleSpec, ELEMENT_STYLE} = require("devtools/shared/specs/styles");
 
-loader.lazyGetter(this, "CssLogic", () => require("devtools/server/css-logic").CssLogic);
-loader.lazyGetter(this, "SharedCssLogic", () => require("devtools/shared/inspector/css-logic"));
-loader.lazyGetter(this, "DOMUtils", () => Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils));
+loader.lazyRequireGetter(this, "CssLogic", "devtools/server/css-logic", true);
+loader.lazyRequireGetter(this, "SharedCssLogic", "devtools/shared/inspector/css-logic");
+loader.lazyRequireGetter(this, "getDefinedGeometryProperties",
+  "devtools/server/actors/highlighters/geometry-editor", true);
+loader.lazyRequireGetter(this, "isCssPropertyKnown",
+  "devtools/server/actors/css-properties", true);
+loader.lazyRequireGetter(this, "parseNamedDeclarations",
+  "devtools/shared/css/parsing-utils", true);
+loader.lazyRequireGetter(this, "UPDATE_PRESERVING_RULES",
+  "devtools/server/actors/stylesheets", true);
+loader.lazyRequireGetter(this, "UPDATE_GENERAL",
+  "devtools/server/actors/stylesheets", true);
+
+loader.lazyServiceGetter(this, "DOMUtils", "@mozilla.org/inspector/dom-utils;1", "inIDOMUtils");
 
 loader.lazyGetter(this, "PSEUDO_ELEMENTS", () => {
   return DOMUtils.getCSSPseudoElementNames();
@@ -67,8 +74,8 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
     this.onFrameUnload = this.onFrameUnload.bind(this);
     this.onStyleSheetAdded = this.onStyleSheetAdded.bind(this);
 
-    events.on(this.inspector.tabActor, "will-navigate", this.onFrameUnload);
-    events.on(this.inspector.tabActor, "stylesheet-added", this.onStyleSheetAdded);
+    this.inspector.tabActor.on("will-navigate", this.onFrameUnload);
+    this.inspector.tabActor.on("stylesheet-added", this.onStyleSheetAdded);
 
     this._styleApplied = this._styleApplied.bind(this);
     this._watchedSheets = new Set();
@@ -79,8 +86,8 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
       return;
     }
     protocol.Actor.prototype.destroy.call(this);
-    events.off(this.inspector.tabActor, "will-navigate", this.onFrameUnload);
-    events.off(this.inspector.tabActor, "stylesheet-added", this.onStyleSheetAdded);
+    this.inspector.tabActor.off("will-navigate", this.onFrameUnload);
+    this.inspector.tabActor.off("stylesheet-added", this.onStyleSheetAdded);
     this.inspector = null;
     this.walker = null;
     this.refMap = null;
@@ -124,7 +131,7 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
     // the keyframe cache.
     this.cssLogic.reset();
     if (kind === UPDATE_GENERAL) {
-      events.emit(this, "stylesheet-updated", styleSheet);
+      this.emit("stylesheet-updated", styleSheet);
     }
   },
 
@@ -437,6 +444,7 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
    *   `inherited`: Include styles inherited from parent nodes.
    *   `matchedSelectors`: Include an array of specific selectors that
    *     caused this rule to match its node.
+   *   `skipPseudo`: Exclude styles applied to pseudo elements of the provided node.
    */
   getApplied: Task.async(function* (node, options) {
     if (!node) {
@@ -537,7 +545,7 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
         });
 
     // Now any pseudos.
-    if (showElementStyles) {
+    if (showElementStyles && !options.skipPseudo) {
       for (let readPseudo of PSEUDO_ELEMENTS) {
         this._getElementRules(bindingElement, readPseudo, inherited, options)
             .forEach(oneRule => {
@@ -636,6 +644,7 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
    *   `inherited`: Include styles inherited from parent nodes.
    *   `matchedSelectors`: Include an array of specific selectors that
    *     caused this rule to match its node.
+   *   `skipPseudo`: Exclude styles applied to pseudo elements of the provided node.
    * @param array entries
    *   List of appliedstyle objects that lists the rules that apply to the
    *   node. If adding a new rule to the stylesheet, only the new rule entry
@@ -771,6 +780,10 @@ var PageStyleActor = protocol.ActorClassWithSpec(pageStyleSpec, {
     let style = CssLogic.getComputedStyle(node.rawNode);
     for (let prop of [
       "position",
+      "top",
+      "right",
+      "bottom",
+      "left",
       "margin-top",
       "margin-right",
       "margin-bottom",
@@ -1001,15 +1014,13 @@ var StyleRuleActor = protocol.ActorClassWithSpec(styleRuleSpec, {
   },
 
   getDocument: function (sheet) {
-    let document;
-
-    if (sheet.ownerNode instanceof Ci.nsIDOMHTMLDocument) {
-      document = sheet.ownerNode;
-    } else {
-      document = sheet.ownerNode.ownerDocument;
+    if (sheet.ownerNode) {
+      return sheet.ownerNode instanceof Ci.nsIDOMHTMLDocument ?
+             sheet.ownerNode : sheet.ownerNode.ownerDocument;
+    } else if (sheet.parentStyleSheet) {
+      return this.getDocument(sheet.parentStyleSheet);
     }
-
-    return document;
+    throw (new Error("Failed trying to get the document of an invalid stylesheet"));
   },
 
   toString: function () {
@@ -1100,8 +1111,14 @@ var StyleRuleActor = protocol.ActorClassWithSpec(styleRuleSpec, {
       let declarations = parseNamedDeclarations(isCssPropertyKnown,
                                                 form.authoredText || form.cssText,
                                                 true);
+
+      // We need to grab CSS from the window, since calling supports() on the
+      // one from the current global will fail due to not being an HTML global.
+      let CSS = this.pageStyle.inspector.tabActor.window.CSS;
       form.declarations = declarations.map(decl => {
-        decl.isValid = DOMUtils.cssPropertyIsValid(decl.name, decl.value);
+        // Use the 1-arg CSS.supports() call so that we also accept !important
+        // in the value.
+        decl.isValid = CSS.supports(`${decl.name}:${decl.value}`);
         return decl;
       });
     }
@@ -1117,7 +1134,7 @@ var StyleRuleActor = protocol.ActorClassWithSpec(styleRuleSpec, {
    * @param {Number} column the new column number
    */
   _notifyLocationChanged: function (line, column) {
-    events.emit(this, "location-changed", line, column);
+    this.emit("location-changed", line, column);
   },
 
   /**
@@ -1513,9 +1530,9 @@ function getFontPreviewData(font, doc, options) {
   // Get the correct preview text measurements and set the canvas dimensions
   ctx.font = fontValue;
   ctx.fillStyle = fillStyle;
-  let textWidth = ctx.measureText(previewText).width;
+  let textWidth = Math.round(ctx.measureText(previewText).width);
 
-  canvas.width = textWidth * 2 + FONT_PREVIEW_OFFSET * 2;
+  canvas.width = textWidth * 2 + FONT_PREVIEW_OFFSET * 4;
   canvas.height = previewFontSize * 3;
 
   // we have to reset these after changing the canvas size
