@@ -9,12 +9,13 @@
 var { Ci } = require("chrome");
 var Services = require("Services");
 var promise = require("promise");
+const defer = require("devtools/shared/defer");
 var { DebuggerServer } = require("devtools/server/main");
 var DevToolsUtils = require("devtools/shared/DevToolsUtils");
 
 loader.lazyRequireGetter(this, "RootActor", "devtools/server/actors/root", true);
 loader.lazyRequireGetter(this, "BrowserAddonActor", "devtools/server/actors/addon", true);
-loader.lazyRequireGetter(this, "WebExtensionActor", "devtools/server/actors/webextension", true);
+loader.lazyRequireGetter(this, "WebExtensionParentActor", "devtools/server/actors/webextension-parent", true);
 loader.lazyRequireGetter(this, "WorkerActorList", "devtools/server/actors/worker-list", true);
 loader.lazyRequireGetter(this, "ServiceWorkerRegistrationActorList", "devtools/server/actors/worker-list", true);
 loader.lazyRequireGetter(this, "ProcessActorList", "devtools/server/actors/process", true);
@@ -327,7 +328,7 @@ BrowserTabList.prototype.getTab = function ({ outerWindowID, tabId }) {
     // First look for in-process frames with this ID
     let window = Services.wm.getOuterWindowWithId(outerWindowID);
     // Safety check to prevent debugging top level window via getTab
-    if (window instanceof Ci.nsIDOMChromeWindow) {
+    if (window && window.isChromeWindow) {
       return promise.reject({
         error: "forbidden",
         message: "Window with outerWindowID '" + outerWindowID + "' is chrome"
@@ -355,7 +356,8 @@ BrowserTabList.prototype.getTab = function ({ outerWindowID, tabId }) {
   } else if (typeof tabId == "number") {
     // Tabs OOP
     for (let browser of this._getBrowsers()) {
-      if (browser.frameLoader.tabParent &&
+      if (browser.frameLoader &&
+          browser.frameLoader.tabParent &&
           browser.frameLoader.tabParent.tabId === tabId) {
         return this._getActorForBrowser(browser);
       }
@@ -671,7 +673,7 @@ DevToolsUtils.makeInfallible(function (window) {
    * a nsIWindowMediatorListener's onCloseWindow hook (bug 873589), so
    * handle the close in a different tick.
    */
-  Services.tm.currentThread.dispatch(DevToolsUtils.makeInfallible(() => {
+  Services.tm.dispatchToMainThread(DevToolsUtils.makeInfallible(() => {
     /*
      * Scan the entire map for actors representing tabs that were in this
      * top-level window, and exit them.
@@ -682,7 +684,7 @@ DevToolsUtils.makeInfallible(function (window) {
         this._handleActorClose(actor, browser);
       }
     }
-  }, "BrowserTabList.prototype.onCloseWindow's delayed body"), 0);
+  }, "BrowserTabList.prototype.onCloseWindow's delayed body"));
 }, "BrowserTabList.prototype.onCloseWindow");
 
 exports.BrowserTabList = BrowserTabList;
@@ -700,6 +702,7 @@ function BrowserTabActor(connection, browser) {
   this._conn = connection;
   this._browser = browser;
   this._form = null;
+  this.exited = false;
 }
 
 BrowserTabActor.prototype = {
@@ -712,7 +715,7 @@ BrowserTabActor.prototype = {
           message: "Tab destroyed while performing a BrowserTabActor update"
         });
       }
-      this._form = null;
+      this.exit();
     };
     let connect = DebuggerServer.connectToChild(this._conn, this._browser, onDestroy);
     return connect.then(form => {
@@ -722,7 +725,7 @@ BrowserTabActor.prototype = {
   },
 
   get _tabbrowser() {
-    if (typeof this._browser.getTabBrowser == "function") {
+    if (this._browser && typeof this._browser.getTabBrowser == "function") {
       return this._browser.getTabBrowser();
     }
     return null;
@@ -739,8 +742,8 @@ BrowserTabActor.prototype = {
     // If the child happens to be crashed/close/detach, it won't have _form set,
     // so only request form update if some code is still listening on the other
     // side.
-    if (this._form) {
-      this._deferredUpdate = promise.defer();
+    if (!this.exited) {
+      this._deferredUpdate = defer();
       let onFormUpdate = msg => {
         // There may be more than just one childtab.js up and running
         if (this._form.actor != msg.json.actor) {
@@ -764,7 +767,7 @@ BrowserTabActor.prototype = {
    */
   get title() {
     // On Fennec, we can check the session store data for zombie tabs
-    if (this._browser.__SS_restore) {
+    if (this._browser && this._browser.__SS_restore) {
       let sessionStore = this._browser.__SS_data;
       // Get the last selected entry
       let entry = sessionStore.entries[sessionStore.index - 1];
@@ -788,7 +791,7 @@ BrowserTabActor.prototype = {
    */
   get url() {
     // On Fennec, we can check the session store data for zombie tabs
-    if (this._browser.__SS_restore) {
+    if (this._browser && this._browser.__SS_restore) {
       let sessionStore = this._browser.__SS_data;
       // Get the last selected entry
       let entry = sessionStore.entries[sessionStore.index - 1];
@@ -813,6 +816,8 @@ BrowserTabActor.prototype = {
 
   exit() {
     this._browser = null;
+    this._form = null;
+    this.exited = true;
   },
 };
 
@@ -825,13 +830,13 @@ function BrowserAddonList(connection) {
 }
 
 BrowserAddonList.prototype.getList = function () {
-  let deferred = promise.defer();
+  let deferred = defer();
   AddonManager.getAllAddons((addons) => {
     for (let addon of addons) {
       let actor = this._actorByAddonId.get(addon.id);
       if (!actor) {
         if (addon.isWebExtension) {
-          actor = new WebExtensionActor(this._connection, addon);
+          actor = new WebExtensionParentActor(this._connection, addon);
         } else {
           actor = new BrowserAddonActor(this._connection, addon);
         }
@@ -839,8 +844,10 @@ BrowserAddonList.prototype.getList = function () {
         this._actorByAddonId.set(addon.id, actor);
       }
     }
+
     deferred.resolve([...this._actorByAddonId].map(([_, actor]) => actor));
   });
+
   return deferred.promise;
 };
 
