@@ -17,7 +17,6 @@ try {
     const DevToolsUtils = require("devtools/shared/DevToolsUtils");
     const { dumpn } = DevToolsUtils;
     const { DebuggerServer, ActorPool } = require("devtools/server/main");
-    const { ContentActor } = require("devtools/server/actors/childtab");
 
     if (!DebuggerServer.initialized) {
       DebuggerServer.init();
@@ -34,17 +33,36 @@ try {
 
       let mm = msg.target;
       let prefix = msg.data.prefix;
+      let addonId = msg.data.addonId;
 
-      let conn = DebuggerServer.connectToParent(prefix, mm);
-      conn.parentMessageManager = mm;
-      connections.set(prefix, conn);
+      // Using the JS debugger causes problems when we're trying to
+      // schedule those zone groups across different threads. Calling
+      // blockThreadedExecution causes Gecko to switch to a simpler
+      // single-threaded model until unblockThreadedExecution is
+      // called later. We cannot start the debugger until the callback
+      // passed to blockThreadedExecution has run, signaling that
+      // we're running single-threaded.
+      Cu.blockThreadedExecution(() => {
+        let conn = DebuggerServer.connectToParent(prefix, mm);
+        conn.parentMessageManager = mm;
+        connections.set(prefix, conn);
 
-      let actor = new ContentActor(conn, chromeGlobal, prefix);
-      let actorPool = new ActorPool(conn);
-      actorPool.addActor(actor);
-      conn.addActorPool(actorPool);
+        let actor;
 
-      sendAsyncMessage("debug:actor", {actor: actor.form(), prefix: prefix});
+        if (addonId) {
+          const { WebExtensionChildActor } = require("devtools/server/actors/webextension");
+          actor = new WebExtensionChildActor(conn, chromeGlobal, prefix, addonId);
+        } else {
+          const { ContentActor } = require("devtools/server/actors/childtab");
+          actor = new ContentActor(conn, chromeGlobal, prefix);
+        }
+
+        let actorPool = new ActorPool(conn);
+        actorPool.addActor(actor);
+        conn.addActorPool(actorPool);
+
+        sendAsyncMessage("debug:actor", {actor: actor.form(), prefix: prefix});
+      });
     });
 
     addMessageListener("debug:connect", onConnect);
@@ -91,6 +109,8 @@ try {
         return;
       }
 
+      Cu.unblockThreadedExecution();
+
       removeMessageListener("debug:disconnect", onDisconnect);
       // Call DebuggerServerConnection.close to destroy all child actors. It should end up
       // calling DebuggerServerConnection.onClosed that would actually cleanup all actor
