@@ -14,7 +14,8 @@
 
 "use strict";
 
-const {CSS_ANGLEUNIT} = require("devtools/shared/css/properties-db");
+loader.lazyRequireGetter(this, "CSS_ANGLEUNIT",
+  "devtools/shared/css/properties-db", true);
 
 const promise = require("promise");
 const {getCSSLexer} = require("devtools/shared/css/lexer");
@@ -248,6 +249,28 @@ function getEmptyDeclaration() {
           colonOffsets: false};
 }
 
+/**
+ * Like trim, but only trims CSS-allowed whitespace.
+ */
+function cssTrim(str) {
+  let match = /^[ \t\r\n\f]*(.*?)[ \t\r\n\f]*$/.exec(str);
+  if (match) {
+    return match[1];
+  }
+  return str;
+}
+
+/**
+ * Like trimRight, but only trims CSS-allowed whitespace.
+ */
+function cssTrimRight(str) {
+  let match = /^(.*?)[ \t\r\n\f]*$/.exec(str);
+  if (match) {
+    return match[1];
+  }
+  return str;
+}
+
 /**
  * A helper function that does all the parsing work for
  * parseDeclarations.  This is separate because it has some arguments
@@ -309,7 +332,7 @@ function parseDeclarationsInternal(isCssPropertyKnown, inputString,
     if (token.tokenType === "symbol" && token.text === ":") {
       if (!lastProp.name) {
         // Set the current declaration name if there's no name yet
-        lastProp.name = current.trim();
+        lastProp.name = cssTrim(current);
         lastProp.colonOffsets = [token.startOffset, token.endOffset];
         current = "";
         hasBang = false;
@@ -335,7 +358,7 @@ function parseDeclarationsInternal(isCssPropertyKnown, inputString,
         current = "";
         break;
       }
-      lastProp.value = current.trim();
+      lastProp.value = cssTrim(current);
       current = "";
       hasBang = false;
       declarations.push(getEmptyDeclaration());
@@ -383,11 +406,11 @@ function parseDeclarationsInternal(isCssPropertyKnown, inputString,
       // Ignore this case in comments.
       if (!inComment) {
         // Trailing property found, e.g. p1:v1;p2:v2;p3
-        lastProp.name = current.trim();
+        lastProp.name = cssTrim(current);
       }
     } else {
       // Trailing value found, i.e. value without an ending ;
-      lastProp.value = current.trim();
+      lastProp.value = cssTrim(current);
       let terminator = lexer.performEOFFixup("", true);
       lastProp.terminator = terminator + ";";
       // If the input was unterminated, attribute the remainder to
@@ -601,12 +624,48 @@ RuleRewriter.prototype = {
    *                  to be "lexically safe".
    */
   sanitizePropertyValue: function (text) {
+    // Start by stripping any trailing ";".  This is done here to
+    // avoid the case where the user types "url(" (which is turned
+    // into "url(;" by the rule view before coming here), being turned
+    // into "url(;)" by this code -- due to the way "url(...)" is
+    // parsed as a single token.
+    text = text.replace(/;$/, "");
     let lexer = getCSSLexer(text);
 
     let result = "";
     let previousOffset = 0;
-    let braceDepth = 0;
+    let parenStack = [];
     let anySanitized = false;
+
+    // Push a closing paren on the stack.
+    let pushParen = (token, closer) => {
+      result = result + text.substring(previousOffset, token.startOffset) +
+        text.substring(token.startOffset, token.endOffset);
+      // We set the location of the paren in a funny way, to handle
+      // the case where we've seen a function token, where the paren
+      // appears at the end.
+      parenStack.push({closer, offset: result.length - 1});
+      previousOffset = token.endOffset;
+    };
+
+    // Pop a closing paren from the stack.
+    let popSomeParens = (closer) => {
+      while (parenStack.length > 0) {
+        let paren = parenStack.pop();
+
+        if (paren.closer === closer) {
+          return true;
+        }
+
+        // Found a non-matching closing paren, so quote it.  Note that
+        // these are processed in reverse order.
+        result = result.substring(0, paren.offset) + "\\" +
+          result.substring(paren.offset);
+        anySanitized = true;
+      }
+      return false;
+    };
+
     while (true) {
       let token = lexer.nextToken();
       if (!token) {
@@ -624,14 +683,22 @@ RuleRewriter.prototype = {
             break;
 
           case "{":
-            ++braceDepth;
+            pushParen(token, "}");
+            break;
+
+          case "(":
+            pushParen(token, ")");
+            break;
+
+          case "[":
+            pushParen(token, "]");
             break;
 
           case "}":
-            --braceDepth;
-            if (braceDepth < 0) {
-              // Found an unmatched close bracket.
-              braceDepth = 0;
+          case ")":
+          case "]":
+            // Did we find an unmatched close bracket?
+            if (!popSomeParens(token.text)) {
               // Copy out text from |previousOffset|.
               result += text.substring(previousOffset, token.startOffset);
               // Quote the offending symbol.
@@ -641,9 +708,14 @@ RuleRewriter.prototype = {
             }
             break;
         }
+      } else if (token.tokenType === "function") {
+        pushParen(token, ")");
       }
     }
 
+    // Fix up any unmatched parens.
+    popSomeParens(null);
+
     // Copy out any remaining text, then any needed terminators.
     result += text.substring(previousOffset, text.length);
     let eofFixup = lexer.performEOFFixup("", true);
@@ -783,7 +855,7 @@ RuleRewriter.prototype = {
       // a property but which would break the entire style sheet.
       let newText = this.inputString.substring(decl.colonOffsets[1],
                                                decl.offsets[1]);
-      newText = unescapeCSSComment(newText).trimRight();
+      newText = cssTrimRight(unescapeCSSComment(newText));
       this.result += this.sanitizeText(newText, index) + ";";
 
       // See if the comment end can be deleted.
