@@ -4,16 +4,17 @@
 
 "use strict";
 
-const events = require("sdk/event/core");
+const EventEmitter = require("devtools/shared/event-emitter");
 
-loader.lazyRequireGetter(this, "getOuterId", "sdk/window/utils", true);
 loader.lazyRequireGetter(this, "CommandState",
   "devtools/shared/gcli/command-state", true);
 
 const l10n = require("gcli/l10n");
 require("devtools/server/actors/inspector");
-const { RulersHighlighter, HighlighterEnvironment } =
+const { HighlighterEnvironment } =
   require("devtools/server/actors/highlighters");
+const { RulersHighlighter } =
+  require("devtools/server/actors/highlighters/rulers");
 
 const highlighters = new WeakMap();
 
@@ -27,7 +28,7 @@ exports.items = [
     description: l10n.lookup("rulersDesc"),
     manual: l10n.lookup("rulersManual"),
     buttonId: "command-button-rulers",
-    buttonClass: "command-button command-button-invertable",
+    buttonClass: "command-button",
     tooltipText: l10n.lookup("rulersTooltip"),
     state: {
       isChecked: (target) => CommandState.isEnabledForTarget(target, "rulers"),
@@ -82,7 +83,7 @@ exports.items = [
 
       // Listen to the highlighter's destroy event which may happen if the
       // window is refreshed or closed with the rulers shown.
-      events.once(highlighter, "destroy", () => {
+      EventEmitter.once(highlighter, "destroy", () => {
         if (highlighters.has(document)) {
           let { environment: toDestroy } = highlighters.get(document);
           toDestroy.destroy();
