@@ -11,7 +11,6 @@
 #include "base/platform_thread.h"
 #include "nsTArray.h"
 #include "mozilla/dom/battery/Types.h"
-#include "mozilla/dom/MozPowerManagerBinding.h"
 #include "mozilla/dom/network/Types.h"
 #include "mozilla/dom/power/Types.h"
 #include "mozilla/dom/ScreenOrientation.h"
@@ -109,60 +108,6 @@ void GetCurrentBatteryInformation(hal::BatteryInformation* aBatteryInfo);
  */
 void NotifyBatteryChange(const hal::BatteryInformation& aBatteryInfo);
 
-/**
- * Determine whether the device's screen is currently enabled.
- */
-bool GetScreenEnabled();
-
-/**
- * Enable or disable the device's screen.
- *
- * Note that it may take a few seconds for the screen to turn on or off.
- */
-void SetScreenEnabled(bool aEnabled);
-
-/**
- * Determine whether the device's keypad/button backlight is currently enabled.
- */
-bool GetKeyLightEnabled();
-
-/**
- * Enable or disable the device's keypad/button backlight.
- */
-void SetKeyLightEnabled(bool aEnabled);
-
-/**
- * Get the brightness of the device's screen's backlight, on a scale from 0
- * (very dim) to 1 (full blast).
- *
- * If the display is currently disabled, this returns the brightness the
- * backlight will have when the display is re-enabled.
- */
-double GetScreenBrightness();
-
-/**
- * Set the brightness of the device's screen's backlight, on a scale from 0
- * (very dimm) to 1 (full blast).  Values larger than 1 are treated like 1, and
- * values smaller than 0 are treated like 0.
- *
- * Note that we may reduce the resolution of the given brightness value before
- * sending it to the screen.  Therefore if you call SetScreenBrightness(x)
- * followed by GetScreenBrightness(), the value returned by
- * GetScreenBrightness() may not be exactly x.
- */
-void SetScreenBrightness(double aBrightness);
-
-/**
- * Determine whether the device is allowed to sleep.
- */
-bool GetCpuSleepAllowed();
-
-/**
- * Set whether the device is allowed to suspend automatically after
- * the screen is disabled.
- */
-void SetCpuSleepAllowed(bool aAllowed);
-
 /**
  * Register an observer for the sensor of given type.
  *
@@ -231,25 +176,6 @@ void NotifyNetworkChange(const hal::NetworkInformation& aNetworkInfo);
  */
 void AdjustSystemClock(int64_t aDeltaMilliseconds);
 
-/**
- * Set timezone
- * @param aTimezoneSpec The definition can be found in
- * http://en.wikipedia.org/wiki/List_of_tz_database_time_zones
- */
-void SetTimezone(const nsCString& aTimezoneSpec);
-
-/**
- * Get timezone
- * http://en.wikipedia.org/wiki/List_of_tz_database_time_zones
- */
-nsCString GetTimezone();
-
-/**
- * Get timezone offset
- * returns the timezone offset relative to UTC in minutes (DST effect included)
- */
-int32_t GetTimezoneOffset();
-
 /**
  * Register observer for system clock changed notification.
  * @param aObserver The observer that should be added.
@@ -291,20 +217,6 @@ void UnregisterSystemTimezoneChangeObserver(
 void NotifySystemTimezoneChange(
   const hal::SystemTimezoneChangeInformation& aSystemTimezoneChangeInfo);
 
-/**
- * Reboot the device.
- *
- * This API is currently only allowed to be used from the main process.
- */
-void Reboot();
-
-/**
- * Power off the device.
- *
- * This API is currently only allowed to be used from the main process.
- */
-void PowerOff();
-
 /**
  * Enable wake lock notifications from the backend.
  *
@@ -419,17 +331,6 @@ void UnregisterSwitchObserver(hal::SwitchDevice aDevice, hal::SwitchObserver *aS
  */
 void NotifySwitchChange(const hal::SwitchEvent& aEvent);
 
-/**
- * Get current switch information.
- */
-hal::SwitchState GetCurrentSwitchState(hal::SwitchDevice aDevice);
-
-/**
- * Notify switch status change from input device.
- */
-void NotifySwitchStateFromInputDevice(hal::SwitchDevice aDevice,
-                                      hal::SwitchState aState);
-
 /**
  * Return true if the current platform supports the setting of process
  * priority.
@@ -461,20 +362,6 @@ void SetCurrentThreadPriority(hal::ThreadPriority aThreadPriority);
 void SetThreadPriority(PlatformThreadId aThreadId,
                        hal::ThreadPriority aThreadPriority);
 
-/**
- * Start a watchdog to compulsively shutdown the system if it hangs.
- * @param aMode Specify how to shutdown the system.
- * @param aTimeoutSecs Specify the delayed seconds to shutdown the system.
- *
- * This API is currently only allowed to be used from the main process.
- */
-void StartForceQuitWatchdog(hal::ShutdownMode aMode, int32_t aTimeoutSecs);
-
-/**
- * Perform Factory Reset to wipe out all user data.
- */
-void FactoryReset(mozilla::dom::FactoryResetReason& aReason);
-
 /**
  * Start monitoring disk space for low space situations.
  *
@@ -489,33 +376,6 @@ void StartDiskSpaceWatcher();
  */
 void StopDiskSpaceWatcher();
 
-/**
- * Get total system memory of device being run on in bytes.
- *
- * Returns 0 if we are unable to determine this information from /proc/meminfo.
- */
-uint32_t GetTotalSystemMemory();
-
-/**
- * Determine whether the headphone switch event is from input device
- */
-bool IsHeadphoneEventFromInputDev();
-
-/**
- * Start the system service with the specified name and arguments.
- */
-nsresult StartSystemService(const char* aSvcName, const char* aArgs);
-
-/**
- * Stop the system service with the specified name.
- */
-void StopSystemService(const char* aSvcName);
-
-/**
- * Determine whether the system service with the specified name is running.
- */
-bool SystemServiceIsRunning(const char* aSvcName);
-
 } // namespace MOZ_HAL_NAMESPACE
 } // namespace mozilla
 
