@@ -15,6 +15,7 @@
 #include "mozilla/dom/battery/Types.h"
 #include "mozilla/dom/network/Types.h"
 #include "mozilla/dom/ScreenOrientation.h"
+#include "mozilla/fallback/FallbackScreenConfiguration.h"
 #include "mozilla/Observer.h"
 #include "mozilla/Unused.h"
 #include "nsAutoPtr.h"
@@ -118,7 +119,7 @@ DisableScreenConfigurationNotifications()
 void
 GetCurrentScreenConfiguration(ScreenConfiguration* aScreenConfiguration)
 {
-  Hal()->SendGetCurrentScreenConfiguration(aScreenConfiguration);
+  fallback::GetCurrentScreenConfiguration(aScreenConfiguration);
 }
 
 bool
@@ -135,90 +136,12 @@ UnlockScreenOrientation()
   Hal()->SendUnlockScreenOrientation();
 }
 
-bool
-GetScreenEnabled()
-{
-  bool enabled = false;
-  Hal()->SendGetScreenEnabled(&enabled);
-  return enabled;
-}
-
-void
-SetScreenEnabled(bool aEnabled)
-{
-  Hal()->SendSetScreenEnabled(aEnabled);
-}
-
-bool
-GetKeyLightEnabled()
-{
-  bool enabled = false;
-  Hal()->SendGetKeyLightEnabled(&enabled);
-  return enabled;
-}
-
-void
-SetKeyLightEnabled(bool aEnabled)
-{
-  Hal()->SendSetKeyLightEnabled(aEnabled);
-}
-
-bool
-GetCpuSleepAllowed()
-{
-  bool allowed = true;
-  Hal()->SendGetCpuSleepAllowed(&allowed);
-  return allowed;
-}
-
-void
-SetCpuSleepAllowed(bool aAllowed)
-{
-  Hal()->SendSetCpuSleepAllowed(aAllowed);
-}
-
-double
-GetScreenBrightness()
-{
-  double brightness = 0;
-  Hal()->SendGetScreenBrightness(&brightness);
-  return brightness;
-}
-
-void
-SetScreenBrightness(double aBrightness)
-{
-  Hal()->SendSetScreenBrightness(aBrightness);
-}
-
 void
 AdjustSystemClock(int64_t aDeltaMilliseconds)
 {
   Hal()->SendAdjustSystemClock(aDeltaMilliseconds);
 }
 
-void
-SetTimezone(const nsCString& aTimezoneSpec)
-{
-  Hal()->SendSetTimezone(nsCString(aTimezoneSpec));
-}
-
-nsCString
-GetTimezone()
-{
-  nsCString timezone;
-  Hal()->SendGetTimezone(&timezone);
-  return timezone;
-}
-
-int32_t
-GetTimezoneOffset()
-{
-  int32_t timezoneOffset;
-  Hal()->SendGetTimezoneOffset(&timezoneOffset);
-  return timezoneOffset;
-}
-
 void
 EnableSystemClockChangeNotifications()
 {
@@ -243,24 +166,6 @@ DisableSystemTimezoneChangeNotifications()
   Hal()->SendDisableSystemTimezoneChangeNotifications();
 }
 
-void
-Reboot()
-{
-  MOZ_CRASH("Reboot() can't be called from sandboxed contexts.");
-}
-
-void
-PowerOff()
-{
-  MOZ_CRASH("PowerOff() can't be called from sandboxed contexts.");
-}
-
-void
-StartForceQuitWatchdog(ShutdownMode aMode, int32_t aTimeoutSecs)
-{
-  MOZ_CRASH("StartForceQuitWatchdog() can't be called from sandboxed contexts.");
-}
-
 void
 EnableSensorNotifications(SensorType aSensor) {
   Hal()->SendEnableSensorNotifications(aSensor);
@@ -311,71 +216,47 @@ DisableSwitchNotifications(SwitchDevice aDevice)
   Hal()->SendDisableSwitchNotifications(aDevice);
 }
 
-SwitchState
-GetCurrentSwitchState(SwitchDevice aDevice)
-{
-  SwitchState state;
-  Hal()->SendGetCurrentSwitchState(aDevice, &state);
-  return state;
-}
-
-void
-NotifySwitchStateFromInputDevice(SwitchDevice aDevice, SwitchState aState)
-{
-  Unused << aDevice;
-  Unused << aState;
-  NS_RUNTIMEABORT("Only the main process may notify switch state change.");
-}
-
 bool
 EnableAlarm()
 {
-  NS_RUNTIMEABORT("Alarms can't be programmed from sandboxed contexts.  Yet.");
-  return false;
+  MOZ_CRASH("Alarms can't be programmed from sandboxed contexts.  Yet.");
 }
 
 void
 DisableAlarm()
 {
-  NS_RUNTIMEABORT("Alarms can't be programmed from sandboxed contexts.  Yet.");
+  MOZ_CRASH("Alarms can't be programmed from sandboxed contexts.  Yet.");
 }
 
 bool
 SetAlarm(int32_t aSeconds, int32_t aNanoseconds)
 {
-  NS_RUNTIMEABORT("Alarms can't be programmed from sandboxed contexts.  Yet.");
-  return false;
+  MOZ_CRASH("Alarms can't be programmed from sandboxed contexts.  Yet.");
 }
 
 void
-SetProcessPriority(int aPid, ProcessPriority aPriority, uint32_t aLRU)
+SetProcessPriority(int aPid, ProcessPriority aPriority)
 {
-  NS_RUNTIMEABORT("Only the main process may set processes' priorities.");
+  MOZ_CRASH("Only the main process may set processes' priorities.");
+}
+
+bool
+SetProcessPrioritySupported()
+{
+  MOZ_CRASH("Only the main process may call SetProcessPrioritySupported().");
 }
 
 void
 SetCurrentThreadPriority(ThreadPriority aThreadPriority)
 {
-  NS_RUNTIMEABORT("Setting current thread priority cannot be called from sandboxed contexts.");
+  MOZ_CRASH("Setting current thread priority cannot be called from sandboxed contexts.");
 }
 
 void
 SetThreadPriority(PlatformThreadId aThreadId,
                   ThreadPriority aThreadPriority)
 {
-  NS_RUNTIMEABORT("Setting thread priority cannot be called from sandboxed contexts.");
-}
-
-void
-FactoryReset(FactoryResetReason& aReason)
-{
-  if (aReason == FactoryResetReason::Normal) {
-    Hal()->SendFactoryReset(NS_LITERAL_STRING("normal"));
-  } else if (aReason == FactoryResetReason::Wipe) {
-    Hal()->SendFactoryReset(NS_LITERAL_STRING("wipe"));
-  } else if (aReason == FactoryResetReason::Root) {
-    Hal()->SendFactoryReset(NS_LITERAL_STRING("root"));
-  }
+  MOZ_CRASH("Setting thread priority cannot be called from sandboxed contexts.");
 }
 
 void
@@ -390,29 +271,6 @@ StopDiskSpaceWatcher()
   MOZ_CRASH("StopDiskSpaceWatcher() can't be called from sandboxed contexts.");
 }
 
-bool IsHeadphoneEventFromInputDev()
-{
-  MOZ_CRASH("IsHeadphoneEventFromInputDev() cannot be called from sandboxed contexts.");
-  return false;
-}
-
-nsresult StartSystemService(const char* aSvcName, const char* aArgs)
-{
-  MOZ_CRASH("System services cannot be controlled from sandboxed contexts.");
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-void StopSystemService(const char* aSvcName)
-{
-  MOZ_CRASH("System services cannot be controlled from sandboxed contexts.");
-}
-
-bool SystemServiceIsRunning(const char* aSvcName)
-{
-  MOZ_CRASH("System services cannot be controlled from sandboxed contexts.");
-  return false;
-}
-
 class HalParent : public PHalParent
                 , public BatteryObserver
                 , public NetworkObserver
@@ -536,12 +394,6 @@ public:
     return IPC_OK();
   }
 
-  virtual mozilla::ipc::IPCResult
-  RecvGetCurrentScreenConfiguration(ScreenConfiguration* aScreenConfiguration) override {
-    hal::GetCurrentScreenConfiguration(aScreenConfiguration);
-    return IPC_OK();
-  }
-
   virtual mozilla::ipc::IPCResult
   RecvLockScreenOrientation(const dom::ScreenOrientationInternal& aOrientation, bool* aAllowed) override
   {
@@ -564,62 +416,6 @@ public:
     Unused << SendNotifyScreenConfigurationChange(aScreenConfiguration);
   }
 
-  virtual mozilla::ipc::IPCResult
-  RecvGetScreenEnabled(bool* aEnabled) override
-  {
-    *aEnabled = hal::GetScreenEnabled();
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvSetScreenEnabled(const bool& aEnabled) override
-  {
-    hal::SetScreenEnabled(aEnabled);
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvGetKeyLightEnabled(bool* aEnabled) override
-  {
-    *aEnabled = hal::GetKeyLightEnabled();
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvSetKeyLightEnabled(const bool& aEnabled) override
-  {
-    hal::SetKeyLightEnabled(aEnabled);
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvGetCpuSleepAllowed(bool* aAllowed) override
-  {
-    *aAllowed = hal::GetCpuSleepAllowed();
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvSetCpuSleepAllowed(const bool& aAllowed) override
-  {
-    hal::SetCpuSleepAllowed(aAllowed);
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvGetScreenBrightness(double* aBrightness) override
-  {
-    *aBrightness = hal::GetScreenBrightness();
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvSetScreenBrightness(const double& aBrightness) override
-  {
-    hal::SetScreenBrightness(aBrightness);
-    return IPC_OK();
-  }
-
   virtual mozilla::ipc::IPCResult
   RecvAdjustSystemClock(const int64_t &aDeltaMilliseconds) override
   {
@@ -627,27 +423,6 @@ public:
     return IPC_OK();
   }
 
-  virtual mozilla::ipc::IPCResult
-  RecvSetTimezone(const nsCString& aTimezoneSpec) override
-  {
-    hal::SetTimezone(aTimezoneSpec);
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvGetTimezone(nsCString *aTimezoneSpec) override
-  {
-    *aTimezoneSpec = hal::GetTimezone();
-    return IPC_OK();
-  }
-
-  virtual mozilla::ipc::IPCResult
-  RecvGetTimezoneOffset(int32_t *aTimezoneOffset) override
-  {
-    *aTimezoneOffset = hal::GetTimezoneOffset();
-    return IPC_OK();
-  }
-
   virtual mozilla::ipc::IPCResult
   RecvEnableSystemClockChangeNotifications() override
   {
@@ -754,14 +529,6 @@ public:
     Unused << SendNotifySwitchChange(aSwitchEvent);
   }
 
-  virtual mozilla::ipc::IPCResult
-  RecvGetCurrentSwitchState(const SwitchDevice& aDevice, hal::SwitchState *aState) override
-  {
-    // Content has no reason to listen to switch events currently.
-    *aState = hal::GetCurrentSwitchState(aDevice);
-    return IPC_OK();
-  }
-
   void Notify(const int64_t& aClockDeltaMS) override
   {
     Unused << SendNotifySystemClockChange(aClockDeltaMS);
@@ -771,25 +538,6 @@ public:
   {
     Unused << SendNotifySystemTimezoneChange(aSystemTimezoneChangeInfo);
   }
-
-  virtual mozilla::ipc::IPCResult
-  RecvFactoryReset(const nsString& aReason) override
-  {
-    FactoryResetReason reason = FactoryResetReason::Normal;
-    if (aReason.EqualsLiteral("normal")) {
-      reason = FactoryResetReason::Normal;
-    } else if (aReason.EqualsLiteral("wipe")) {
-      reason = FactoryResetReason::Wipe;
-    } else if (aReason.EqualsLiteral("root")) {
-      reason = FactoryResetReason::Root;
-    } else {
-      // Invalid factory reset reason. That should never happen.
-      return IPC_FAIL_NO_REASON(this);
-    }
-
-    hal::FactoryReset(reason);
-    return IPC_OK();
-  }
 };
 
 class HalChild : public PHalChild {
