@@ -32,17 +32,6 @@ class WeakMapObject : public WeakCollectionObject
     static const Class class_;
 };
 
-// WeakMap methods exposed so they can be installed in the self-hosting global.
-
-extern bool
-WeakMap_get(JSContext* cx, unsigned argc, Value* vp);
-
-extern bool
-WeakMap_set(JSContext* cx, unsigned argc, Value* vp);
-
-extern JSObject*
-InitBareWeakMapCtor(JSContext* cx, HandleObject obj);
-
 extern JSObject*
 InitWeakMapClass(JSContext* cx, HandleObject obj);
 
