@@ -7,15 +7,13 @@
 #ifndef builtin_WeakSetObject_h
 #define builtin_WeakSetObject_h
 
-#include "vm/NativeObject.h"
+#include "builtin/WeakMapObject.h"
 
 namespace js {
 
-class WeakSetObject : public NativeObject
+class WeakSetObject : public WeakCollectionObject
 {
   public:
-    static const unsigned RESERVED_SLOTS = 1;
-
     static JSObject* initClass(JSContext* cx, HandleObject obj);
     static const Class class_;
 
@@ -34,4 +32,11 @@ InitWeakSetClass(JSContext* cx, HandleObject obj);
 
 } // namespace js
 
+template<>
+inline bool
+JSObject::is<js::WeakCollectionObject>() const
+{
+    return is<js::WeakMapObject>() || is<js::WeakSetObject>();
+}
+
 #endif /* builtin_WeakSetObject_h */
