@@ -404,10 +404,9 @@ class FunctionBox : public ObjectBox, public SharedContext
     uint32_t        toStringEnd;
     uint16_t        length;
 
-    uint8_t         generatorKindBits_;     /* The GeneratorKind of this function. */
+    uint8_t         generatorKind_;         /* The GeneratorKind of this function. */
     uint8_t         asyncKindBits_;         /* The FunctionAsyncKind of this function. */
 
-    bool            isGenexpLambda:1;       /* lambda from generator expression */
     bool            hasDestructuringArgs:1; /* parameter list contains destructuring expression */
     bool            hasParameterExprs:1;    /* parameter list contains expressions */
     bool            hasDirectEvalInParameterExpr:1; /* parameter list contains direct eval */
@@ -468,8 +467,7 @@ class FunctionBox : public ObjectBox, public SharedContext
         return hasExtensibleScope() ||
                needsHomeObject() ||
                isDerivedClassConstructor() ||
-               isStarGenerator() ||
-               isLegacyGenerator() ||
+               isGenerator() ||
                isAsync();
     }
 
@@ -488,19 +486,18 @@ class FunctionBox : public ObjectBox, public SharedContext
         return usesArguments && usesApply && usesThis && !usesReturn;
     }
 
-    GeneratorKind generatorKind() const { return GeneratorKindFromBits(generatorKindBits_); }
-    bool isLegacyGenerator() const { return generatorKind() == LegacyGenerator; }
-    bool isStarGenerator() const { return generatorKind() == StarGenerator; }
+    GeneratorKind generatorKind() const { return GeneratorKindFromBit(generatorKind_); }
+    bool isGenerator() const { return generatorKind() == GeneratorKind::Generator; }
     FunctionAsyncKind asyncKind() const { return AsyncKindFromBits(asyncKindBits_); }
 
     bool needsFinalYield() const {
-        return isStarGenerator() || isLegacyGenerator() || isAsync();
+        return isGenerator() || isAsync();
     }
     bool needsDotGeneratorName() const {
-        return isStarGenerator() || isLegacyGenerator() || isAsync();
+        return isGenerator() || isAsync();
     }
     bool needsIteratorResult() const {
-        return isStarGenerator();
+        return isGenerator();
     }
 
     bool isAsync() const { return asyncKind() == AsyncFunction; }
@@ -516,14 +513,6 @@ class FunctionBox : public ObjectBox, public SharedContext
         isExprBody_ = true;
     }
 
-    void setGeneratorKind(GeneratorKind kind) {
-        // A generator kind can be set at initialization, or when "yield" is
-        // first seen.  In both cases the transition can only happen from
-        // NotGenerator.
-        MOZ_ASSERT(!isStarGenerator() && !isLegacyGenerator());
-        generatorKindBits_ = GeneratorKindAsBits(kind);
-    }
-
     bool hasExtensibleScope()        const { return funCxFlags.hasExtensibleScope; }
     bool hasThisBinding()            const { return funCxFlags.hasThisBinding; }
     bool argumentsHasLocalBinding()  const { return funCxFlags.argumentsHasLocalBinding; }
@@ -623,8 +612,7 @@ SharedContext::allBindingsClosedOver()
 {
     return bindingsAccessedDynamically() ||
            (isFunctionBox() &&
-            (asFunctionBox()->isStarGenerator() ||
-             asFunctionBox()->isLegacyGenerator() ||
+            (asFunctionBox()->isGenerator() ||
              asFunctionBox()->isAsync()));
 }
 
