@@ -8,8 +8,6 @@
 
 #include "mozilla/Assertions.h"
 
-#include "jscompartment.h"
-
 #include "gc/Statistics.h"
 #include "vm/ArgumentsObject.h"
 #include "vm/Runtime.h"
@@ -28,11 +26,9 @@ StoreBuffer::GenericBuffer::trace(StoreBuffer* owner, JSTracer* trc)
         return;
 
     for (LifoAlloc::Enum e(*storage_); !e.empty();) {
-        unsigned size = *e.get<unsigned>();
-        e.popFront<unsigned>();
-        BufferableRef* edge = e.get<BufferableRef>(size);
+        unsigned size = *e.read<unsigned>();
+        BufferableRef* edge = e.read<BufferableRef>(size);
         edge->trace(trc);
-        e.popFront(size);
     }
 }
 
