@@ -15,6 +15,7 @@
 
 #include "jsalloc.h"
 
+#include "ds/BitArray.h"
 #include "ds/LifoAlloc.h"
 #include "gc/Nursery.h"
 #include "js/MemoryMetrics.h"
@@ -22,6 +23,7 @@
 namespace js {
 namespace gc {
 
+class Arena;
 class ArenaCellSet;
 
 /*
