@@ -33,6 +33,8 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Move.h"
 
+#include "jsgc.h"
+
 #include "frontend/TokenStream.h"
 #include "irregexp/RegExpCharacters.h"
 #include "vm/ErrorReporting.h"
@@ -328,6 +330,7 @@ RegExpParser<CharT>::Advance()
         next_pos_++;
     } else {
         current_ = kEndMarker;
+        next_pos_ = end_ + 1;
         has_more_ = false;
     }
 }
