@@ -1,13 +1,13 @@
 // |jit-test| error:98; need-for-each
 function foo() {
-    function gen() {
+    function* gen() {
         try {
             yield 1;
         } finally {
             throw 98;
         }
     }
-    for (i in gen()) {
+    for (i of gen()) {
         for each (var i in this)
             return false;
     }
