@@ -13,7 +13,7 @@ const Table = WebAssembly.Table;
 function normalize(stack)
 {
     var wasmFrameTypes = [
-        {re:/^entry trampoline \(in wasm\)$/,                        sub:">"},
+        {re:/^slow entry trampoline \(in wasm\)$/,                   sub:">"},
         {re:/^wasm-function\[(\d+)\] \(.*\)$/,                       sub:"$1"},
         {re:/^(fast|slow) FFI trampoline (to native )?\(in wasm\)$/, sub:"<"},
         {re:/^call to[ asm.js]? native (.*) \(in wasm\)$/,           sub:"$1"},
@@ -176,40 +176,44 @@ for (let type of ['f32', 'f64']) {
     }
 }
 
-function testError(code, error, expect)
-{
-    enableGeckoProfiling();
-    var f = wasmEvalText(code).exports[""];
-    enableSingleStepProfiling();
-    assertThrowsInstanceOf(f, error);
-    assertEqStacks(disableSingleStepProfiling(), expect);
-    disableGeckoProfiling();
-}
+(function() {
+    // Error handling.
+    function testError(code, error, expect)
+    {
+        enableGeckoProfiling();
+        var f = wasmEvalText(code).exports[""];
+        enableSingleStepProfiling();
+        assertThrowsInstanceOf(f, error);
+        assertEqStacks(disableSingleStepProfiling(), expect);
+        disableGeckoProfiling();
+    }
 
-testError(
-`(module
-    (func $foo (unreachable))
-    (func (export "") (call $foo))
-)`,
-WebAssembly.RuntimeError,
-["", ">", "1,>", "0,1,>", "interstitial,0,1,>", "trap handling,0,1,>", "", ">", ""]);
+    testError(
+    `(module
+        (func $foo (unreachable))
+        (func (export "") (call $foo))
+    )`,
+    WebAssembly.RuntimeError,
+    ["", ">", "1,>", "0,1,>", "interstitial,0,1,>", "trap handling,0,1,>", "", ">", ""]);
 
-testError(
-`(module
-    (type $good (func))
-    (type $bad (func (param i32)))
-    (func $foo (call_indirect $bad (i32.const 1) (i32.const 0)))
-    (func $bar (type $good))
-    (table anyfunc (elem $bar))
-    (export "" $foo)
-)`,
-WebAssembly.RuntimeError,
-// Technically we have this one *one-instruction* interval where
-// the caller is lost (the stack with "1,>"). It's annoying to fix and shouldn't
-// mess up profiles in practice so we ignore it.
-["", ">", "0,>", "1,0,>", "1,>", "trap handling,0,>", "", ">", ""]);
+    testError(
+    `(module
+        (type $good (func))
+        (type $bad (func (param i32)))
+        (func $foo (call_indirect $bad (i32.const 1) (i32.const 0)))
+        (func $bar (type $good))
+        (table anyfunc (elem $bar))
+        (export "" $foo)
+    )`,
+    WebAssembly.RuntimeError,
+    // Technically we have this one *one-instruction* interval where
+    // the caller is lost (the stack with "1,>"). It's annoying to fix and shouldn't
+    // mess up profiles in practice so we ignore it.
+    ["", ">", "0,>", "1,0,>", "1,>", "trap handling,0,>", "", ">", ""]);
+})();
 
 (function() {
+    // Tables fun.
     var e = wasmEvalText(`
     (module
         (func $foo (result i32) (i32.const 42))
@@ -280,6 +284,7 @@ WebAssembly.RuntimeError,
 })();
 
 (function() {
+    // Optimized wasm->wasm import.
     var m1 = new Module(wasmTextToBinary(`(module
         (func $foo (result i32) (i32.const 42))
         (export "foo" $foo)
@@ -310,3 +315,126 @@ WebAssembly.RuntimeError,
     disableGeckoProfiling();
     assertEq(e4.bar(), 42);
 })();
+
+(function() {
+    // FFIs test.
+    let prevOptions = getJitCompilerOptions();
+
+    // Skip tests if baseline isn't enabled, since the stacks might differ by
+    // a few instructions.
+    if (prevOptions['baseline.enable'] === 0)
+        return;
+
+    setJitCompilerOption("baseline.warmup.trigger", 10);
+
+    enableGeckoProfiling();
+
+    var m = new Module(wasmTextToBinary(`(module
+        (import $ffi "a" "ffi" (param i32) (result i32))
+
+        (import $missingOneArg "a" "sumTwo" (param i32) (result i32))
+
+        (func (export "foo") (param i32) (result i32)
+         get_local 0
+         call $ffi)
+
+        (func (export "id") (param i32) (result i32)
+         get_local 0
+         call $missingOneArg
+        )
+    )`));
+
+    var valueToConvert = 0;
+    function ffi(n) {
+        new Error().stack; // enter VM to clobber FP register.
+        if (n == 1337) { return valueToConvert };
+        return 42;
+    }
+
+    function sumTwo(a, b) {
+        return (a|0)+(b|0)|0;
+    }
+
+    // Baseline compile ffi.
+    for (var i = 20; i --> 0;) {
+        ffi(i);
+        sumTwo(i-1, i+1);
+    }
+
+    var imports = {
+        a: {
+            ffi,
+            sumTwo
+        }
+    };
+
+    var i = new Instance(m, imports).exports;
+
+    // Enable the jit exit.
+    assertEq(i.foo(0), 42);
+    assertEq(i.id(13), 13);
+
+    // Test normal conditions.
+    enableSingleStepProfiling();
+    assertEq(i.foo(0), 42);
+    assertEqStacks(disableSingleStepProfiling(), ["", ">", "2,>", "<,2,>",
+        // Losing stack information while the JIT func prologue sets profiler
+        // virtual FP.
+        "",
+        // Callee time.
+        "<,2,>",
+        // Losing stack information while we're exiting JIT func epilogue and
+        // recovering wasm FP.
+        "",
+        // Back into the jit exit (frame info has been recovered).
+        "<,2,>",
+        // Normal unwinding.
+        "2,>", ">", ""]);
+
+    // Test rectifier frame.
+    enableSingleStepProfiling();
+    assertEq(i.id(100), 100);
+    assertEqStacks(disableSingleStepProfiling(), ["", ">", "3,>", "<,3,>",
+        // Rectifier frame time is spent here (lastProfilingFrame has not been
+        // set).
+        "",
+        "<,3,>",
+        // Rectifier frame unwinding time is spent here.
+        "",
+        "<,3,>",
+        "3,>", ">", ""]);
+
+    // Test OOL coercion path.
+    valueToConvert = 2**31;
+
+    enableSingleStepProfiling();
+    assertEq(i.foo(1337), -(2**31));
+    assertEqStacks(disableSingleStepProfiling(), ["", ">", "2,>", "<,2,>", "", "<,2,>", "",
+        // Back into the jit exit (frame info has been recovered).
+        // Inline conversion fails, we skip to the OOL path, call from there
+        // and get back to the jit exit.
+        "<,2,>",
+        // Normal unwinding.
+        "2,>", ">", ""]);
+
+    disableGeckoProfiling();
+    setJitCompilerOption("baseline.warmup.trigger", prevOptions["baseline.warmup.trigger"]);
+})();
+
+// Make sure it's possible to single-step through call through debug-enabled code.
+(function() {
+ enableGeckoProfiling();
+
+ let g = newGlobal('');
+ let dbg = new Debugger(g);
+ dbg.onEnterFrame = () => {};
+ enableSingleStepProfiling();
+ g.eval(`
+    var code = wasmTextToBinary('(module (func (export "run") (result i32) i32.const 42))');
+    var i = new WebAssembly.Instance(new WebAssembly.Module(code));
+    assertEq(i.exports.run(), 42);
+ `);
+
+ disableSingleStepProfiling();
+ disableGeckoProfiling();
+})();
