@@ -556,9 +556,6 @@ CanEnterBaselineMethod(JSContext* cx, RunState& state);
 MethodStatus
 CanEnterBaselineAtBranch(JSContext* cx, InterpreterFrame* fp, bool newType);
 
-JitExecStatus
-EnterBaselineMethod(JSContext* cx, RunState& state);
-
 JitExecStatus
 EnterBaselineAtBranch(JSContext* cx, InterpreterFrame* fp, jsbytecode* pc);
 
@@ -642,6 +639,8 @@ MarkActiveBaselineScripts(Zone* zone);
 MethodStatus
 BaselineCompile(JSContext* cx, JSScript* script, bool forceDebugInstrumentation = false);
 
+static const unsigned BASELINE_MAX_ARGS_LENGTH = 20000;
+
 } // namespace jit
 } // namespace js
 
