@@ -269,6 +269,7 @@ struct ResumeFromException
     static const uint32_t RESUME_FINALLY = 2;
     static const uint32_t RESUME_FORCED_RETURN = 3;
     static const uint32_t RESUME_BAILOUT = 4;
+    static const uint32_t RESUME_WASM = 5;
 
     uint8_t* framePointer;
     uint8_t* stackPointer;
@@ -444,6 +445,14 @@ class RectifierFrameLayout : public JitFrameLayout
     }
 };
 
+class WasmToJSJitFrameLayout : public JitFrameLayout
+{
+  public:
+    static inline size_t Size() {
+        return sizeof(WasmToJSJitFrameLayout);
+    }
+};
+
 class IonICCallFrameLayout : public CommonFrameLayout
 {
   protected:
