@@ -21,13 +21,13 @@ MIRGenerator::MIRGenerator(CompileCompartment* compartment, const JitCompileOpti
                            TempAllocator* alloc, MIRGraph* graph, const CompileInfo* info,
                            const OptimizationInfo* optimizationInfo)
   : compartment(compartment),
+    runtime(compartment ? compartment->runtime() : nullptr),
     info_(info),
     optimizationInfo_(optimizationInfo),
     alloc_(alloc),
     graph_(graph),
     offThreadStatus_(Ok()),
     abortedPreliminaryGroups_(*alloc_),
-    pauseBuild_(nullptr),
     cancelBuild_(false),
     wasmMaxStackArgBytes_(0),
     needsOverrecursedCheck_(false),
@@ -286,8 +286,8 @@ MIRGraph::unmarkBlocks()
 }
 
 MBasicBlock*
-MBasicBlock::New(MIRGraph& graph, BytecodeAnalysis* analysis, const CompileInfo& info,
-                 MBasicBlock* pred, BytecodeSite* site, Kind kind)
+MBasicBlock::New(MIRGraph& graph, size_t stackDepth, const CompileInfo& info,
+                 MBasicBlock* maybePred, BytecodeSite* site, Kind kind)
 {
     MOZ_ASSERT(site->pc() != nullptr);
 
@@ -295,7 +295,7 @@ MBasicBlock::New(MIRGraph& graph, BytecodeAnalysis* analysis, const CompileInfo&
     if (!block->init())
         return nullptr;
 
-    if (!block->inherit(graph.alloc(), analysis, pred, 0))
+    if (!block->inherit(graph.alloc(), stackDepth, maybePred, 0))
         return nullptr;
 
     return block;
@@ -309,7 +309,7 @@ MBasicBlock::NewPopN(MIRGraph& graph, const CompileInfo& info,
     if (!block->init())
         return nullptr;
 
-    if (!block->inherit(graph.alloc(), nullptr, pred, popped))
+    if (!block->inherit(graph.alloc(), pred->stackDepth(), pred, popped))
         return nullptr;
 
     return block;
@@ -324,7 +324,7 @@ MBasicBlock::NewWithResumePoint(MIRGraph& graph, const CompileInfo& info,
 
     MOZ_ASSERT(!resumePoint->instruction());
     resumePoint->block()->discardResumePoint(resumePoint, RefType_None);
-    resumePoint->block_ = block;
+    resumePoint->setBlock(block);
     block->addResumePoint(resumePoint);
     block->entryResumePoint_ = resumePoint;
 
@@ -348,7 +348,7 @@ MBasicBlock::NewPendingLoopHeader(MIRGraph& graph, const CompileInfo& info,
     if (!block->init())
         return nullptr;
 
-    if (!block->inherit(graph.alloc(), nullptr, pred, 0, stackPhiCount))
+    if (!block->inherit(graph.alloc(), pred->stackDepth(), pred, 0, stackPhiCount))
         return nullptr;
 
     return block;
@@ -546,35 +546,31 @@ MBasicBlock::copySlots(MBasicBlock* from)
 }
 
 bool
-MBasicBlock::inherit(TempAllocator& alloc, BytecodeAnalysis* analysis, MBasicBlock* pred,
+MBasicBlock::inherit(TempAllocator& alloc, size_t stackDepth, MBasicBlock* maybePred,
                      uint32_t popped, unsigned stackPhiCount)
 {
-    if (pred) {
-        stackPosition_ = pred->stackPosition_;
-        MOZ_ASSERT(stackPosition_ >= popped);
-        stackPosition_ -= popped;
-        if (kind_ != PENDING_LOOP_HEADER)
-            copySlots(pred);
-    } else {
-        uint32_t stackDepth = analysis->info(pc()).stackDepth;
-        stackPosition_ = info().firstStackSlot() + stackDepth;
-        MOZ_ASSERT(stackPosition_ >= popped);
-        stackPosition_ -= popped;
-    }
+    MOZ_ASSERT_IF(maybePred, maybePred->stackDepth() == stackDepth);
+
+    MOZ_ASSERT(stackDepth >= popped);
+    stackDepth -= popped;
+    stackPosition_ = stackDepth;
+
+    if (maybePred && kind_ != PENDING_LOOP_HEADER)
+        copySlots(maybePred);
 
     MOZ_ASSERT(info_.nslots() >= stackPosition_);
     MOZ_ASSERT(!entryResumePoint_);
 
     // Propagate the caller resume point from the inherited block.
-    callerResumePoint_ = pred ? pred->callerResumePoint() : nullptr;
+    callerResumePoint_ = maybePred ? maybePred->callerResumePoint() : nullptr;
 
     // Create a resume point using our initial stack state.
     entryResumePoint_ = new(alloc) MResumePoint(this, pc(), MResumePoint::ResumeAt);
     if (!entryResumePoint_->init(alloc))
         return false;
 
-    if (pred) {
-        if (!predecessors_.append(pred))
+    if (maybePred) {
+        if (!predecessors_.append(maybePred))
             return false;
 
         if (kind_ == PENDING_LOOP_HEADER) {
@@ -583,35 +579,35 @@ MBasicBlock::inherit(TempAllocator& alloc, BytecodeAnalysis* analysis, MBasicBlo
                 MPhi* phi = MPhi::New(alloc.fallible());
                 if (!phi)
                     return false;
-                phi->addInlineInput(pred->getSlot(i));
+                phi->addInlineInput(maybePred->getSlot(i));
                 addPhi(phi);
                 setSlot(i, phi);
                 entryResumePoint()->initOperand(i, phi);
             }
 
-            MOZ_ASSERT(stackPhiCount <= stackDepth());
-            MOZ_ASSERT(info().firstStackSlot() <= stackDepth() - stackPhiCount);
+            MOZ_ASSERT(stackPhiCount <= stackDepth);
+            MOZ_ASSERT(info().firstStackSlot() <= stackDepth - stackPhiCount);
 
             // Avoid creating new phis for stack values that aren't part of the
             // loop.  Note that for loop headers that can OSR, all values on the
             // stack are part of the loop.
-            for (; i < stackDepth() - stackPhiCount; i++) {
-                MDefinition* val = pred->getSlot(i);
+            for (; i < stackDepth - stackPhiCount; i++) {
+                MDefinition* val = maybePred->getSlot(i);
                 setSlot(i, val);
                 entryResumePoint()->initOperand(i, val);
             }
 
-            for (; i < stackDepth(); i++) {
+            for (; i < stackDepth; i++) {
                 MPhi* phi = MPhi::New(alloc.fallible());
                 if (!phi)
                     return false;
-                phi->addInlineInput(pred->getSlot(i));
+                phi->addInlineInput(maybePred->getSlot(i));
                 addPhi(phi);
                 setSlot(i, phi);
                 entryResumePoint()->initOperand(i, phi);
             }
         } else {
-            for (size_t i = 0; i < stackDepth(); i++)
+            for (size_t i = 0; i < stackDepth; i++)
                 entryResumePoint()->initOperand(i, getSlot(i));
         }
     } else {
@@ -619,7 +615,7 @@ MBasicBlock::inherit(TempAllocator& alloc, BytecodeAnalysis* analysis, MBasicBlo
          * Don't leave the operands uninitialized for the caller, as it may not
          * initialize them later on.
          */
-        for (size_t i = 0; i < stackDepth(); i++)
+        for (size_t i = 0; i < stackDepth; i++)
             entryResumePoint()->clearOperand(i);
     }
 
@@ -666,21 +662,6 @@ MBasicBlock::initEntrySlots(TempAllocator& alloc)
     return true;
 }
 
-MDefinition*
-MBasicBlock::getSlot(uint32_t index)
-{
-    MOZ_ASSERT(index < stackPosition_);
-    return slots_[index];
-}
-
-void
-MBasicBlock::initSlot(uint32_t slot, MDefinition* ins)
-{
-    slots_[slot] = ins;
-    if (entryResumePoint())
-        entryResumePoint()->initOperand(slot, ins);
-}
-
 void
 MBasicBlock::shimmySlots(int discardDepth)
 {
@@ -743,43 +724,6 @@ MBasicBlock::linkOsrValues(MStart* start)
     return true;
 }
 
-void
-MBasicBlock::setSlot(uint32_t slot, MDefinition* ins)
-{
-    slots_[slot] = ins;
-}
-
-void
-MBasicBlock::setVariable(uint32_t index)
-{
-    MOZ_ASSERT(stackPosition_ > info_.firstStackSlot());
-    setSlot(index, slots_[stackPosition_ - 1]);
-}
-
-void
-MBasicBlock::setArg(uint32_t arg)
-{
-    setVariable(info_.argSlot(arg));
-}
-
-void
-MBasicBlock::setLocal(uint32_t local)
-{
-    setVariable(info_.localSlot(local));
-}
-
-void
-MBasicBlock::setSlot(uint32_t slot)
-{
-    setVariable(slot);
-}
-
-void
-MBasicBlock::rewriteSlot(uint32_t slot, MDefinition* ins)
-{
-    setSlot(slot, ins);
-}
-
 void
 MBasicBlock::rewriteAtDepth(int32_t depth, MDefinition* ins)
 {
@@ -788,52 +732,6 @@ MBasicBlock::rewriteAtDepth(int32_t depth, MDefinition* ins)
     rewriteSlot(stackPosition_ + depth, ins);
 }
 
-void
-MBasicBlock::push(MDefinition* ins)
-{
-    MOZ_ASSERT(stackPosition_ < nslots());
-    slots_[stackPosition_++] = ins;
-}
-
-void
-MBasicBlock::pushVariable(uint32_t slot)
-{
-    push(slots_[slot]);
-}
-
-void
-MBasicBlock::pushArg(uint32_t arg)
-{
-    pushVariable(info_.argSlot(arg));
-}
-
-void
-MBasicBlock::pushLocal(uint32_t local)
-{
-    pushVariable(info_.localSlot(local));
-}
-
-void
-MBasicBlock::pushSlot(uint32_t slot)
-{
-    pushVariable(slot);
-}
-
-MDefinition*
-MBasicBlock::pop()
-{
-    MOZ_ASSERT(stackPosition_ > info_.firstStackSlot());
-    return slots_[--stackPosition_];
-}
-
-void
-MBasicBlock::popn(uint32_t n)
-{
-    MOZ_ASSERT(stackPosition_ - n >= info_.firstStackSlot());
-    MOZ_ASSERT(stackPosition_ >= stackPosition_ - n);
-    stackPosition_ -= n;
-}
-
 MDefinition*
 MBasicBlock::environmentChain()
 {
@@ -894,14 +792,6 @@ MBasicBlock::swapAt(int32_t depth)
     slots_[rhsDepth] = temp;
 }
 
-MDefinition*
-MBasicBlock::peek(int32_t depth)
-{
-    MOZ_ASSERT(depth < 0);
-    MOZ_ASSERT(stackPosition_ + depth >= info_.firstStackSlot());
-    return getSlot(stackPosition_ + depth);
-}
-
 void
 MBasicBlock::discardLastIns()
 {
@@ -1036,9 +926,9 @@ void
 MBasicBlock::discardDef(MDefinition* at)
 {
     if (at->isPhi())
-        at->block_->discardPhi(at->toPhi());
+        at->block()->discardPhi(at->toPhi());
     else
-        at->block_->discard(at->toInstruction());
+        at->block()->discard(at->toInstruction());
 }
 
 void
@@ -1141,24 +1031,6 @@ MBasicBlock::insertAtEnd(MInstruction* ins)
         add(ins);
 }
 
-void
-MBasicBlock::add(MInstruction* ins)
-{
-    MOZ_ASSERT(!hasLastIns());
-    ins->setBlock(this);
-    graph().allocDefinitionId(ins);
-    instructions_.pushBack(ins);
-    ins->setTrackedSite(trackedSite_);
-}
-
-void
-MBasicBlock::end(MControlInstruction* ins)
-{
-    MOZ_ASSERT(!hasLastIns()); // Existing control instructions should be removed first.
-    MOZ_ASSERT(ins);
-    add(ins);
-}
-
 void
 MBasicBlock::addPhi(MPhi* phi)
 {
@@ -1464,20 +1336,6 @@ MBasicBlock::setLoopHeader(MBasicBlock* newBackedge)
     MOZ_ASSERT(backedge() == newBackedge);
 }
 
-size_t
-MBasicBlock::numSuccessors() const
-{
-    MOZ_ASSERT(lastIns());
-    return lastIns()->numSuccessors();
-}
-
-MBasicBlock*
-MBasicBlock::getSuccessor(size_t index) const
-{
-    MOZ_ASSERT(lastIns());
-    return lastIns()->getSuccessor(index);
-}
-
 size_t
 MBasicBlock::getSuccessorIndex(MBasicBlock* block) const
 {
