@@ -104,6 +104,7 @@ namespace jit {
     _(NewArray)                                 \
     _(NewIterator)                              \
     _(NewDerivedTypedObject)                    \
+    _(NewCallObject)                            \
     _(CreateThisWithTemplate)                   \
     _(Lambda)                                   \
     _(LambdaArrow)                              \
@@ -639,6 +640,14 @@ class RLambdaArrow final : public RInstruction
     MOZ_MUST_USE bool recover(JSContext* cx, SnapshotIterator& iter) const override;
 };
 
+class RNewCallObject final : public RInstruction
+{
+  public:
+    RINSTRUCTION_HEADER_NUM_OP_(NewCallObject, 1)
+
+    MOZ_MUST_USE bool recover(JSContext* cx, SnapshotIterator& iter) const override;
+};
+
 class RSimdBox final : public RInstruction
 {
   private:
