@@ -11,8 +11,8 @@
 
 #include "jsfun.h"
 
-#include "jit/JitFrameIterator.h"
 #include "jit/JitFrames.h"
+#include "jit/JSJitFrameIter.h"
 
 #include "vm/EnvironmentObject.h"
 #include "vm/Stack.h"
@@ -170,6 +170,7 @@ class RematerializedFrame
     }
     JSFunction* callee() const {
         MOZ_ASSERT(isFunctionFrame());
+        MOZ_ASSERT(callee_);
         return callee_;
     }
     Value calleev() const {
@@ -258,16 +259,8 @@ struct MapTypeToRootKind<js::jit::RematerializedFrame*>
 
 template <>
 struct GCPolicy<js::jit::RematerializedFrame*>
-{
-    static js::jit::RematerializedFrame* initial() {
-        return nullptr;
-    }
-
-    static void trace(JSTracer* trc, js::jit::RematerializedFrame** frame, const char* name) {
-        if (*frame)
-            (*frame)->trace(trc);
-    }
-};
+  : public NonGCPointerPolicy<js::jit::RematerializedFrame*>
+{};
 
 } // namespace JS
 
