@@ -7,6 +7,8 @@
 #ifndef jit_TypePolicy_h
 #define jit_TypePolicy_h
 
+#include "mozilla/TypeTraits.h"
+
 #include "jit/IonTypes.h"
 #include "jit/JitAllocPolicy.h"
 
@@ -30,7 +32,7 @@ class TypePolicy
     //  * If untyped, optionally ask the input to try and specialize its value.
     //  * Replace the operand with a conversion instruction.
     //  * Insert an unconditional deoptimization (no conversion possible).
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) = 0;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) = 0;
 };
 
 struct TypeSpecializationData
@@ -80,8 +82,8 @@ class BoxInputsPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -90,28 +92,28 @@ class ArithPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 class AllDoublePolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    bool adjustInputs(TempAllocator& alloc, MInstruction* def);
+    MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def);
 };
 
 class BitwisePolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 class ComparePolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 // Policy for MTest instructions.
@@ -119,21 +121,21 @@ class TestPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class TypeBarrierPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class CallPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 // Policy for MPow. First operand Double; second Double or Int32.
@@ -141,7 +143,7 @@ class PowPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 // Expect a string for operand Op. If the input is a Value, it is unboxed.
@@ -150,8 +152,8 @@ class StringPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -162,8 +164,8 @@ class ConvertToStringPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -174,8 +176,8 @@ class BooleanPolicy final : private TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -186,8 +188,8 @@ class IntPolicy final : private TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -198,8 +200,8 @@ class ConvertToInt32Policy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -210,8 +212,8 @@ class TruncateToInt32Policy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -222,8 +224,8 @@ class DoublePolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -234,8 +236,8 @@ class Float32Policy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -247,7 +249,7 @@ class FloatingPointPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 template <unsigned Op>
@@ -255,8 +257,8 @@ class NoFloatPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -268,7 +270,7 @@ class NoFloatPolicyAfter final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 // Box objects or strings as an input to a ToDouble instruction.
@@ -276,8 +278,8 @@ class ToDoublePolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -287,8 +289,8 @@ class ToInt32Policy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -298,8 +300,8 @@ class ToStringPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -309,8 +311,8 @@ class ObjectPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
         return staticAdjustInputs(alloc, ins);
     }
 };
@@ -326,8 +328,8 @@ class SimdScalarPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* def);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override {
         return staticAdjustInputs(alloc, def);
     }
 };
@@ -336,7 +338,7 @@ class SimdAllPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 template <unsigned Op>
@@ -344,21 +346,21 @@ class SimdPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class SimdSelectPolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class SimdShufflePolicy final : public TypePolicy
 {
   public:
     SPECIALIZATION_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 // SIMD value-type policy, use the returned type of the instruction to determine
@@ -368,8 +370,8 @@ class SimdSameAsReturnedTypePolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
         return staticAdjustInputs(alloc, ins);
     }
 };
@@ -379,8 +381,8 @@ class BoxPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
         return staticAdjustInputs(alloc, ins);
     }
 };
@@ -391,8 +393,8 @@ class BoxExceptPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
-    bool adjustInputs(TempAllocator& alloc, MInstruction* ins) {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
+    MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) {
         return staticAdjustInputs(alloc, ins);
     }
 };
@@ -403,55 +405,34 @@ class CacheIdPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
-    bool adjustInputs(TempAllocator& alloc, MInstruction* ins) {
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins);
+    MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) {
         return staticAdjustInputs(alloc, ins);
     }
 };
 
 // Combine multiple policies.
-template <class Lhs, class Rhs>
+template <class... Policies>
 class MixPolicy final : public TypePolicy
 {
-  public:
-    EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins) {
-        return Lhs::staticAdjustInputs(alloc, ins) && Rhs::staticAdjustInputs(alloc, ins);
+    template <class P>
+    static bool staticAdjustInputsHelper(TempAllocator& alloc, MInstruction* ins) {
+        return P::staticAdjustInputs(alloc, ins);
     }
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
-        return staticAdjustInputs(alloc, ins);
-    }
-};
 
-// Combine three policies.
-template <class Policy1, class Policy2, class Policy3>
-class Mix3Policy final : public TypePolicy
-{
-  public:
-    EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins) {
-        return Policy1::staticAdjustInputs(alloc, ins) &&
-               Policy2::staticAdjustInputs(alloc, ins) &&
-               Policy3::staticAdjustInputs(alloc, ins);
+    template <class P, class... Rest>
+    static typename mozilla::EnableIf<(sizeof...(Rest) > 0), bool>::Type
+    staticAdjustInputsHelper(TempAllocator& alloc, MInstruction* ins) {
+        return P::staticAdjustInputs(alloc, ins) &&
+               MixPolicy::staticAdjustInputsHelper<Rest...>(alloc, ins);
     }
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
-        return staticAdjustInputs(alloc, ins);
-    }
-};
 
-// Combine four policies.  (Missing variadic templates yet?)
-template <class Policy1, class Policy2, class Policy3, class Policy4>
-class Mix4Policy : public TypePolicy
-{
   public:
     EMPTY_DATA_;
-    static bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins) {
-        return Policy1::staticAdjustInputs(alloc, ins) &&
-               Policy2::staticAdjustInputs(alloc, ins) &&
-               Policy3::staticAdjustInputs(alloc, ins) &&
-               Policy4::staticAdjustInputs(alloc, ins);
+    static MOZ_MUST_USE bool staticAdjustInputs(TempAllocator& alloc, MInstruction* ins) {
+        return MixPolicy::staticAdjustInputsHelper<Policies...>(alloc, ins);
     }
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override {
         return staticAdjustInputs(alloc, ins);
     }
 };
@@ -460,7 +441,7 @@ class CallSetElementPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 // First operand will be boxed to a Value (except for an object)
@@ -469,7 +450,7 @@ class InstanceOfPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 class StoreTypedArrayHolePolicy;
@@ -478,36 +459,37 @@ class StoreTypedArrayElementStaticPolicy;
 class StoreUnboxedScalarPolicy : public TypePolicy
 {
   private:
-    static bool adjustValueInput(TempAllocator& alloc, MInstruction* ins, Scalar::Type arrayType,
-                                 MDefinition* value, int valueOperand);
+    static MOZ_MUST_USE bool adjustValueInput(TempAllocator& alloc, MInstruction* ins,
+                                              Scalar::Type arrayType, MDefinition* value,
+                                              int valueOperand);
 
     friend class StoreTypedArrayHolePolicy;
     friend class StoreTypedArrayElementStaticPolicy;
 
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class StoreTypedArrayHolePolicy final : public StoreUnboxedScalarPolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class StoreTypedArrayElementStaticPolicy final : public StoreUnboxedScalarPolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class StoreUnboxedObjectOrNullPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* def) override;
 };
 
 // Accepts integers and doubles. Everything else is boxed.
@@ -515,14 +497,14 @@ class ClampPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 class FilterTypeSetPolicy final : public TypePolicy
 {
   public:
     EMPTY_DATA_;
-    virtual bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
+    virtual MOZ_MUST_USE bool adjustInputs(TempAllocator& alloc, MInstruction* ins) override;
 };
 
 #undef SPECIALIZATION_DATA_
