@@ -41,11 +41,9 @@
 #include "wasm/WasmCode.h"
 
 namespace js {
-
-class WasmActivation;
-
 namespace jit {
 
+class JitActivation;
 class Simulator;
 class Redirection;
 class CachePage;
@@ -293,7 +291,7 @@ class Simulator
 
     // Handle a wasm interrupt triggered by an async signal handler.
     void handleWasmInterrupt();
-    void startInterrupt(WasmActivation* act);
+    void startWasmInterrupt(JitActivation* act);
 
     // Handle any wasm faults, returning true if the fault was handled.
     bool handleWasmFault(int32_t addr, unsigned numBytes);
@@ -426,7 +424,6 @@ class Simulator
 
     // wasm async interrupt / fault support
     bool wasm_interrupt_;
-    wasm::SharedCode wasm_code_;
 
     // Debugger input.
     char* lastDebuggerInput_;
