@@ -631,12 +631,6 @@ Assembler::FixupNurseryObjects(JSContext* cx, JitCode* code, CompactBufferReader
         cx->zone()->group()->storeBuffer().putWholeCell(code);
 }
 
-void
-Assembler::PatchInstructionImmediate(uint8_t* code, PatchedImmPtr imm)
-{
-    MOZ_CRASH("PatchInstructionImmediate()");
-}
-
 void
 Assembler::retarget(Label* label, Label* target)
 {
