@@ -80,7 +80,6 @@ struct SecondScratchRegisterScope : public AutoRegisterScope
 
 // Use arg reg from EnterJIT function as OsrFrameReg.
 static constexpr Register OsrFrameReg = a3;
-static constexpr Register ArgumentsRectifierReg = s3;
 static constexpr Register CallTempReg0 = t0;
 static constexpr Register CallTempReg1 = t1;
 static constexpr Register CallTempReg2 = t2;
@@ -733,7 +732,8 @@ PatchJump(CodeLocationJump& jump_, CodeLocationLabel label,
 void
 PatchBackedge(CodeLocationJump& jump_, CodeLocationLabel label, JitZoneGroup::BackedgeTarget target);
 
-typedef js::jit::AssemblerBuffer<1024, Instruction> MIPSBuffer;
+static constexpr int32_t SliceSize = 1024;
+typedef js::jit::AssemblerBuffer<SliceSize, Instruction> MIPSBuffer;
 
 class MIPSBufferWithExecutableCopy : public MIPSBuffer
 {
@@ -748,16 +748,16 @@ class MIPSBufferWithExecutableCopy : public MIPSBuffer
         }
     }
 
-    bool appendBuffer(const MIPSBufferWithExecutableCopy& other) {
+    bool appendRawCode(const uint8_t* code, size_t numBytes) {
         if (this->oom())
             return false;
-
-        for (Slice* cur = other.head; cur != nullptr; cur = cur->getNext()) {
-            this->putBytes(cur->length(), &cur->instructions);
-            if (this->oom())
-                return false;
+        while (numBytes > SliceSize) {
+            this->putBytes(SliceSize, code);
+            numBytes -= SliceSize;
+            code += SliceSize;
         }
-        return true;
+        this->putBytes(numBytes, code);
+        return !this->oom();
     }
 };
 
@@ -958,7 +958,9 @@ class AssemblerMIPSShared : public AssemblerShared
     bool isFinished;
   public:
     void finish();
-    bool asmMergeWith(const AssemblerMIPSShared& other);
+    bool appendRawCode(const uint8_t* code, size_t numBytes);
+    bool reserve(size_t size);
+    bool swapBuffer(wasm::Bytes& bytes);
     void executableCopy(void* buffer, bool flushICache = true);
     void copyJumpRelocationTable(uint8_t* dest);
     void copyDataRelocationTable(uint8_t* dest);
@@ -1233,7 +1235,6 @@ class AssemblerMIPSShared : public AssemblerShared
     void bind(Label* label, BufferOffset boff = BufferOffset());
     void bindLater(Label* label, wasm::TrapDesc target);
     virtual void bind(InstImm* inst, uintptr_t branch, uintptr_t target) = 0;
-    virtual void Bind(uint8_t* rawCode, CodeOffset* label, const void* address) = 0;
     void bind(CodeOffset* label) {
         label->bind(currentOffset());
     }
@@ -1245,9 +1246,6 @@ class AssemblerMIPSShared : public AssemblerShared
     }
     void retarget(Label* label, Label* target);
 
-    // See Bind
-    size_t labelToPatchOffset(CodeOffset label) { return label.offset(); }
-
     void call(Label* label);
     void call(void* target);
 
@@ -1316,8 +1314,6 @@ class AssemblerMIPSShared : public AssemblerShared
 
     static void UpdateLuiOriValue(Instruction* inst0, Instruction* inst1, uint32_t value);
 
-    void processCodeLabels(uint8_t* rawCode);
-
     bool bailed() {
         return m_buffer.bail();
     }
@@ -1562,7 +1558,15 @@ class InstGS : public Instruction
 inline bool
 IsUnaligned(const wasm::MemoryAccessDesc& access)
 {
-    return access.align() && access.align() < access.byteSize();
+    if (!access.align())
+        return false;
+
+#ifdef JS_CODEGEN_MIPS32
+    if (access.type() == Scalar::Int64 && access.align() >= 4)
+        return false;
+#endif
+
+    return access.align() < access.byteSize();
 }
 
 } // namespace jit
