@@ -71,13 +71,27 @@ MacroAssemblerMIPSShared::ma_sra(Register rd, Register rt, Imm32 shift)
 void
 MacroAssemblerMIPSShared::ma_ror(Register rd, Register rt, Imm32 shift)
 {
-    as_rotr(rd, rt, shift.value % 32);
+    if (hasR2()) {
+        as_rotr(rd, rt, shift.value % 32);
+    } else {
+        ScratchRegisterScope scratch(asMasm());
+        as_srl(scratch, rt, shift.value % 32);
+        as_sll(rd, rt, (32 - (shift.value % 32)) % 32);
+        as_or(rd, rd, scratch);
+    }
 }
 
 void
 MacroAssemblerMIPSShared::ma_rol(Register rd, Register rt, Imm32 shift)
 {
-    as_rotr(rd, rt, 32 - (shift.value % 32));
+    if (hasR2()) {
+        as_rotr(rd, rt, (32 - (shift.value % 32)) % 32);
+    } else {
+        ScratchRegisterScope scratch(asMasm());
+        as_srl(scratch, rt, (32 - (shift.value % 32)) % 32);
+        as_sll(rd, rt, shift.value % 32);
+        as_or(rd, rd, scratch);
+    }
 }
 
 void
@@ -101,14 +115,29 @@ MacroAssemblerMIPSShared::ma_sra(Register rd, Register rt, Register shift)
 void
 MacroAssemblerMIPSShared::ma_ror(Register rd, Register rt, Register shift)
 {
-    as_rotrv(rd, rt, shift);
+    if (hasR2()) {
+        as_rotrv(rd, rt, shift);
+    } else {
+        ScratchRegisterScope scratch(asMasm());
+        ma_negu(scratch, shift);
+        as_sllv(scratch, rt, scratch);
+        as_srlv(rd, rt, shift);
+        as_or(rd, rd, scratch);
+    }
 }
 
 void
 MacroAssemblerMIPSShared::ma_rol(Register rd, Register rt, Register shift)
 {
-    ma_negu(ScratchRegister, shift);
-    as_rotrv(rd, rt, ScratchRegister);
+    ScratchRegisterScope scratch(asMasm());
+    ma_negu(scratch, shift);
+    if (hasR2()) {
+        as_rotrv(rd, rt, scratch);
+    } else {
+        as_srlv(rd, rt, scratch);
+        as_sllv(scratch, rt, shift);
+        as_or(rd, rd, scratch);
+    }
 }
 
 void
@@ -123,6 +152,69 @@ MacroAssemblerMIPSShared::ma_not(Register rd, Register rs)
     as_nor(rd, rs, zero);
 }
 
+// Bit extract/insert
+void
+MacroAssemblerMIPSShared::ma_ext(Register rt, Register rs, uint16_t pos, uint16_t size) {
+    MOZ_ASSERT(pos < 32);
+    MOZ_ASSERT(pos + size < 33);
+
+    if (hasR2()) {
+        as_ext(rt, rs, pos, size);
+    } else {
+        int shift_left = 32 - (pos + size);
+        as_sll(rt, rs, shift_left);
+        int shift_right = 32 - size;
+        if (shift_right > 0) {
+            as_srl(rt, rt, shift_right);
+        }
+    }
+}
+
+void
+MacroAssemblerMIPSShared::ma_ins(Register rt, Register rs, uint16_t pos, uint16_t size) {
+    MOZ_ASSERT(pos < 32);
+    MOZ_ASSERT(pos + size <= 32);
+    MOZ_ASSERT(size != 0);
+
+    if (hasR2()) {
+        as_ins(rt, rs, pos, size);
+    } else {
+        ScratchRegisterScope scratch(asMasm());
+        SecondScratchRegisterScope scratch2(asMasm());
+        ma_subu(scratch, zero, Imm32(1));
+        as_srl(scratch, scratch, 32 - size);
+        as_and(scratch2, rs, scratch);
+        as_sll(scratch2, scratch2, pos);
+        as_sll(scratch, scratch, pos);
+        as_nor(scratch, scratch, zero);
+        as_and(scratch, rt, scratch);
+        as_or(rt, scratch2, scratch);
+    }
+}
+
+// Sign extend
+void
+MacroAssemblerMIPSShared::ma_seb(Register rd, Register rt)
+{
+    if (hasR2()) {
+        as_seb(rd, rt);
+    } else {
+        as_sll(rd, rt, 24);
+        as_sra(rd, rd, 24);
+    }
+}
+
+void
+MacroAssemblerMIPSShared::ma_seh(Register rd, Register rt)
+{
+    if (hasR2()) {
+        as_seh(rd, rt);
+    } else {
+        as_sll(rd, rt, 16);
+        as_sra(rd, rd, 16);
+    }
+}
+
 // And.
 void
 MacroAssemblerMIPSShared::ma_and(Register rd, Register rs)
@@ -457,7 +549,7 @@ MacroAssemblerMIPSShared::ma_load(Register dest, const BaseIndex& src,
 }
 
 void
-MacroAssemblerMIPSShared::ma_load_unaligned(Register dest, const BaseIndex& src, Register temp,
+MacroAssemblerMIPSShared::ma_load_unaligned(const wasm::MemoryAccessDesc& access, Register dest, const BaseIndex& src, Register temp,
                                             LoadStoreSize size, LoadStoreExtension extension)
 {
     int16_t lowOffset, hiOffset;
@@ -471,36 +563,41 @@ MacroAssemblerMIPSShared::ma_load_unaligned(Register dest, const BaseIndex& src,
         hiOffset = Imm16(src.offset + size / 8 - 1).encode();
     } else {
         ma_li(ScratchRegister, Imm32(src.offset));
-        as_daddu(ScratchRegister, SecondScratchReg, ScratchRegister);
+        asMasm().addPtr(SecondScratchReg, ScratchRegister);
         base = ScratchRegister;
         lowOffset = Imm16(0).encode();
         hiOffset = Imm16(size / 8 - 1).encode();
     }
 
+    BufferOffset load;
     switch (size) {
       case SizeHalfWord:
-        as_lbu(dest, base, lowOffset);
         if (extension != ZeroExtend)
-            as_lbu(temp, base, hiOffset);
+            load = as_lbu(temp, base, hiOffset);
         else
-            as_lb(temp, base, hiOffset);
-        as_ins(dest, temp, 8, 24);
+            load = as_lb(temp, base, hiOffset);
+        as_lbu(dest, base, lowOffset);
+        ma_ins(dest, temp, 8, 24);
         break;
       case SizeWord:
-        as_lwl(dest, base, hiOffset);
+        load = as_lwl(dest, base, hiOffset);
         as_lwr(dest, base, lowOffset);
 #ifdef JS_CODEGEN_MIPS64
         if (extension != ZeroExtend)
             as_dext(dest, dest, 0, 32);
 #endif
         break;
+#ifdef JS_CODEGEN_MIPS64
       case SizeDouble:
-        as_ldl(dest, base, hiOffset);
+        load = as_ldl(dest, base, hiOffset);
         as_ldr(dest, base, lowOffset);
         break;
+#endif
       default:
         MOZ_CRASH("Invalid argument for ma_load");
     }
+
+    append(access, load.getOffset(), asMasm().framePushed());
 }
 
 void
@@ -604,7 +701,7 @@ MacroAssemblerMIPSShared::ma_store(Imm32 imm, const BaseIndex& dest,
 }
 
 void
-MacroAssemblerMIPSShared::ma_store_unaligned(Register data, const BaseIndex& dest, Register temp,
+MacroAssemblerMIPSShared::ma_store_unaligned(const wasm::MemoryAccessDesc& access, Register data, const BaseIndex& dest, Register temp,
                                              LoadStoreSize size, LoadStoreExtension extension)
 {
     int16_t lowOffset, hiOffset;
@@ -618,29 +715,33 @@ MacroAssemblerMIPSShared::ma_store_unaligned(Register data, const BaseIndex& des
         hiOffset = Imm16(dest.offset + size / 8 - 1).encode();
     } else {
         ma_li(ScratchRegister, Imm32(dest.offset));
-        as_daddu(ScratchRegister, SecondScratchReg, ScratchRegister);
+        asMasm().addPtr(SecondScratchReg, ScratchRegister);
         base = ScratchRegister;
         lowOffset = Imm16(0).encode();
         hiOffset = Imm16(size / 8 - 1).encode();
     }
 
+    BufferOffset store;
     switch (size) {
       case SizeHalfWord:
+        ma_ext(temp, data, 8, 8);
+        store = as_sb(temp, base, hiOffset);
         as_sb(data, base, lowOffset);
-        as_ext(temp, data, 8, 8);
-        as_sb(temp, base, hiOffset);
         break;
       case SizeWord:
-        as_swl(data, base, hiOffset);
+        store = as_swl(data, base, hiOffset);
         as_swr(data, base, lowOffset);
         break;
+#ifdef JS_CODEGEN_MIPS64
       case SizeDouble:
-        as_sdl(data, base, hiOffset);
+        store = as_sdl(data, base, hiOffset);
         as_sdr(data, base, lowOffset);
         break;
+#endif
       default:
         MOZ_CRASH("Invalid argument for ma_store");
     }
+    append(access, store.getOffset(), asMasm().framePushed());
 }
 
 // Branches when done from within mips-specific code.
@@ -1243,10 +1344,10 @@ MacroAssemblerMIPSShared::atomicFetchOpMIPSr2(int nbytes, bool signExtend, Atomi
         if (signExtend) {
             switch (nbytes) {
             case 1:
-                as_seb(output, output);
+                ma_seb(output, output);
                 break;
             case 2:
-                as_seh(output, output);
+                ma_seh(output, output);
                 break;
             case 4:
                 break;
@@ -1418,10 +1519,10 @@ MacroAssemblerMIPSShared::compareExchangeMIPSr2(int nbytes, bool signExtend, con
     if (signExtend) {
         switch (nbytes) {
         case 1:
-            as_seb(output, output);
+            ma_seb(output, output);
             break;
         case 2:
-            as_seh(output, output);
+            ma_seh(output, output);
             break;
         case 4:
             break;
@@ -1551,7 +1652,7 @@ MacroAssembler::Pop(const ValueOperand& val)
 void
 MacroAssembler::PopStackPtr()
 {
-    asMasm().ma_load(StackPointer, Address(StackPointer, 0), SizeWord);
+    loadPtr(Address(StackPointer, 0), StackPointer);
     framePushed_ -= sizeof(intptr_t);
 }
 
@@ -1769,7 +1870,7 @@ MacroAssembler::wasmTruncateDoubleToInt32(FloatRegister input, Register output,
     as_truncwd(ScratchFloat32Reg, input);
     as_cfc1(ScratchRegister, Assembler::FCSR);
     moveFromFloat32(ScratchFloat32Reg, output);
-    as_ext(ScratchRegister, ScratchRegister, 6, 1);
+    ma_ext(ScratchRegister, ScratchRegister, 6, 1);
     ma_b(ScratchRegister, Imm32(0), oolEntry, Assembler::NotEqual);
 }
 
@@ -1780,7 +1881,7 @@ MacroAssembler::wasmTruncateFloat32ToInt32(FloatRegister input, Register output,
     as_truncws(ScratchFloat32Reg, input);
     as_cfc1(ScratchRegister, Assembler::FCSR);
     moveFromFloat32(ScratchFloat32Reg, output);
-    as_ext(ScratchRegister, ScratchRegister, 6, 1);
+    ma_ext(ScratchRegister, ScratchRegister, 6, 1);
     ma_b(ScratchRegister, Imm32(0), oolEntry, Assembler::NotEqual);
 }
 
