@@ -271,50 +271,6 @@ MacroAssemblerX86Shared::getSimdData(const SimdConstant& v)
     return getConstant<SimdData, SimdMap>(v, simdMap_, simds_);
 }
 
-template<class T, class Map>
-static bool
-MergeConstants(size_t delta, const Vector<T, 0, SystemAllocPolicy>& other,
-               Map& map, Vector<T, 0, SystemAllocPolicy>& vec)
-{
-    typedef typename Map::AddPtr AddPtr;
-    if (!map.initialized() && !map.init())
-        return false;
-
-    for (const T& c : other) {
-        size_t index;
-        if (AddPtr p = map.lookupForAdd(c.value)) {
-            index = p->value();
-        } else {
-            index = vec.length();
-            if (!vec.append(T(c.value)) || !map.add(p, c.value, index))
-                return false;
-        }
-        MacroAssemblerX86Shared::UsesVector& uses = vec[index].uses;
-        for (CodeOffset use : c.uses) {
-            use.offsetBy(delta);
-            if (!uses.append(use))
-                return false;
-        }
-    }
-
-    return true;
-}
-
-bool
-MacroAssemblerX86Shared::asmMergeWith(const MacroAssemblerX86Shared& other)
-{
-    size_t sizeBefore = masm.size();
-    if (!Assembler::asmMergeWith(other))
-        return false;
-    if (!MergeConstants<Double, DoubleMap>(sizeBefore, other.doubles_, doubleMap_, doubles_))
-        return false;
-    if (!MergeConstants<Float, FloatMap>(sizeBefore, other.floats_, floatMap_, floats_))
-        return false;
-    if (!MergeConstants<SimdData, SimdMap>(sizeBefore, other.simds_, simdMap_, simds_))
-        return false;
-    return true;
-}
-
 void
 MacroAssemblerX86Shared::minMaxDouble(FloatRegister first, FloatRegister second, bool canBeNaN,
                                       bool isMax)
@@ -461,6 +417,44 @@ MacroAssembler::PushRegsInMask(LiveRegisterSet set)
     MOZ_ASSERT(diffF == 0);
 }
 
+void
+MacroAssembler::storeRegsInMask(LiveRegisterSet set, Address dest, Register)
+{
+    FloatRegisterSet fpuSet(set.fpus().reduceSetForPush());
+    unsigned numFpu = fpuSet.size();
+    int32_t diffF = fpuSet.getPushSizeInBytes();
+    int32_t diffG = set.gprs().size() * sizeof(intptr_t);
+
+    MOZ_ASSERT(dest.offset >= diffG + diffF);
+
+    for (GeneralRegisterBackwardIterator iter(set.gprs()); iter.more(); ++iter) {
+        diffG -= sizeof(intptr_t);
+        dest.offset -= sizeof(intptr_t);
+        storePtr(*iter, dest);
+    }
+    MOZ_ASSERT(diffG == 0);
+
+    for (FloatRegisterBackwardIterator iter(fpuSet); iter.more(); ++iter) {
+        FloatRegister reg = *iter;
+        diffF -= reg.size();
+        numFpu -= 1;
+        dest.offset -= reg.size();
+        if (reg.isDouble())
+            storeDouble(reg, dest);
+        else if (reg.isSingle())
+            storeFloat32(reg, dest);
+        else if (reg.isSimd128())
+            storeUnalignedSimd128Float(reg, dest);
+        else
+            MOZ_CRASH("Unknown register type.");
+    }
+    MOZ_ASSERT(numFpu == 0);
+    // x64 padding to keep the stack aligned on uintptr_t. Keep in sync with
+    // GetPushBytesInSize.
+    diffF -= diffF % sizeof(uintptr_t);
+    MOZ_ASSERT(diffF == 0);
+}
+
 void
 MacroAssembler::PopRegsInMaskIgnore(LiveRegisterSet set, LiveRegisterSet ignore)
 {
@@ -562,6 +556,13 @@ MacroAssembler::Push(FloatRegister t)
     adjustFrame(sizeof(double));
 }
 
+void
+MacroAssembler::PushFlags()
+{
+    pushFlags();
+    adjustFrame(sizeof(intptr_t));
+}
+
 void
 MacroAssembler::Pop(const Operand op)
 {
@@ -590,6 +591,19 @@ MacroAssembler::Pop(const ValueOperand& val)
     implicitPop(sizeof(Value));
 }
 
+void
+MacroAssembler::PopFlags()
+{
+    popFlags();
+    implicitPop(sizeof(intptr_t));
+}
+
+void
+MacroAssembler::PopStackPtr()
+{
+    Pop(StackPointer);
+}
+
 // ===============================================================
 // Simple call functions.
 
@@ -647,22 +661,37 @@ MacroAssembler::patchCall(uint32_t callerOffset, uint32_t calleeOffset)
     Assembler::patchCall(callerOffset, calleeOffset);
 }
 
+void
+MacroAssembler::callAndPushReturnAddress(Register reg)
+{
+    call(reg);
+}
+
+void
+MacroAssembler::callAndPushReturnAddress(Label* label)
+{
+    call(label);
+}
+
+// ===============================================================
+// Patchable near/far jumps.
+
 CodeOffset
-MacroAssembler::thunkWithPatch()
+MacroAssembler::farJumpWithPatch()
 {
-    return Assembler::thunkWithPatch();
+    return Assembler::farJumpWithPatch();
 }
 
 void
-MacroAssembler::patchThunk(uint32_t thunkOffset, uint32_t targetOffset)
+MacroAssembler::patchFarJump(CodeOffset farJump, uint32_t targetOffset)
 {
-    Assembler::patchThunk(thunkOffset, targetOffset);
+    Assembler::patchFarJump(farJump, targetOffset);
 }
 
 void
-MacroAssembler::repatchThunk(uint8_t* code, uint32_t thunkOffset, uint32_t targetOffset)
+MacroAssembler::repatchFarJump(uint8_t* code, uint32_t farJumpOffset, uint32_t targetOffset)
 {
-    Assembler::repatchThunk(code, thunkOffset, targetOffset);
+    Assembler::repatchFarJump(code, farJumpOffset, targetOffset);
 }
 
 CodeOffset
@@ -683,16 +712,26 @@ MacroAssembler::patchNearJumpToNop(uint8_t* jump)
     Assembler::patchJumpToTwoByteNop(jump);
 }
 
-void
-MacroAssembler::callAndPushReturnAddress(Register reg)
+CodeOffset
+MacroAssembler::nopPatchableToCall(const wasm::CallSiteDesc& desc)
 {
-    call(reg);
+    CodeOffset offset(currentOffset());
+    masm.nop_five();
+    append(desc, CodeOffset(currentOffset()));
+    MOZ_ASSERT_IF(!oom(), size() - offset.offset() == ToggledCallSize(nullptr));
+    return offset;
 }
 
 void
-MacroAssembler::callAndPushReturnAddress(Label* label)
+MacroAssembler::patchNopToCall(uint8_t* callsite, uint8_t* target)
 {
-    call(label);
+    Assembler::patchFiveByteNopToCall(callsite, target);
+}
+
+void
+MacroAssembler::patchCallToNop(uint8_t* callsite)
+{
+    Assembler::patchCallToFiveByteNop(callsite);
 }
 
 // ===============================================================
@@ -721,18 +760,19 @@ struct MOZ_RAII AutoHandleWasmTruncateToIntErrors
     MacroAssembler& masm;
     Label inputIsNaN;
     Label fail;
+    wasm::BytecodeOffset off;
 
-    explicit AutoHandleWasmTruncateToIntErrors(MacroAssembler& masm)
-      : masm(masm)
+    explicit AutoHandleWasmTruncateToIntErrors(MacroAssembler& masm, wasm::BytecodeOffset off)
+      : masm(masm), off(off)
     { }
 
     ~AutoHandleWasmTruncateToIntErrors() {
         // Handle errors.
         masm.bind(&fail);
-        masm.jump(wasm::JumpTarget::IntegerOverflow);
+        masm.jump(wasm::TrapDesc(off, wasm::Trap::IntegerOverflow, masm.framePushed()));
 
         masm.bind(&inputIsNaN);
-        masm.jump(wasm::JumpTarget::InvalidConversionToInteger);
+        masm.jump(wasm::TrapDesc(off, wasm::Trap::InvalidConversionToInteger, masm.framePushed()));
     }
 };
 
@@ -754,9 +794,9 @@ MacroAssembler::wasmTruncateFloat32ToInt32(FloatRegister input, Register output,
 
 void
 MacroAssembler::outOfLineWasmTruncateDoubleToInt32(FloatRegister input, bool isUnsigned,
-                                                   Label* rejoin)
+                                                   wasm::BytecodeOffset off, Label* rejoin)
 {
-    AutoHandleWasmTruncateToIntErrors traps(*this);
+    AutoHandleWasmTruncateToIntErrors traps(*this, off);
 
     // Eagerly take care of NaNs.
     branchDouble(Assembler::DoubleUnordered, input, input, &traps.inputIsNaN);
@@ -777,9 +817,9 @@ MacroAssembler::outOfLineWasmTruncateDoubleToInt32(FloatRegister input, bool isU
 
 void
 MacroAssembler::outOfLineWasmTruncateFloat32ToInt32(FloatRegister input, bool isUnsigned,
-                                                    Label* rejoin)
+                                                    wasm::BytecodeOffset off, Label* rejoin)
 {
-    AutoHandleWasmTruncateToIntErrors traps(*this);
+    AutoHandleWasmTruncateToIntErrors traps(*this, off);
 
     // Eagerly take care of NaNs.
     branchFloat(Assembler::DoubleUnordered, input, input, &traps.inputIsNaN);
@@ -798,9 +838,9 @@ MacroAssembler::outOfLineWasmTruncateFloat32ToInt32(FloatRegister input, bool is
 
 void
 MacroAssembler::outOfLineWasmTruncateDoubleToInt64(FloatRegister input, bool isUnsigned,
-                                                   Label* rejoin)
+                                                   wasm::BytecodeOffset off, Label* rejoin)
 {
-    AutoHandleWasmTruncateToIntErrors traps(*this);
+    AutoHandleWasmTruncateToIntErrors traps(*this, off);
 
     // Eagerly take care of NaNs.
     branchDouble(Assembler::DoubleUnordered, input, input, &traps.inputIsNaN);
@@ -825,9 +865,9 @@ MacroAssembler::outOfLineWasmTruncateDoubleToInt64(FloatRegister input, bool isU
 
 void
 MacroAssembler::outOfLineWasmTruncateFloat32ToInt64(FloatRegister input, bool isUnsigned,
-                                                    Label* rejoin)
+                                                    wasm::BytecodeOffset off, Label* rejoin)
 {
-    AutoHandleWasmTruncateToIntErrors traps(*this);
+    AutoHandleWasmTruncateToIntErrors traps(*this, off);
 
     // Eagerly take care of NaNs.
     branchFloat(Assembler::DoubleUnordered, input, input, &traps.inputIsNaN);
