@@ -142,10 +142,7 @@ struct JSContext : public JS::RootingContext,
     /* Clear the pending exception (if any) due to OOM. */
     void recoverFromOutOfMemory();
 
-    inline void updateMallocCounter(size_t nbytes) {
-        // Note: this is racy.
-        runtime()->updateMallocCounter(zone(), nbytes);
-    }
+    void updateMallocCounter(size_t nbytes);
 
     void reportAllocationOverflow() {
         js::ReportAllocationOverflow(this);
@@ -377,7 +374,11 @@ struct JSContext : public JS::RootingContext,
     }
     static size_t offsetOfProfilingActivation() {
         return offsetof(JSContext, profilingActivation_);
-     }
+    }
+
+    static size_t offsetOfJitActivation() {
+        return offsetof(JSContext, jitActivation);
+    }
 
 #ifdef DEBUG
     static size_t offsetOfInUnsafeCallWithABI() {
@@ -613,13 +614,6 @@ struct JSContext : public JS::RootingContext,
 
     js::ThreadLocalData<uint32_t> debuggerMutations;
 
-    /*
-     * The propertyRemovals counter is incremented for every JSObject::clear,
-     * and for each JSObject::remove method call that frees a slot in the given
-     * object. See js_NativeGet and js_NativeSet in jsobj.cpp.
-     */
-    js::ThreadLocalData<uint32_t> propertyRemovals;
-
     // Cache for jit::GetPcScript().
     js::ThreadLocalData<js::jit::PcScriptCache*> ionPcScriptCache;
 
@@ -1197,9 +1191,6 @@ class AutoAssertNoException
     }
 };
 
-/* Exposed intrinsics for the JITs. */
-bool intrinsic_IsSuspendedStarGenerator(JSContext* cx, unsigned argc, Value* vp);
-
 class MOZ_RAII AutoLockForExclusiveAccess
 {
     JSRuntime* runtime;
