@@ -20,7 +20,8 @@
 #include "gc/Barrier.h"
 #include "gc/NurseryAwareHashMap.h"
 #include "gc/Zone.h"
-#include "vm/PIC.h"
+#include "vm/ArrayBufferObject.h"
+#include "vm/GlobalObject.h"
 #include "vm/ReceiverGuard.h"
 #include "vm/RegExpShared.h"
 #include "vm/SavedStacks.h"
@@ -546,7 +547,6 @@ struct IteratorHashPolicy
 namespace js {
 class DebugEnvironments;
 class ObjectWeakMap;
-class WatchpointMap;
 class WeakMapBase;
 } // namespace js
 
@@ -623,11 +623,8 @@ struct JSCompartment
   public:
     bool                         isSelfHosting;
     bool                         marked;
-    bool                         warnedAboutDateToLocaleFormat : 1;
     bool                         warnedAboutExprClosure : 1;
     bool                         warnedAboutForEach : 1;
-    bool                         warnedAboutLegacyGenerator : 1;
-    bool                         warnedAboutObjectWatch : 1;
     uint32_t                     warnedAboutStringGenericsMethods;
 
 #ifdef DEBUG
@@ -672,7 +669,7 @@ struct JSCompartment
     const JS::CompartmentBehaviors& behaviors() const { return behaviors_; }
 
     JSRuntime* runtimeFromActiveCooperatingThread() const {
-        MOZ_ASSERT(CurrentThreadCanAccessRuntime(runtime_));
+        MOZ_ASSERT(js::CurrentThreadCanAccessRuntime(runtime_));
         return runtime_;
     }
 
@@ -968,7 +965,6 @@ struct JSCompartment
     void sweepNativeIterators();
     void sweepTemplateObjects();
     void sweepVarNames();
-    void sweepWatchpoints();
 
     void purge();
     void clearTables();
@@ -1010,11 +1006,11 @@ struct JSCompartment
     // and a template object. If a template object is found in template
     // registry, that object is returned. Otherwise, the passed-in templateObj
     // is added to the registry.
-    bool getTemplateLiteralObject(JSContext* cx, js::HandleObject rawStrings,
+    bool getTemplateLiteralObject(JSContext* cx, js::HandleArrayObject rawStrings,
                                   js::MutableHandleObject templateObj);
 
     // Per above, but an entry must already exist in the template registry.
-    JSObject* getExistingTemplateLiteralObject(JSObject* rawStrings);
+    JSObject* getExistingTemplateLiteralObject(js::ArrayObject* rawStrings);
 
     void findOutgoingEdges(js::gc::ZoneComponentFinder& finder);
 
@@ -1157,8 +1153,6 @@ struct JSCompartment
     void sweepBreakpoints(js::FreeOp* fop);
 
   public:
-    js::WatchpointMap* watchpointMap;
-
     js::ScriptCountsMap* scriptCountsMap;
     js::ScriptNameMap* scriptNameMap;
 
