@@ -9,6 +9,7 @@
 #include "jsiter.h"
 
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/DebugOnly.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/PodOperations.h"
@@ -17,7 +18,6 @@
 #include "jsarray.h"
 #include "jsatom.h"
 #include "jscntxt.h"
-#include "jsgc.h"
 #include "jsobj.h"
 #include "jsopcode.h"
 #include "jsscript.h"
@@ -31,7 +31,6 @@
 #include "vm/GlobalObject.h"
 #include "vm/Interpreter.h"
 #include "vm/Shape.h"
-#include "vm/StopIterationObject.h"
 #include "vm/TypedArrayObject.h"
 
 #include "jsscriptinlines.h"
@@ -46,6 +45,7 @@ using namespace js::gc;
 using JS::ForOfIterator;
 
 using mozilla::ArrayLength;
+using mozilla::DebugOnly;
 using mozilla::Maybe;
 using mozilla::PodCopy;
 using mozilla::PodEqual;
@@ -73,12 +73,6 @@ NativeIterator::trace(JSTracer* trc)
 
 typedef HashSet<jsid, DefaultHasher<jsid>> IdSet;
 
-static inline bool
-NewKeyValuePair(JSContext* cx, jsid id, const Value& val, MutableHandleValue rval)
-{
-    return NewValuePair(cx, IdToValue(id), val, rval);
-}
-
 template <bool CheckForDuplicates>
 static inline bool
 Enumerate(JSContext* cx, HandleObject pobj, jsid id,
@@ -451,7 +445,7 @@ Snapshot(JSContext* cx, HandleObject pobj_, unsigned flags, AutoIdVector* props)
             } else {
                 // The newEnumerate hook may return duplicates. Whitelist the
                 // unboxed object hooks because we know they are well-behaved.
-                if (!pobj->is<UnboxedPlainObject>() && !pobj->is<UnboxedArrayObject>())
+                if (!pobj->is<UnboxedPlainObject>())
                     checkForDuplicates = true;
 
                 if (checkForDuplicates) {
@@ -542,57 +536,6 @@ js::GetPropertyKeys(JSContext* cx, HandleObject obj, unsigned flags, AutoIdVecto
                     props);
 }
 
-size_t sCustomIteratorCount = 0;
-
-static inline bool
-GetCustomIterator(JSContext* cx, HandleObject obj, unsigned flags, MutableHandleObject objp)
-{
-    if (MOZ_UNLIKELY(!CheckRecursionLimit(cx)))
-        return false;
-
-    RootedValue rval(cx);
-    /* Check whether we have a valid __iterator__ method. */
-    HandlePropertyName name = cx->names().iteratorIntrinsic;
-    if (!GetProperty(cx, obj, obj, name, &rval))
-        return false;
-
-    /* If there is no custom __iterator__ method, we are done here. */
-    if (MOZ_LIKELY(!rval.isObject())) {
-        objp.set(nullptr);
-        return true;
-    }
-
-    if (!cx->runningWithTrustedPrincipals())
-        ++sCustomIteratorCount;
-
-    /* Otherwise call it and return that object. */
-    {
-        FixedInvokeArgs<1> args(cx);
-
-        args[0].setBoolean((flags & JSITER_FOREACH) == 0);
-
-        RootedValue thisv(cx, ObjectValue(*obj));
-        if (!js::Call(cx, rval, thisv, args, &rval))
-            return false;
-    }
-
-    if (rval.isPrimitive()) {
-        // Ignore the stack when throwing. We can't tell whether we were
-        // supposed to skip over a new.target or not.
-        JSAutoByteString bytes;
-        if (!AtomToPrintableString(cx, name, &bytes))
-            return false;
-        RootedValue val(cx, ObjectValue(*obj));
-        ReportValueError2(cx, JSMSG_BAD_TRAP_RETURN_VALUE,
-                          JSDVG_IGNORE_STACK, val, nullptr, bytes.ptr());
-        return false;
-    }
-    objp.set(&rval.toObject());
-    return true;
-}
-
-static bool legacy_iterator_next(JSContext* cx, unsigned argc, Value* vp);
-
 static inline PropertyIteratorObject*
 NewPropertyIteratorObject(JSContext* cx, unsigned flags)
 {
@@ -623,26 +566,7 @@ NewPropertyIteratorObject(JSContext* cx, unsigned flags)
         return res;
     }
 
-    Rooted<PropertyIteratorObject*> res(cx, NewBuiltinClassInstance<PropertyIteratorObject>(cx));
-    if (!res)
-        return nullptr;
-
-    if (flags == 0) {
-        // Redefine next as an own property. This ensure that deleting the
-        // next method on the prototype doesn't break cross-global for .. in.
-        // We don't have to do this for JSITER_ENUMERATE because that object always
-        // takes an optimized path.
-        RootedFunction next(cx, NewNativeFunction(cx, legacy_iterator_next, 0,
-                                                  HandlePropertyName(cx->names().next)));
-        if (!next)
-            return nullptr;
-
-        RootedValue value(cx, ObjectValue(*next));
-        if (!DefineProperty(cx, res, cx->names().next, value))
-            return nullptr;
-    }
-
-    return res;
+    return NewBuiltinClassInstance<PropertyIteratorObject>(cx);
 }
 
 NativeIterator*
@@ -926,9 +850,6 @@ CanStoreInIteratorCache(JSContext* cx, JSObject* obj)
                 return false;
             if (MOZ_UNLIKELY(clasp->getNewEnumerate() || clasp->getEnumerate()))
                 return false;
-
-            if (MOZ_UNLIKELY(obj->as<NativeObject>().containsPure(cx->names().iteratorIntrinsic)))
-                return false;
         } else {
             MOZ_ASSERT(obj->is<UnboxedPlainObject>());
         }
@@ -985,7 +906,7 @@ js::GetIterator(JSContext* cx, HandleObject obj, unsigned flags)
             numGuards = 0;
     }
 
-    if (MOZ_UNLIKELY(obj->is<PropertyIteratorObject>() || obj->is<LegacyGeneratorObject>()))
+    if (MOZ_UNLIKELY(obj->is<PropertyIteratorObject>()))
         return obj;
 
     // We should only call the enumerate trap for "for-in".
@@ -997,18 +918,12 @@ js::GetIterator(JSContext* cx, HandleObject obj, unsigned flags)
             return Proxy::enumerate(cx, obj);
     }
 
-    RootedObject res(cx);
-    if (!GetCustomIterator(cx, obj, flags, &res))
-        return nullptr;
-    if (res) {
-        assertSameCompartment(cx, res);
-        return res;
-    }
 
     AutoIdVector keys(cx);
     if (!Snapshot(cx, obj, flags, &keys))
         return nullptr;
 
+    JSObject* res;
     if (flags & JSITER_FOREACH) {
         MOZ_ASSERT(numGuards == 0);
         res = VectorToValueIterator(cx, obj, flags, keys);
@@ -1046,68 +961,85 @@ js::CreateIterResultObject(JSContext* cx, HandleValue value, bool done)
     // Step 1 (implicit).
 
     // Step 2.
-    RootedObject resultObj(cx, NewBuiltinClassInstance<PlainObject>(cx));
-    if (!resultObj)
+    RootedObject templateObject(cx, cx->compartment()->getOrCreateIterResultTemplateObject(cx));
+    if (!templateObject)
         return nullptr;
 
+    NativeObject* resultObj;
+    JS_TRY_VAR_OR_RETURN_NULL(cx, resultObj, NativeObject::createWithTemplate(cx, gc::DefaultHeap,
+                                                                              templateObject));
+
     // Step 3.
-    if (!DefineProperty(cx, resultObj, cx->names().value, value))
-        return nullptr;
+    resultObj->setSlot(JSCompartment::IterResultObjectValueSlot, value);
 
     // Step 4.
-    if (!DefineProperty(cx, resultObj, cx->names().done,
-                        done ? TrueHandleValue : FalseHandleValue))
-    {
-        return nullptr;
-    }
+    resultObj->setSlot(JSCompartment::IterResultObjectDoneSlot,
+                       done ? TrueHandleValue : FalseHandleValue);
 
     // Step 5.
     return resultObj;
 }
 
-bool
-js::ThrowStopIteration(JSContext* cx)
+NativeObject*
+JSCompartment::getOrCreateIterResultTemplateObject(JSContext* cx)
 {
-    MOZ_ASSERT(!JS_IsExceptionPending(cx));
+    if (iterResultTemplate_)
+        return iterResultTemplate_;
 
-    // StopIteration isn't a constructor, but it's stored in GlobalObject
-    // as one, out of laziness. Hence the GetBuiltinConstructor call here.
-    RootedObject ctor(cx);
-    if (GetBuiltinConstructor(cx, JSProto_StopIteration, &ctor))
-        cx->setPendingException(ObjectValue(*ctor));
-    return false;
+    // Create template plain object
+    RootedNativeObject templateObject(cx, NewBuiltinClassInstance<PlainObject>(cx, TenuredObject));
+    if (!templateObject)
+        return iterResultTemplate_; // = nullptr
+
+    // Create a new group for the template.
+    Rooted<TaggedProto> proto(cx, templateObject->taggedProto());
+    RootedObjectGroup group(cx, ObjectGroupCompartment::makeGroup(cx, templateObject->getClass(),
+                                                                  proto));
+    if (!group)
+        return iterResultTemplate_; // = nullptr
+    templateObject->setGroup(group);
+
+    // Set dummy `value` property
+    if (!NativeDefineDataProperty(cx, templateObject, cx->names().value, UndefinedHandleValue,
+                                  JSPROP_ENUMERATE))
+    {
+        return iterResultTemplate_; // = nullptr
+    }
+
+    // Set dummy `done` property
+    if (!NativeDefineDataProperty(cx, templateObject, cx->names().done, TrueHandleValue,
+                                  JSPROP_ENUMERATE))
+    {
+        return iterResultTemplate_; // = nullptr
+    }
+
+    // Update `value` property typeset, since it can be any value.
+    HeapTypeSet* types = group->maybeGetProperty(NameToId(cx->names().value));
+    MOZ_ASSERT(types);
+    {
+        AutoEnterAnalysis enter(cx);
+        types->makeUnknown(cx);
+    }
+
+    // Make sure that the properties are in the right slots.
+    DebugOnly<Shape*> shape = templateObject->lastProperty();
+    MOZ_ASSERT(shape->previous()->slot() == JSCompartment::IterResultObjectValueSlot &&
+               shape->previous()->propidRef() == NameToId(cx->names().value));
+    MOZ_ASSERT(shape->slot() == JSCompartment::IterResultObjectDoneSlot &&
+               shape->propidRef() == NameToId(cx->names().done));
+
+    iterResultTemplate_.set(templateObject);
+
+    return iterResultTemplate_;
 }
 
 /*** Iterator objects ****************************************************************************/
 
-bool
-js::IteratorConstructor(JSContext* cx, unsigned argc, Value* vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-    if (args.length() == 0) {
-        ReportMissingArg(cx, args.calleev(), 0);
-        return false;
-    }
-
-    bool keyonly = false;
-    if (args.length() >= 2)
-        keyonly = ToBoolean(args[1]);
-    unsigned flags = JSITER_OWNONLY | (keyonly ? 0 : (JSITER_FOREACH | JSITER_KEYVALUE));
-
-    RootedObject iterobj(cx, ValueToIterator(cx, flags, args[0]));
-    if (!iterobj)
-        return false;
-    args.rval().setObject(*iterobj);
-    return true;
-}
-
 MOZ_ALWAYS_INLINE bool
-NativeIteratorNext(JSContext* cx, NativeIterator* ni, MutableHandleValue rval, bool* done)
+NativeIteratorNext(JSContext* cx, NativeIterator* ni, MutableHandleValue rval)
 {
-    *done = false;
-
     if (ni->props_cursor >= ni->props_end) {
-        *done = true;
+        rval.setMagic(JS_NO_ITER_VALUE);
         return true;
     }
 
@@ -1127,54 +1059,15 @@ NativeIteratorNext(JSContext* cx, NativeIterator* ni, MutableHandleValue rval, b
     if (!GetProperty(cx, obj, obj, id, rval))
         return false;
 
-    // JS 1.7 only: for each (let [k, v] in obj)
-    if (ni->flags & JSITER_KEYVALUE)
-        return NewKeyValuePair(cx, id, rval, rval);
     return true;
 }
 
-MOZ_ALWAYS_INLINE bool
-IsIterator(HandleValue v)
+bool
+js::IsPropertyIterator(HandleValue v)
 {
-    return v.isObject() && v.toObject().hasClass(&PropertyIteratorObject::class_);
+    return v.isObject() && v.toObject().is<PropertyIteratorObject>();
 }
 
-MOZ_ALWAYS_INLINE bool
-legacy_iterator_next_impl(JSContext* cx, const CallArgs& args)
-{
-    MOZ_ASSERT(IsIterator(args.thisv()));
-
-    RootedObject thisObj(cx, &args.thisv().toObject());
-
-    NativeIterator* ni = thisObj.as<PropertyIteratorObject>()->getNativeIterator();
-    RootedValue value(cx);
-    bool done;
-    if (!NativeIteratorNext(cx, ni, &value, &done))
-         return false;
-
-    // Use old iterator protocol for compatibility reasons.
-    if (done) {
-        ThrowStopIteration(cx);
-        return false;
-    }
-
-    args.rval().set(value);
-    return true;
-}
-
-static bool
-legacy_iterator_next(JSContext* cx, unsigned argc, Value* vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-    return CallNonGenericMethod<IsIterator, legacy_iterator_next_impl>(cx, args);
-}
-
-static const JSFunctionSpec legacy_iterator_methods[] = {
-    JS_SELF_HOSTED_SYM_FN(iterator, "LegacyIteratorShim", 0, 0),
-    JS_FN("next",      legacy_iterator_next,       0, 0),
-    JS_FS_END
-};
-
 size_t
 PropertyIteratorObject::sizeOfMisc(mozilla::MallocSizeOf mallocSizeOf) const
 {
@@ -1198,8 +1091,6 @@ PropertyIteratorObject::finalize(FreeOp* fop, JSObject* obj)
 const ClassOps PropertyIteratorObject::classOps_ = {
     nullptr, /* addProperty */
     nullptr, /* delProperty */
-    nullptr, /* getProperty */
-    nullptr, /* setProperty */
     nullptr, /* enumerate */
     nullptr, /* newEnumerate */
     nullptr, /* resolve */
@@ -1213,7 +1104,6 @@ const ClassOps PropertyIteratorObject::classOps_ = {
 
 const Class PropertyIteratorObject::class_ = {
     "Iterator",
-    JSCLASS_HAS_CACHED_PROTO(JSProto_Iterator) |
     JSCLASS_HAS_PRIVATE |
     JSCLASS_BACKGROUND_FINALIZE,
     &PropertyIteratorObject::classOps_
@@ -1286,9 +1176,6 @@ js::NewStringIteratorObject(JSContext* cx, NewObjectKind newKind)
 JSObject*
 js::ValueToIterator(JSContext* cx, unsigned flags, HandleValue vp)
 {
-    /* JSITER_KEYVALUE must always come with JSITER_FOREACH */
-    MOZ_ASSERT_IF(flags & JSITER_KEYVALUE, flags & JSITER_FOREACH);
-
     RootedObject obj(cx);
     if (vp.isObject()) {
         /* Common case. */
@@ -1309,8 +1196,8 @@ js::ValueToIterator(JSContext* cx, unsigned flags, HandleValue vp)
     return GetIterator(cx, obj, flags);
 }
 
-bool
-js::CloseIterator(JSContext* cx, HandleObject obj)
+void
+js::CloseIterator(JSObject* obj)
 {
     if (obj->is<PropertyIteratorObject>()) {
         /* Remove enumerators from the active list, which is a stack. */
@@ -1328,32 +1215,7 @@ js::CloseIterator(JSContext* cx, HandleObject obj)
              */
             ni->props_cursor = ni->props_array;
         }
-    } else if (obj->is<LegacyGeneratorObject>()) {
-        Rooted<LegacyGeneratorObject*> genObj(cx, &obj->as<LegacyGeneratorObject>());
-        if (genObj->isClosed())
-            return true;
-        if (genObj->isRunning() || genObj->isClosing()) {
-            // Nothing sensible to do.
-            return true;
-        }
-        return LegacyGeneratorObject::close(cx, obj);
     }
-
-    return true;
-}
-
-bool
-js::UnwindIteratorForException(JSContext* cx, HandleObject obj)
-{
-    RootedValue v(cx);
-    bool getOk = cx->getPendingException(&v);
-    cx->clearPendingException();
-    if (!CloseIterator(cx, obj))
-        return false;
-    if (!getOk)
-        return false;
-    cx->setPendingException(v);
-    return true;
 }
 
 bool
@@ -1560,74 +1422,32 @@ bool
 js::IteratorMore(JSContext* cx, HandleObject iterobj, MutableHandleValue rval)
 {
     // Fast path for native iterators.
-    if (iterobj->is<PropertyIteratorObject>()) {
+    if (MOZ_LIKELY(iterobj->is<PropertyIteratorObject>())) {
         NativeIterator* ni = iterobj->as<PropertyIteratorObject>().getNativeIterator();
-        bool done;
-        if (!NativeIteratorNext(cx, ni, rval, &done))
-            return false;
-
-        if (done)
-            rval.setMagic(JS_NO_ITER_VALUE);
-        return true;
+        return NativeIteratorNext(cx, ni, rval);
     }
 
-    // We're reentering below and can call anything.
-    if (!CheckRecursionLimit(cx))
+    if (JS_IsDeadWrapper(iterobj)) {
+        JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr, JSMSG_DEAD_OBJECT);
         return false;
-
-    // Call the iterator object's .next method.
-    if (!GetProperty(cx, iterobj, iterobj, cx->names().next, rval))
-        return false;
-
-    // Call the .next method.  Fall through to the error-handling cases in the
-    // unlikely event that either one of the fallible operations performed
-    // during the call process fails.
-    FixedInvokeArgs<0> args(cx);
-    RootedValue iterval(cx, ObjectValue(*iterobj));
-    if (!js::Call(cx, rval, iterval, args, rval)) {
-        // Check for StopIteration.
-        if (!cx->isExceptionPending())
-            return false;
-        RootedValue exception(cx);
-        if (!cx->getPendingException(&exception))
-            return false;
-        if (!JS_IsStopIteration(exception))
-            return false;
-
-        cx->clearPendingException();
-        rval.setMagic(JS_NO_ITER_VALUE);
     }
 
-    return true;
+    MOZ_ASSERT(IsWrapper(iterobj));
+
+    RootedObject obj(cx, CheckedUnwrap(iterobj));
+    if (!obj)
+        return false;
+
+    MOZ_RELEASE_ASSERT(obj->is<PropertyIteratorObject>());
+    {
+        AutoCompartment ac(cx, obj);
+        NativeIterator* ni = obj->as<PropertyIteratorObject>().getNativeIterator();
+        if (!NativeIteratorNext(cx, ni, rval))
+            return false;
+    }
+    return cx->compartment()->wrap(cx, rval);
 }
 
-static bool
-stopiter_hasInstance(JSContext* cx, HandleObject obj, MutableHandleValue v, bool* bp)
-{
-    *bp = JS_IsStopIteration(v);
-    return true;
-}
-
-static const ClassOps StopIterationObjectClassOps = {
-    nullptr, /* addProperty */
-    nullptr, /* delProperty */
-    nullptr, /* getProperty */
-    nullptr, /* setProperty */
-    nullptr, /* enumerate */
-    nullptr, /* enumerate */
-    nullptr, /* resolve */
-    nullptr, /* mayResolve */
-    nullptr, /* finalize */
-    nullptr, /* call */
-    stopiter_hasInstance
-};
-
-const Class StopIterationObject::class_ = {
-    "StopIteration",
-    JSCLASS_HAS_CACHED_PROTO(JSProto_StopIteration),
-    &StopIterationObjectClassOps
-};
-
 static const JSFunctionSpec iterator_proto_methods[] = {
     JS_SELF_HOSTED_SYM_FN(iterator, "IteratorIdentity", 0, 0),
     JS_FS_END
@@ -1694,61 +1514,3 @@ GlobalObject::initStringIteratorProto(JSContext* cx, Handle<GlobalObject*> globa
     global->setReservedSlot(STRING_ITERATOR_PROTO, ObjectValue(*proto));
     return true;
 }
-
-JSObject*
-js::InitLegacyIteratorClass(JSContext* cx, HandleObject obj)
-{
-    Handle<GlobalObject*> global = obj.as<GlobalObject>();
-
-    if (global->getPrototype(JSProto_Iterator).isObject())
-        return &global->getPrototype(JSProto_Iterator).toObject();
-
-    RootedObject iteratorProto(cx);
-    iteratorProto = GlobalObject::createBlankPrototype(cx, global,
-                                                       &PropertyIteratorObject::class_);
-    if (!iteratorProto)
-        return nullptr;
-
-    NativeIterator* ni = NativeIterator::allocateIterator(cx, 0, 0);
-    if (!ni)
-        return nullptr;
-
-    iteratorProto->as<PropertyIteratorObject>().setNativeIterator(ni);
-    ni->init(nullptr, nullptr, 0 /* flags */, 0, 0);
-
-    Rooted<JSFunction*> ctor(cx);
-    ctor = GlobalObject::createConstructor(cx, IteratorConstructor, cx->names().Iterator, 2);
-    if (!ctor)
-        return nullptr;
-    if (!LinkConstructorAndPrototype(cx, ctor, iteratorProto))
-        return nullptr;
-    if (!DefinePropertiesAndFunctions(cx, iteratorProto, nullptr, legacy_iterator_methods))
-        return nullptr;
-    if (!GlobalObject::initBuiltinConstructor(cx, global, JSProto_Iterator,
-                                              ctor, iteratorProto))
-    {
-        return nullptr;
-    }
-
-    return &global->getPrototype(JSProto_Iterator).toObject();
-}
-
-JSObject*
-js::InitStopIterationClass(JSContext* cx, HandleObject obj)
-{
-    Handle<GlobalObject*> global = obj.as<GlobalObject>();
-    if (!global->getPrototype(JSProto_StopIteration).isObject()) {
-        RootedObject proto(cx, GlobalObject::createBlankPrototype(cx, global,
-                                                                  &StopIterationObject::class_));
-        if (!proto || !FreezeObject(cx, proto))
-            return nullptr;
-
-        // This should use a non-JSProtoKey'd slot, but this is easier for now.
-        if (!GlobalObject::initBuiltinConstructor(cx, global, JSProto_StopIteration, proto, proto))
-            return nullptr;
-
-        global->setConstructor(JSProto_StopIteration, ObjectValue(*proto));
-    }
-
-    return &global->getPrototype(JSProto_StopIteration).toObject();
-}
