@@ -27,8 +27,9 @@
 
 #include "jsatominlines.h"
 #include "jscompartmentinlines.h"
-#include "jsgcinlines.h"
 
+#include "gc/Marking-inl.h"
+#include "gc/ObjectKind-inl.h"
 #include "vm/ShapedObject-inl.h"
 #include "vm/TypeInference-inl.h"
 
@@ -40,8 +41,6 @@ MaybeConvertUnboxedObjectToNative(JSContext* cx, JSObject* obj)
 {
     if (obj->is<UnboxedPlainObject>())
         return UnboxedPlainObject::convertToNative(cx, obj);
-    if (obj->is<UnboxedArrayObject>())
-        return UnboxedArrayObject::convertToNative(cx, obj);
     return true;
 }
 
@@ -154,6 +153,13 @@ js::NativeObject::updateDictionaryListPointerAfterMinorGC(NativeObject* old)
         shape_->listp = &shape_;
 }
 
+inline void
+js::gc::MakeAccessibleAfterMovingGC(JSObject* obj)
+{
+    if (obj->isNative())
+        obj->as<NativeObject>().updateShapeAfterMovingGC();
+}
+
 /* static */ inline bool
 JSObject::setSingleton(JSContext* cx, js::HandleObject obj)
 {
@@ -450,12 +456,6 @@ JSObject::isBoundFunction() const
     return is<JSFunction>() && as<JSFunction>().isBoundFunction();
 }
 
-inline bool
-JSObject::watched() const
-{
-    return hasAllFlags(js::BaseShape::WATCHED);
-}
-
 inline bool
 JSObject::isDelegate() const
 {
@@ -560,10 +560,21 @@ HasNativeMethodPure(JSObject* obj, PropertyName* name, JSNative native, JSContex
 static MOZ_ALWAYS_INLINE bool
 HasNoToPrimitiveMethodPure(JSObject* obj, JSContext* cx)
 {
-    jsid id = SYMBOL_TO_JSID(cx->wellKnownSymbols().toPrimitive);
+    Symbol* toPrimitive = cx->wellKnownSymbols().toPrimitive;
+    JSObject* holder;
+    if (!MaybeHasInterestingSymbolProperty(cx, obj, toPrimitive, &holder)) {
+#ifdef DEBUG
+        JSObject* pobj;
+        PropertyResult prop;
+        MOZ_ASSERT(LookupPropertyPure(cx, obj, SYMBOL_TO_JSID(toPrimitive), &pobj, &prop));
+        MOZ_ASSERT(!prop);
+#endif
+        return true;
+    }
+
     JSObject* pobj;
     PropertyResult prop;
-    if (!LookupPropertyPure(cx, obj, id, &pobj, &prop))
+    if (!LookupPropertyPure(cx, holder, SYMBOL_TO_JSID(toPrimitive), &pobj, &prop))
         return false;
 
     return !prop;
@@ -832,6 +843,31 @@ IsConstructor(const Value& v)
     return v.isObject() && v.toObject().isConstructor();
 }
 
+MOZ_ALWAYS_INLINE bool
+CreateThis(JSContext* cx, HandleObject callee, JSScript* calleeScript, HandleObject newTarget,
+           NewObjectKind newKind, MutableHandleValue thisv)
+{
+    if (callee->isBoundFunction()) {
+        thisv.setMagic(JS_UNINITIALIZED_LEXICAL);
+        return true;
+    }
+
+    if (calleeScript->isDerivedClassConstructor()) {
+        MOZ_ASSERT(callee->as<JSFunction>().isClassConstructor());
+        thisv.setMagic(JS_UNINITIALIZED_LEXICAL);
+        return true;
+    }
+
+    MOZ_ASSERT(thisv.isMagic(JS_IS_CONSTRUCTING));
+
+    JSObject* obj = CreateThisForFunction(cx, callee, newTarget, newKind);
+    if (!obj)
+        return false;
+
+    thisv.setObject(*obj);
+    return true;
+}
+
 } /* namespace js */
 
 MOZ_ALWAYS_INLINE bool
