@@ -752,17 +752,19 @@ class ScriptSourceObject : public NativeObject
     static const uint32_t RESERVED_SLOTS = 4;
 };
 
-enum GeneratorKind { NotGenerator, LegacyGenerator, StarGenerator };
+enum class GeneratorKind : bool { NotGenerator, Generator };
 enum FunctionAsyncKind { SyncFunction, AsyncFunction };
 
 static inline unsigned
-GeneratorKindAsBits(GeneratorKind generatorKind) {
+GeneratorKindAsBit(GeneratorKind generatorKind)
+{
     return static_cast<unsigned>(generatorKind);
 }
 
 static inline GeneratorKind
-GeneratorKindFromBits(unsigned val) {
-    MOZ_ASSERT(val <= StarGenerator);
+GeneratorKindFromBit(unsigned val)
+{
+    MOZ_ASSERT(val <= unsigned(GeneratorKind::Generator));
     return static_cast<GeneratorKind>(val);
 }
 
@@ -1033,7 +1035,7 @@ class JSScript : public js::gc::TenuredCell
     uint8_t         hasArrayBits:ARRAY_KIND_BITS;
 
     // The GeneratorKind of the script.
-    uint8_t         generatorKindBits_:2;
+    uint8_t         generatorKind_:1;
 
     // 1-bit fields.
 
@@ -1100,10 +1102,6 @@ class JSScript : public js::gc::TenuredCell
     // Lexical check did fail and bail out.
     bool failedLexicalCheck_:1;
 
-    // If the generator was created implicitly via a generator expression,
-    // isGeneratorExp will be true.
-    bool isGeneratorExp_:1;
-
     // Script has an entry in JSCompartment::scriptCountsMap.
     bool hasScriptCounts_:1;
 
@@ -1388,8 +1386,6 @@ class JSScript : public js::gc::TenuredCell
     }
     void setLikelyConstructorWrapper() { isLikelyConstructorWrapper_ = true; }
 
-    bool isGeneratorExp() const { return isGeneratorExp_; }
-
     bool failedBoundsCheck() const {
         return failedBoundsCheck_;
     }
@@ -1443,15 +1439,14 @@ class JSScript : public js::gc::TenuredCell
     }
 
     js::GeneratorKind generatorKind() const {
-        return js::GeneratorKindFromBits(generatorKindBits_);
+        return js::GeneratorKindFromBit(generatorKind_);
     }
-    bool isLegacyGenerator() const { return generatorKind() == js::LegacyGenerator; }
-    bool isStarGenerator() const { return generatorKind() == js::StarGenerator; }
+    bool isGenerator() const { return generatorKind() == js::GeneratorKind::Generator; }
     void setGeneratorKind(js::GeneratorKind kind) {
         // A script only gets its generator kind set as part of initialization,
         // so it can only transition from not being a generator.
-        MOZ_ASSERT(!isStarGenerator() && !isLegacyGenerator());
-        generatorKindBits_ = GeneratorKindAsBits(kind);
+        MOZ_ASSERT(!isGenerator());
+        generatorKind_ = GeneratorKindAsBit(kind);
     }
 
     js::FunctionAsyncKind asyncKind() const {
@@ -1613,7 +1608,7 @@ class JSScript : public js::gc::TenuredCell
 
     bool isRelazifiable() const {
         return (selfHosted() || lazyScript) && !hasInnerFunctions_ && !types_ &&
-               !isStarGenerator() && !isLegacyGenerator() && !isAsync() &&
+               !isGenerator() && !isAsync() &&
                !isDefaultClassConstructor() &&
                !hasBaselineScript() && !hasAnyIonScript() &&
                !doNotRelazify_;
@@ -1714,7 +1709,7 @@ class JSScript : public js::gc::TenuredCell
     bool isTopLevel() { return code() && !functionNonDelazifying(); }
 
     /* Ensure the script has a TypeScript. */
-    inline bool ensureHasTypes(JSContext* cx);
+    inline bool ensureHasTypes(JSContext* cx, js::AutoKeepTypeScripts&);
 
     inline js::TypeScript* types();
 
@@ -1835,7 +1830,7 @@ class JSScript : public js::gc::TenuredCell
     bool hasTrynotes() const     { return hasArray(TRYNOTES); }
     bool hasScopeNotes() const   { return hasArray(SCOPENOTES); }
     bool hasYieldAndAwaitOffsets() const {
-        return isStarGenerator() || isLegacyGenerator() || isAsync();
+        return isGenerator() || isAsync();
     }
 
 #define OFF(fooOff, hasFoo, t)   (fooOff() + (hasFoo() ? sizeof(t) : 0))
@@ -2130,7 +2125,7 @@ class LazyScript : public gc::TenuredCell
 
         uint32_t numInnerFunctions : NumInnerFunctionsBits;
 
-        uint32_t generatorKindBits : 2;
+        uint32_t generatorKind : 1;
 
         // N.B. These are booleans but need to be uint32_t to pack correctly on MSVC.
         // If you add another boolean here, make sure to initialze it in
@@ -2254,19 +2249,15 @@ class LazyScript : public gc::TenuredCell
         return (GCPtrFunction*)&closedOverBindings()[numClosedOverBindings()];
     }
 
-    GeneratorKind generatorKind() const { return GeneratorKindFromBits(p_.generatorKindBits); }
+    GeneratorKind generatorKind() const { return GeneratorKindFromBit(p_.generatorKind); }
 
-    bool isLegacyGenerator() const { return generatorKind() == LegacyGenerator; }
-
-    bool isStarGenerator() const { return generatorKind() == StarGenerator; }
+    bool isGenerator() const { return generatorKind() == GeneratorKind::Generator; }
 
     void setGeneratorKind(GeneratorKind kind) {
         // A script only gets its generator kind set as part of initialization,
         // so it can only transition from NotGenerator.
-        MOZ_ASSERT(!isStarGenerator() && !isLegacyGenerator());
-        // Legacy generators cannot currently be lazy.
-        MOZ_ASSERT(kind != LegacyGenerator);
-        p_.generatorKindBits = GeneratorKindAsBits(kind);
+        MOZ_ASSERT(!isGenerator());
+        p_.generatorKind = GeneratorKindAsBit(kind);
     }
 
     FunctionAsyncKind asyncKind() const {
