@@ -5,7 +5,7 @@
 
 
 /**
-   File Name:          11.2.2-3-n.js
+   File Name:          11.2.2-5-n.js
    ECMA Section:       11.2.2. The new operator
    Description:
 
@@ -47,17 +47,21 @@
    Date:               12 november 1997
 */
 
-var SECTION = "11.2.2-3-n.js";
+var SECTION = "11.2.2-5-n.js";
 var TITLE   = "The new operator";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-var DESCRIPTION = "NULL = null; var o = new NULL()";
-var NULL = null;
+var NUMBER = 0;
+
+DESCRIPTION = "NUMBER=0, var n = new NUMBER()";
 
 new TestCase(
-	      "NULL = null; var o = new NULL()",
+	      "NUMBER=0, var n = new NUMBER()",
 	      "error",
-	      eval("o = new NULL()") );
+	      eval("n = new NUMBER()") );
 test();
 
+function TestFunction() {
+  return arguments;
+}
