@@ -4,12 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 351102;
+var summary = 'try/catch-guard/finally GC issues';
 var actual = '';
 var expect = '';
 
-//-------  Comment #196  From  Gary Kwong
 
 //-----------------------------------------------------------------------------
 test();
@@ -17,28 +16,26 @@ test();
 
 function test()
 {
-  enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  var f;
+  f = function()
+    {
+      try
+      {
+        d.d.d;
+      }
+      catch({} if gc())
+      {
+      }
+      catch(y)
+      {
+      }
+    };
 
-// Assertion failure: localKind == JSLOCAL_VAR || localKind == JSLOCAL_CONST, at ../jsfun.cpp:916
+  f();
+  f();
 
-  this.x = undefined;
-  this.watch("x", Function);
-  NaN = uneval({ get \u3056 (){ return undefined } });
-  x+=NaN;
-
-  reportCompare(expect, actual, summary + ': 1');
-
-// Assertion failure: lexdep->isLet(), at ../jsparse.cpp:1900
-
-  (function (){
-    var x;
-    eval("var x; (function () { return x; })");
-  }
-    )();
-
-  reportCompare(expect, actual, summary + ': 2');
-
-  exitFunc ('test');
+  reportCompare(expect, actual, summary + ': 3');
 }
