@@ -1,13 +1,11 @@
 /* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/*
- * Any copyright is dedicated to the Public Domain.
- * http://creativecommons.org/licenses/publicdomain/
- * Contributor: Jason Orendorff
- */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 465460;
+var summary = 'TM: valueOf in a loop: do not assert';
 var actual = '';
 var expect = '';
 
@@ -18,17 +16,12 @@ test();
 
 function test()
 {
-  enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
 
-// ------- Comment #79 From Jason Orendorff
+  for each (let x in [1, {}, 1, null, 1, {}, 1, null, 1]) { }
 
-  x; var x; function x() { return 0; }
-
-// Assertion failure: !(pn->pn_dflags & flag), at ../jsparse.h:635
 
   reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
 }
