@@ -1,6 +1,7 @@
 // Copyright (C) 2015 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 /*---
+esid: sec-array.prototype.copywithin
 es6id: 22.1.3.3
 description: >
   Max value of end position is the this.length.
@@ -41,6 +42,14 @@ assert(
   '[0, 1, 2, 3].copyWithin(0, 1, 6) -> [1, 2, 3, 3]'
 );
 
+assert(
+  compareArray(
+    [0, 1, 2, 3].copyWithin(0, 1, Infinity),
+    [1, 2, 3, 3]
+  ),
+  '[0, 1, 2, 3].copyWithin(0, 1, Infinity) -> [1, 2, 3, 3]'
+);
+
 assert(
   compareArray(
     [0, 1, 2, 3, 4, 5].copyWithin(1, 3, 6),
@@ -48,4 +57,13 @@ assert(
   ),
   '[0, 1, 2, 3, 4, 5].copyWithin(1, 3, 6) -> [0, 3, 4, 5, 4, 5]'
 );
+
+assert(
+  compareArray(
+    [0, 1, 2, 3, 4, 5].copyWithin(1, 3, Infinity),
+    [0, 3, 4, 5, 4, 5]
+  ),
+  '[0, 1, 2, 3, 4, 5].copyWithin(1, 3, Infinity) -> [0, 3, 4, 5, 4, 5]'
+);
+
 reportCompare(0, 0);
