@@ -5,50 +5,71 @@
 info: >
     The [[Class]] property of the newly constructed object
     is set to "Date"
-esid: sec-date-value
-es5id: 15.9.3.2_A3_T1.1
-description: Test based on delete prototype.toString
-includes: [dateConstants.js]
+esid: sec-date-year-month-date-hours-minutes-seconds-ms
+es5id: 15.9.3.1_A3_T2.1
+description: >
+    Test based on delete prototype.toString - 3 arguments, (year,
+    month, date)
 ---*/
 
-var x1 = new Date(date_1899_end);
+var x1 = new Date(1899, 11, 31);
 if (Object.prototype.toString.call(x1) !== "[object Date]") {
   $ERROR("#1: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x2 = new Date(date_1900_start);
+var x2 = new Date(1899, 12, 1);
 if (Object.prototype.toString.call(x2) !== "[object Date]") {
   $ERROR("#2: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x3 = new Date(date_1969_end);
+var x3 = new Date(1900, 0, 1);
 if (Object.prototype.toString.call(x3) !== "[object Date]") {
   $ERROR("#3: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x4 = new Date(date_1970_start);
+var x4 = new Date(1969, 11, 31);
 if (Object.prototype.toString.call(x4) !== "[object Date]") {
   $ERROR("#4: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x5 = new Date(date_1999_end);
+var x5 = new Date(1969, 12, 1);
 if (Object.prototype.toString.call(x5) !== "[object Date]") {
   $ERROR("#5: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x6 = new Date(date_2000_start);
+var x6 = new Date(1970, 0, 1);
 if (Object.prototype.toString.call(x6) !== "[object Date]") {
   $ERROR("#6: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x7 = new Date(date_2099_end);
+var x7 = new Date(1999, 11, 31);
 if (Object.prototype.toString.call(x7) !== "[object Date]") {
   $ERROR("#7: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
-var x8 = new Date(date_2100_start);
+var x8 = new Date(1999, 12, 1);
 if (Object.prototype.toString.call(x8) !== "[object Date]") {
   $ERROR("#8: The [[Class]] property of the newly constructed object is set to 'Date'");
 }
 
+var x9 = new Date(2000, 0, 1);
+if (Object.prototype.toString.call(x9) !== "[object Date]") {
+  $ERROR("#9: The [[Class]] property of the newly constructed object is set to 'Date'");
+}
+
+var x10 = new Date(2099, 11, 31);
+if (Object.prototype.toString.call(x10) !== "[object Date]") {
+  $ERROR("#10: The [[Class]] property of the newly constructed object is set to 'Date'");
+}
+
+var x11 = new Date(2099, 12, 1);
+if (Object.prototype.toString.call(x11) !== "[object Date]") {
+  $ERROR("#11: The [[Class]] property of the newly constructed object is set to 'Date'");
+}
+
+var x12 = new Date(2100, 0, 1);
+if (Object.prototype.toString.call(x12) !== "[object Date]") {
+  $ERROR("#12: The [[Class]] property of the newly constructed object is set to 'Date'");
+}
+
 reportCompare(0, 0);
