@@ -5,26 +5,24 @@
 info: >
     PerformPromiseThen
     Ref 25.4.5.3.1
-es6id: S25.4.5.3_A5.3_T1
+es6id: S25.4.5.3_A5.2_T1
 author: Sam Mikes
-description: Promise.prototype.then immediately queues handler if rejected
+description: Promise.prototype.then immediately queues handler if fulfilled
 includes: [promiseHelper.js]
 flags: [async]
 ---*/
 
 var sequence = [],
-    pReject,
+    pResolve,
     p = new Promise(function (resolve, reject) {
-        pReject = reject;
+        pResolve = resolve;
     });
 
 sequence.push(1);
 
-pReject();
+pResolve();
 
 p.then(function () {
-    $ERROR("Should not be called -- Promise rejected.");
-}, function () {
     sequence.push(3);
     checkSequence(sequence, "Should be first");
 }).catch($DONE);
@@ -32,8 +30,6 @@ p.then(function () {
 Promise.resolve().then(function () {
     // enqueue another then-handler
     p.then(function () {
-        $ERROR("Should not be called (2) -- Promise rejected.");
-    }, function () {
         sequence.push(5);
         checkSequence(sequence, "Should be third");
     }).then($DONE, $DONE);
