@@ -2,6 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
+esid: sec-get-regexp.prototype.source
 es6id: 21.2.5.10
 description: >
   get RegExp.prototype.source.length is 0.
@@ -22,12 +23,13 @@ info: >
 includes: [propertyHelper.js]
 ---*/
 
-var desc = Object.getOwnPropertyDescriptor(RegExp.prototype, "source");
+var get = Object.getOwnPropertyDescriptor(RegExp.prototype, 'source').get;
 
-assert.sameValue(desc.get.length, 0);
-
-verifyNotEnumerable(desc.get, "length");
-verifyNotWritable(desc.get, "length");
-verifyConfigurable(desc.get, "length");
+verifyProperty(get, 'length', {
+  value: 0,
+  writable: false,
+  enumerable: false,
+  configurable: true,
+});
 
 reportCompare(0, 0);
