@@ -5,16 +5,14 @@
 info: >
     15.5.5.2 defines [[GetOwnProperty]] for Strings. It supports using indexing
     notation to look up non numeric property names.
-es5id: 15.5.5.5.2-1-1
+es5id: 15.5.5.5.2-7-3
 description: >
-    String object supports bracket notation to lookup of data
-    properties
+    String object indexing returns undefined if the numeric index is
+    greater than the string length
 ---*/
 
   var s = new String("hello world");
-  s.foo = 1;
-  
 
-assert.sameValue(s["foo"], 1, 's["foo"]');
+assert.sameValue(s[11], undefined, 's[11]');
 
 reportCompare(0, 0);
