@@ -10,7 +10,7 @@ info: >
   1. Let O be RequireObjectCoercible(this value).
   2. Let S be ToString(O).
   3. ReturnIfAbrupt(S).
-features: [Symbol]
+features: [Symbol, String.prototype.includes]
 ---*/
 
 var s = Symbol('');
