@@ -24,10 +24,10 @@ info: >
   5. If IsDetachedBuffer(buffer) is true, throw a TypeError exception.
   ...
 includes: [testTypedArray.js, detachArrayBuffer.js]
-features: [Reflect]
+features: [cross-realm, Reflect, TypedArray]
 ---*/
 
-var other = $.createRealm().global;
+var other = $262.createRealm().global;
 var desc = {
   value: 0,
   configurable: false,
