@@ -3,9 +3,9 @@
 
 /*---
 info: >
-    If string.charAt(k) in [0x0080 - 0x07FF], return 2 octets (00000yyy
-    yyzzzzzz -> 110yyyyy 10zzzzzz)
-es5id: 15.1.3.3_A2.2_T1
+    If string.charAt(k) in [0xE000 - 0xFFFF], return 3 octets (xxxxyyyy
+    yyzzzzzz -> 1110xxxx 10yyyyyy 10zzzzzz)
+es5id: 15.1.3.3_A2.5_T1
 es6id: 18.2.6.4
 esid: sec-encodeuri-uri
 description: Complex tests, use RFC 3629
@@ -16,13 +16,13 @@ var errorCount = 0;
 var count = 0;
 var indexP;
 var indexO = 0;
-l:
-for (var index = 0x0080; index <= 0x07FF; index++) {
+for (var index = 0xE000; index <= 0xFFFF; index++) {
   count++;
   var hex1 = decimalToPercentHexString(0x0080 + (index & 0x003F));
-  var hex2 = decimalToPercentHexString(0x00C0 + (index & 0x07C0) / 0x0040);
+  var hex2 = decimalToPercentHexString(0x0080 + (index & 0x0FC0) / 0x0040);
+  var hex3 = decimalToPercentHexString(0x00E0 + (index & 0xF000) / 0x1000);
   var str = String.fromCharCode(index);
-  if (encodeURI(str).toUpperCase() === hex2 + hex1) continue;
+  if (encodeURI(str).toUpperCase() === hex3 + hex2 + hex1) continue;
 
   if (indexO === 0) {
     indexO = index;
